@@ -3,6 +3,8 @@
 # This file is made to be sourced into other test scripts and not executed
 # manually because it sets trap to restore pcscd to working state
 
+# Set PCSCD_DEBUG="-d -a" to debug APDUs before sourcing this file
+
 # register cleanup handler
 function pcscd_cleanup {
 	echo "Process terminated: resetting pcscd"
@@ -20,13 +22,14 @@ if which systemctl && systemctl is-system-running; then
 else
 	sudo pkill pcscd || echo "no pcscd process was running"
 fi
-sudo /usr/sbin/pcscd -f 2>&1 | sed -e 's/^/pcscd: /;' &
+sudo /usr/sbin/pcscd -f $PCSCD_DEBUG 2>&1 | sed -e 's/^/pcscd: /' &
 
 
 # Try to wait up to 30 seconds for pcscd to come up and create PID file
 for ((i=1;i<=30;i++)); do
 	echo "Waiting for pcscd to start: $i s"
 	if [ -f "/var/run/pcscd/pcscd.pid" ]; then
+		echo "PCSC PID: `cat /var/run/pcscd/pcscd.pid`"
 		break
 	fi
 	sleep 1
