@@ -15,10 +15,10 @@
  *
  * You should have received a copy of the GNU Lesser General Public
  * License along with this library; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
  */
 
-#if HAVE_CONFIG_H
+#ifdef HAVE_CONFIG_H
 #include "config.h"
 #endif
 
@@ -322,13 +322,13 @@ sc_check_apdu(sc_card_t *card, const sc_apdu_t *apdu)
 error:
 	sc_log(card->ctx, "Invalid Case %d %s APDU:\n"
 		"cse=%02x cla=%02x ins=%02x p1=%02x p2=%02x lc=%lu le=%lu\n"
-		"resp=%p resplen=%lu data=%p datalen=%lu",
+		"resp=%p resplen=%lu data=%p datalen=%lu flags=0x%8.8lx",
 		apdu->cse & SC_APDU_SHORT_MASK,
 		(apdu->cse & SC_APDU_EXT) != 0 ? "extended" : "short",
 		apdu->cse, apdu->cla, apdu->ins, apdu->p1, apdu->p2,
 		(unsigned long) apdu->lc, (unsigned long) apdu->le,
 		apdu->resp, (unsigned long) apdu->resplen,
-		apdu->data, (unsigned long) apdu->datalen);
+		apdu->data, (unsigned long) apdu->datalen, apdu->flags);
 	return SC_ERROR_INVALID_ARGUMENTS;
 }
 
@@ -402,8 +402,8 @@ sc_set_le_and_transmit(struct sc_card *card, struct sc_apdu *apdu, size_t olen)
 	apdu->resplen = olen;
 	apdu->le      = nlen;
 #ifndef FUZZING_BUILD_MODE_UNSAFE_FOR_PRODUCTION
-	/* Belpic V1 applets have a problem: if the card sends a 6C XX (only XX bytes available), 
-	 * and we resend the command too soon (i.e. the reader is too fast), the card doesn't respond. 
+	/* Belpic V1 applets have a problem: if the card sends a 6C XX (only XX bytes available),
+	 * and we resend the command too soon (i.e. the reader is too fast), the card doesn't respond.
 	 * So we build in a delay. */
 	if (card->type == SC_CARD_TYPE_BELPIC_EID)
 		msleep(40);
@@ -563,6 +563,14 @@ int sc_transmit_apdu(sc_card_t *card, sc_apdu_t *apdu)
 		return r;
 	}
 
+#if ENABLE_SM
+	if (card->sm_ctx.sm_mode == SM_MODE_TRANSMIT
+			&& (apdu->flags & SC_APDU_FLAGS_CHAINING) != 0
+			&& (apdu->flags & SC_APDU_FLAGS_SM_CHAINING) != 0) {
+		sc_log(card->ctx,"Let SM do the chaining");
+		r = sc_transmit(card, apdu);
+	} else
+#endif
 	if ((apdu->flags & SC_APDU_FLAGS_CHAINING) != 0) {
 		/* divide et impera: transmit APDU in chunks with Lc <= max_send_size
 		 * bytes using command chaining */
@@ -587,9 +595,8 @@ int sc_transmit_apdu(sc_card_t *card, sc_apdu_t *apdu)
 				 *      secure messaging is used */
 				plen          = max_send_size;
 				tapdu.cla    |= 0x10;
+				/* the intermediate APDU don't expect response data */
 				tapdu.le      = 0;
-				/* the intermediate APDU don't expect data */
-				tapdu.lc      = 0;
 				tapdu.resplen = 0;
 				tapdu.resp    = NULL;
 			} else {
