@@ -15,10 +15,10 @@
  *
  * You should have received a copy of the GNU Lesser General Public
  * License along with this library; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
  */
 
-#if HAVE_CONFIG_H
+#ifdef HAVE_CONFIG_H
 #include "config.h"
 #endif
 
@@ -68,7 +68,7 @@ int sc_asn1_read_tag(const u8 ** buf, size_t buflen, unsigned int *cla_out,
 
 	*buf = NULL;
 
-	if (left == 0 || !p)
+	if (left == 0 || !p || buflen == 0)
 		return SC_ERROR_INVALID_ASN1_OBJECT;
 	if (*p == 0xff || *p == 0) {
 		/* end of data reached */
@@ -83,6 +83,8 @@ int sc_asn1_read_tag(const u8 ** buf, size_t buflen, unsigned int *cla_out,
 	 */
 	cla = (*p & SC_ASN1_TAG_CLASS) | (*p & SC_ASN1_TAG_CONSTRUCTED);
 	tag = *p & SC_ASN1_TAG_PRIMITIVE;
+	if (left < 1)
+		return SC_ERROR_INVALID_ASN1_OBJECT;
 	p++;
 	left--;
 	if (tag == SC_ASN1_TAG_PRIMITIVE) {
@@ -374,7 +376,8 @@ static void print_tags_recursive(const u8 * buf0, const u8 * buf,
 	const u8 *p = buf;
 
 	while (bytesleft >= 2) {
-		unsigned int cla = 0, tag = 0, hlen;
+		unsigned int cla = 0, tag = 0;
+		size_t hlen;
 		const u8 *tagp = p;
 		size_t len;
 
@@ -649,8 +652,7 @@ static int encode_bit_string(const u8 * inbuf, size_t bits_left, u8 **outbuf,
 {
 	const u8 *in = inbuf;
 	u8 *out;
-	size_t bytes;
-	int skipped = 0;
+	size_t bytes, skipped = 0;
 
 	bytes = (bits_left + 7)/8 + 1;
 	*outbuf = out = malloc(bytes);
@@ -659,7 +661,7 @@ static int encode_bit_string(const u8 * inbuf, size_t bits_left, u8 **outbuf,
 	*outlen = bytes;
 	out += 1;
 	while (bits_left) {
-		int i, bits_to_go = 8;
+		size_t i, bits_to_go = 8;
 
 		*out = 0;
 		if (bits_left < 8) {
@@ -978,8 +980,7 @@ static int sc_asn1_decode_utf8string(const u8 *inbuf, size_t inlen,
 int sc_asn1_put_tag(unsigned int tag, const u8 * data, size_t datalen, u8 * out, size_t outlen, u8 **ptr)
 {
 	size_t c = 0;
-	size_t tag_len;
-	size_t ii;
+	unsigned int tag_len, ii;
 	u8 *p = out;
 	u8 tag_char[4] = {0, 0, 0, 0};
 
@@ -1019,7 +1020,7 @@ int sc_asn1_put_tag(unsigned int tag, const u8 * data, size_t datalen, u8 * out,
 	}
 	if (outlen == 0 || out == NULL) {
 		/* Caller only asks for the length that would be written. */
-		return tag_len + (c+1) + datalen;
+		return (int)(tag_len + (c + 1) + datalen);
 	}
 	/* We will write the tag, so check the length. */
 	if (outlen < tag_len + (c+1) + datalen)
@@ -2098,6 +2099,8 @@ sc_der_copy(sc_pkcs15_der_t *dst, const sc_pkcs15_der_t *src)
 		return SC_ERROR_INVALID_ARGUMENTS;
 	memset(dst, 0, sizeof(*dst));
 	if (src->len) {
+		if (!src->value)
+			return SC_ERROR_INVALID_ARGUMENTS;
 		dst->value = malloc(src->len);
 		if (!dst->value)
 			return SC_ERROR_OUT_OF_MEMORY;
@@ -2270,7 +2273,7 @@ int sc_asn1_decode_ecdsa_signature(sc_context_t *ctx, const u8 *data, size_t dat
 		memcpy(*out + fieldsize * i + fieldsize - intlen , pint, intlen);
 		pint += intlen; /* next integer */
 	}
-	r = 2 * fieldsize;
+	r = (int)(2 * fieldsize);
 err:
 	LOG_FUNC_RETURN(ctx, r);
 }
