@@ -15,7 +15,7 @@
  *
  * You should have received a copy of the GNU Lesser General Public
  * License along with this library; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
  */
 
 #ifndef _OPENSC_ASN1_H
@@ -58,7 +58,7 @@ void sc_format_asn1_entry(struct sc_asn1_entry *entry, void *parm, void *arg,
 			  int set_present);
 void sc_copy_asn1_entry(const struct sc_asn1_entry *src,
 			struct sc_asn1_entry *dest);
-			
+
 /* DER tag and length parsing */
 int sc_asn1_decode(struct sc_context *ctx, struct sc_asn1_entry *asn1,
 		   const u8 *in, size_t len, const u8 **newp, size_t *left);
@@ -84,8 +84,10 @@ const u8 *sc_asn1_skip_tag(struct sc_context *ctx, const u8 ** buf,
 /* DER encoding */
 
 /* Argument 'ptr' is set to the location of the next possible ASN.1 object.
- * If NULL, no action on 'ptr' is performed. */
-int sc_asn1_put_tag(int tag, const u8 * data, size_t datalen, u8 * out, size_t outlen, u8 ** ptr);
+ * If NULL, no action on 'ptr' is performed.
+ * If out is NULL or outlen is zero, the length that would be written is returned.
+ * If data is NULL, the data field will not be written. This is helpful for constructed structures. */
+int sc_asn1_put_tag(unsigned int tag, const u8 * data, size_t datalen, u8 * out, size_t outlen, u8 ** ptr);
 
 /* ASN.1 printing functions */
 void sc_asn1_print_tags(const u8 * buf, size_t buflen);
@@ -94,14 +96,14 @@ void sc_asn1_print_tags(const u8 * buf, size_t buflen);
 int sc_asn1_utf8string_to_ascii(const u8 * buf, size_t buflen,
 				u8 * outbuf, size_t outlen);
 int sc_asn1_decode_bit_string(const u8 * inbuf, size_t inlen,
-			      void *outbuf, size_t outlen);
+			      void *outbuf, size_t outlen, const int strict);
 /* non-inverting version */
 int sc_asn1_decode_bit_string_ni(const u8 * inbuf, size_t inlen,
-				 void *outbuf, size_t outlen);
-int sc_asn1_decode_integer(const u8 * inbuf, size_t inlen, int *out);
+				 void *outbuf, size_t outlen, const int strict);
+int sc_asn1_decode_integer(const u8 * inbuf, size_t inlen, int *out, int strict);
 int sc_asn1_decode_object_id(const u8 * inbuf, size_t inlen,
 			     struct sc_object_id *id);
-int sc_asn1_encode_object_id(u8 **buf, size_t *buflen, 
+int sc_asn1_encode_object_id(u8 **buf, size_t *buflen,
 				const struct sc_object_id *id);
 
 /* algorithm encoding/decoding */
@@ -118,14 +120,29 @@ void sc_asn1_clear_algorithm_id(struct sc_algorithm_id *);
 int sc_asn1_write_element(sc_context_t *ctx, unsigned int tag,
 		const u8 * data, size_t datalen, u8 ** out, size_t * outlen);
 
-#define SC_ASN1_CLASS_MASK		0x30000000
-#define SC_ASN1_UNI			0x00000000 /* Universal */
-#define SC_ASN1_APP			0x10000000 /* Application */
-#define SC_ASN1_CTX			0x20000000 /* Context */
-#define SC_ASN1_PRV			0x30000000 /* Private */
-#define SC_ASN1_CONS			0x01000000
+int sc_asn1_sig_value_rs_to_sequence(struct sc_context *ctx,
+		unsigned char *in, size_t inlen,
+                unsigned char **buf, size_t *buflen);
+int sc_asn1_sig_value_sequence_to_rs(struct sc_context *ctx,
+		const unsigned char *in, size_t inlen,
+                unsigned char *buf, size_t buflen);
 
+/* ECDSA signature decoding*/
+int sc_asn1_decode_ecdsa_signature(sc_context_t *ctx, const u8 *data, size_t datalen,
+		size_t fieldsize, u8 **out, size_t outlen);
+
+/* long form tags use these */
+/* Same as  SC_ASN1_TAG_* shifted left by 24 bits  */
+#define SC_ASN1_CLASS_MASK		0xC0000000
+#define SC_ASN1_UNI			0x00000000 /* Universal */
+#define SC_ASN1_APP			0x40000000 /* Application */
+#define SC_ASN1_CTX			0x80000000 /* Context */
+#define SC_ASN1_PRV			0xC0000000 /* Private */
+#define SC_ASN1_CONS			0x20000000
+
+#define SC_ASN1_CLASS_CONS		0xE0000000 /* CLASS and CONS */
 #define SC_ASN1_TAG_MASK		0x00FFFFFF
+#define SC_ASN1_TAGNUM_SIZE		3
 
 #define SC_ASN1_PRESENT			0x00000001
 #define SC_ASN1_OPTIONAL		0x00000002
@@ -163,6 +180,7 @@ int sc_asn1_write_element(sc_context_t *ctx, unsigned int tag,
 /* use callback function */
 #define SC_ASN1_CALLBACK		384
 
+/* use with short one byte tags */
 #define SC_ASN1_TAG_CLASS		0xC0
 #define SC_ASN1_TAG_UNIVERSAL		0x00
 #define SC_ASN1_TAG_APPLICATION		0x40
@@ -171,6 +189,7 @@ int sc_asn1_write_element(sc_context_t *ctx, unsigned int tag,
 
 #define SC_ASN1_TAG_CONSTRUCTED		0x20
 #define SC_ASN1_TAG_PRIMITIVE		0x1F
+#define SC_ASN1_TAG_CLASS_CONS		0xE0
 
 #define SC_ASN1_TAG_EOC			0
 #define SC_ASN1_TAG_BOOLEAN		1
