@@ -23,10 +23,10 @@
  *
  * You should have received a copy of the GNU Lesser General Public
  * License along with this library; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
  */
 
-#if HAVE_CONFIG_H
+#ifdef HAVE_CONFIG_H
 #include "config.h"
 #endif
 
@@ -49,9 +49,6 @@
 #include "internal.h"
 #include "simpletlv.h"
 #include "cardctl.h"
-#ifdef ENABLE_ZLIB
-#include "compression.h"
-#endif
 #include "iso7816.h"
 #include "card-cac-common.h"
 #include "pkcs15.h"
@@ -157,7 +154,7 @@ typedef struct {
 } cac_properties_object_t;
 
 typedef struct {
-	unsigned int num_objects;
+	size_t num_objects;
 	cac_properties_object_t objects[CAC_MAX_OBJECTS];
 } cac_properties_t;
 
@@ -255,7 +252,7 @@ static int cac_apdu_io(sc_card_t *card, int ins, int p1, int p2,
 	size_t * recvbuflen)
 {
 	int r;
-	sc_apdu_t apdu;
+	sc_apdu_t apdu = {0};
 	u8 rbufinitbuf[CAC_MAX_SIZE];
 	u8 *rbuf;
 	size_t rbuflen;
@@ -264,7 +261,7 @@ static int cac_apdu_io(sc_card_t *card, int ins, int p1, int p2,
 
 	SC_FUNC_CALLED(card->ctx, SC_LOG_DEBUG_VERBOSE);
 
-	sc_log(card->ctx, 
+	sc_log(card->ctx,
 		 "%02x %02x %02x %"SC_FORMAT_LEN_SIZE_T"u : %"SC_FORMAT_LEN_SIZE_T"u %"SC_FORMAT_LEN_SIZE_T"u\n",
 		 ins, p1, p2, sendbuflen, card->max_send_size,
 		 card->max_recv_size);
@@ -303,14 +300,14 @@ static int cac_apdu_io(sc_card_t *card, int ins, int p1, int p2,
 		 apdu.resplen = 0;
 	}
 
-	sc_log(card->ctx, 
+	sc_log(card->ctx,
 		 "calling sc_transmit_apdu flags=%lx le=%"SC_FORMAT_LEN_SIZE_T"u, resplen=%"SC_FORMAT_LEN_SIZE_T"u, resp=%p",
 		 apdu.flags, apdu.le, apdu.resplen, apdu.resp);
 
 	/* with new adpu.c and chaining, this actually reads the whole object */
 	r = sc_transmit_apdu(card, &apdu);
 
-	sc_log(card->ctx, 
+	sc_log(card->ctx,
 		 "result r=%d apdu.resplen=%"SC_FORMAT_LEN_SIZE_T"u sw1=%02x sw2=%02x",
 		 r, apdu.resplen, apdu.sw1, apdu.sw2);
 	if (r < 0) {
@@ -335,7 +332,7 @@ static int cac_apdu_io(sc_card_t *card, int ins, int p1, int p2,
 			memcpy(*recvbuf, rbuf, apdu.resplen);
 		}
 		*recvbuflen =  apdu.resplen;
-		r = *recvbuflen;
+		r = (int)*recvbuflen;
 	}
 
 err:
@@ -392,13 +389,13 @@ fail:
 static int cac_read_file(sc_card_t *card, int file_type, u8 **out_buf, size_t *out_len)
 {
 	u8 params[2];
-	u8 count[2];
+	u8 count[2] = {0};
 	u8 *out = NULL;
-	u8 *out_ptr;
+	u8 *out_ptr = NULL;
 	size_t offset = 0;
 	size_t size = 0;
 	size_t left = 0;
-	size_t len;
+	size_t len = 0;
 	int r;
 
 	params[0] = file_type;
@@ -453,7 +450,7 @@ fail:
  * as well as set that we want the cert from the object.
  */
 static int cac_read_binary(sc_card_t *card, unsigned int idx,
-		unsigned char *buf, size_t count, unsigned long flags)
+		unsigned char *buf, size_t count, unsigned long *flags)
 {
 	cac_private_data_t * priv = CAC_DATA(card);
 	int r = 0;
@@ -461,7 +458,7 @@ static int cac_read_binary(sc_card_t *card, unsigned int idx,
 	const u8 *tl_ptr, *val_ptr, *tl_start;
 	u8 *tlv_ptr;
 	const u8 *cert_ptr;
-	size_t tl_len, val_len, tlv_len;
+	size_t tl_len = 0, val_len = 0, tlv_len;
 	size_t len, tl_head_len, cert_len;
 	u8 cert_type, tag;
 
@@ -469,7 +466,7 @@ static int cac_read_binary(sc_card_t *card, unsigned int idx,
 
 	/* if we didn't return it all last time, return the remainder */
 	if (priv->cached) {
-		sc_log(card->ctx, 
+		sc_log(card->ctx,
 			 "returning cached value idx=%d count=%"SC_FORMAT_LEN_SIZE_T"u",
 			 idx, count);
 		if (idx > priv->cache_buf_len) {
@@ -477,10 +474,10 @@ static int cac_read_binary(sc_card_t *card, unsigned int idx,
 		}
 		len = MIN(count, priv->cache_buf_len-idx);
 		memcpy(buf, &priv->cache_buf[idx], len);
-		LOG_FUNC_RETURN(card->ctx, len);
+		LOG_FUNC_RETURN(card->ctx, (int)len);
 	}
 
-	sc_log(card->ctx, 
+	sc_log(card->ctx,
 		 "clearing cache idx=%d count=%"SC_FORMAT_LEN_SIZE_T"u",
 		 idx, count);
 	if (priv->cache_buf) {
@@ -541,7 +538,7 @@ static int cac_read_binary(sc_card_t *card, unsigned int idx,
 
 	case CAC_OBJECT_TYPE_CERT:
 		/* read file */
-		sc_log(card->ctx, 
+		sc_log(card->ctx,
 			 " obj= cert_file, val_len=%"SC_FORMAT_LEN_SIZE_T"u (0x%04"SC_FORMAT_LEN_SIZE_T"x)",
 			 val_len, val_len);
 		cert_len = 0;
@@ -576,17 +573,10 @@ static int cac_read_binary(sc_card_t *card, unsigned int idx,
 			}
 		}
 		/* if the info byte is 1, then the cert is compressed, decompress it */
-		if ((cert_type & 0x3) == 1) {
-#ifdef ENABLE_ZLIB
-			r = sc_decompress_alloc(&priv->cache_buf, &priv->cache_buf_len,
-				cert_ptr, cert_len, COMPRESSION_AUTO);
-#else
-			sc_log(card->ctx, "CAC compression not supported, no zlib");
-			r = SC_ERROR_NOT_SUPPORTED;
-#endif
-			if (r)
-				goto done;
-		} else if (cert_len > 0) {
+		if ((cert_type & 0x3) == 1 && flags) {
+			*flags |= SC_FILE_FLAG_COMPRESSED_AUTO;
+		}
+		if (cert_len > 0) {
 			priv->cache_buf = malloc(cert_len);
 			if (priv->cache_buf == NULL) {
 				r = SC_ERROR_OUT_OF_MEMORY;
@@ -615,7 +605,7 @@ static int cac_read_binary(sc_card_t *card, unsigned int idx,
 	priv->cached = 1;
 	len = MIN(count, priv->cache_buf_len-idx);
 	memcpy(buf, &priv->cache_buf[idx], len);
-	r = len;
+	r = (int)len;
 done:
 	if (tl)
 		free(tl);
@@ -745,8 +735,8 @@ static int cac_set_security_env(sc_card_t *card, const sc_security_env_t *env, i
 
 	SC_FUNC_CALLED(card->ctx, SC_LOG_DEBUG_VERBOSE);
 
-	sc_log(card->ctx, 
-		 "flags=%08lx op=%d alg=%d algf=%08x algr=%08x kr0=%02x, krfl=%"SC_FORMAT_LEN_SIZE_T"u\n",
+	sc_log(card->ctx,
+		 "flags=%08lx op=%d alg=%lu algf=%08lx algr=%08lx kr0=%02x, krfl=%"SC_FORMAT_LEN_SIZE_T"u\n",
 		 env->flags, env->operation, env->algorithm,
 		 env->algorithm_flags, env->algorithm_ref, env->key_ref[0],
 		 env->key_ref_len);
@@ -777,7 +767,7 @@ static int cac_rsa_op(sc_card_t *card,
 	size_t rbuflen, outplen;
 
 	SC_FUNC_CALLED(card->ctx, SC_LOG_DEBUG_VERBOSE);
-	sc_log(card->ctx, 
+	sc_log(card->ctx,
 		 "datalen=%"SC_FORMAT_LEN_SIZE_T"u outlen=%"SC_FORMAT_LEN_SIZE_T"u\n",
 		 datalen, outlen);
 
@@ -802,8 +792,8 @@ static int cac_rsa_op(sc_card_t *card,
 			break;
 		}
 		if (rbuflen != 0) {
-			int n = MIN(rbuflen, outplen);
-			memcpy(outp,rbuf, n);
+			size_t n = MIN(rbuflen, outplen);
+			memcpy(outp, rbuf, n);
 			outp += n;
 			outplen -= n;
 		}
@@ -821,14 +811,14 @@ static int cac_rsa_op(sc_card_t *card,
 		goto err;
 	}
 	if (rbuflen != 0) {
-		int n = MIN(rbuflen, outplen);
-		memcpy(outp,rbuf, n);
+		size_t n = MIN(rbuflen, outplen);
+		memcpy(outp, rbuf, n);
 		/*outp += n;     unused */
 		outplen -= n;
 	}
 	free(rbuf);
 	rbuf = NULL;
-	r = outlen-outplen;
+	r = (int)(outlen - outplen);
 
 err:
 	sc_unlock(card);
@@ -1035,7 +1025,7 @@ static int cac_get_properties(sc_card_t *card, cac_properties_t *prop)
 	free(rbuf);
 	/* sanity */
 	if (i != prop->num_objects)
-		sc_log(card->ctx, "The announced number of objects (%u) "
+		sc_log(card->ctx, "The announced number of objects (%zu) "
 		    "did not match reality (%"SC_FORMAT_LEN_SIZE_T"u)",
 		    prop->num_objects, i);
 	prop->num_objects = i;
@@ -1057,7 +1047,8 @@ static int cac_select_file_by_type(sc_card_t *card, const sc_path_t *in_path, sc
 	struct sc_apdu apdu;
 	unsigned char buf[SC_MAX_APDU_BUFFER_SIZE];
 	unsigned char pathbuf[SC_MAX_PATH_SIZE], *path = pathbuf;
-	int r, pathlen, pathtype;
+	int r, pathtype;
+	size_t pathlen;
 	struct sc_file *file = NULL;
 	cac_private_data_t * priv = CAC_DATA(card);
 
@@ -1253,7 +1244,7 @@ static int cac_select_ACA(sc_card_t *card)
 	return cac_select_file_by_type(card, &cac_ACA_Path, NULL);
 }
 
-static int cac_path_from_cardurl(sc_card_t *card, sc_path_t *path, cac_card_url_t *val, int len)
+static int cac_path_from_cardurl(sc_card_t *card, sc_path_t *path, cac_card_url_t *val, size_t len)
 {
 	if (len < 10) {
 		return SC_ERROR_INVALID_DATA;
@@ -1302,10 +1293,10 @@ static int cac_parse_aid(sc_card_t *card, cac_private_data_t *priv, const u8 *ai
 	/* Call without OID set will just select the AID without subsequent
 	 * OID selection, which we need to figure out just now
 	 */
-	cac_select_file_by_type(card, &new_object.path, NULL);
+	r = cac_select_file_by_type(card, &new_object.path, NULL);
+	LOG_TEST_RET(card->ctx, r, "Cannot select AID");
 	r = cac_get_properties(card, &prop);
-	if (r < 0)
-		return SC_ERROR_INTERNAL;
+	LOG_TEST_RET(card->ctx, r, "Cannot get CAC properties");
 
 	for (i = 0; i < prop.num_objects; i++) {
 		/* don't fail just because we have more certs than we can support */
@@ -1339,7 +1330,7 @@ static int cac_parse_aid(sc_card_t *card, cac_private_data_t *priv, const u8 *ai
 	return SC_SUCCESS;
 }
 
-static int cac_parse_cardurl(sc_card_t *card, cac_private_data_t *priv, cac_card_url_t *val, int len)
+static int cac_parse_cardurl(sc_card_t *card, cac_private_data_t *priv, cac_card_url_t *val, size_t len)
 {
 	cac_object_t new_object;
 	const cac_object_t *obj;
@@ -1528,7 +1519,7 @@ static int cac_parse_CCC(sc_card_t *card, cac_private_data_t *priv, const u8 *tl
 static int cac_process_CCC(sc_card_t *card, cac_private_data_t *priv, int depth)
 {
 	u8 *tl = NULL, *val = NULL;
-	size_t tl_len, val_len;
+	size_t tl_len = 0, val_len = 0;
 	int r;
 
 	if (depth > CAC_MAX_CCC_DEPTH) {
@@ -1831,9 +1822,6 @@ static int cac_match_card(sc_card_t *card)
 {
 	int r;
 	SC_FUNC_CALLED(card->ctx, SC_LOG_DEBUG_VERBOSE);
-	/* Since we send an APDU, the card's logout function may be called...
-	 * however it may be in dirty memory */
-	card->ops->logout = NULL;
 
 	r = cac_find_and_initialize(card, 0);
 	return (r == SC_SUCCESS); /* never match */
@@ -1862,6 +1850,12 @@ static int cac_init(sc_card_t *card)
 	LOG_FUNC_RETURN(card->ctx, SC_SUCCESS);
 }
 
+static int cac_logout(sc_card_t *card)
+{
+	int index;
+	return cac_find_first_pki_applet(card, &index);
+}
+
 static int cac_pin_cmd(sc_card_t *card, struct sc_pin_cmd_data *data, int *tries_left)
 {
 	/* CAC, like PIV needs Extra validation of (new) PIN during
@@ -1875,7 +1869,7 @@ static int cac_pin_cmd(sc_card_t *card, struct sc_pin_cmd_data *data, int *tries
 	int rv;
 
 	if (data->cmd == SC_PIN_CMD_CHANGE) {
-		int i = 0;
+		size_t i = 0;
 		if (data->pin2.len < 6) {
 			return SC_ERROR_INVALID_PIN_LENGTH;
 		}
@@ -1933,6 +1927,7 @@ static struct sc_card_driver * sc_get_driver(void)
 	cac_ops.decipher =  cac_decipher;
 	cac_ops.card_ctl = cac_card_ctl;
 	cac_ops.pin_cmd = cac_pin_cmd;
+	cac_ops.logout = cac_logout;
 
 	return &cac_drv;
 }
