@@ -23,10 +23,10 @@
  *
  * You should have received a copy of the GNU Lesser General Public
  * License along with this library; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
  */
 
-#if HAVE_CONFIG_H
+#ifdef HAVE_CONFIG_H
 #include "config.h"
 #endif
 
@@ -49,9 +49,6 @@
 #include "internal.h"
 #include "simpletlv.h"
 #include "cardctl.h"
-#ifdef ENABLE_ZLIB
-#include "compression.h"
-#endif
 #include "iso7816.h"
 #include "card-cac-common.h"
 #include "pkcs15.h"
@@ -95,18 +92,18 @@ static int cac_cac1_get_certificate(sc_card_t *card, u8 **out_buf, size_t *out_l
 		if (apdu.sw1 != 0x63 || apdu.sw2 < 1)  {
 			/* we've either finished reading, or hit an error, break */
 			r = sc_check_sw(card, apdu.sw1, apdu.sw2);
-			left -= len;
+			left -= apdu.resplen;
 			break;
 		}
 		/* Adjust the lengths */
-		left -= len;
-		out_ptr += len;
+		left -= apdu.resplen;
+		out_ptr += apdu.resplen;
 		len = MIN(left, apdu.sw2);
 	}
 	if (r < 0) {
 		SC_FUNC_RETURN(card->ctx, SC_LOG_DEBUG_VERBOSE, r);
 	}
-	r = size - left;
+	r = (int)(size - left);
 	if (*out_buf == NULL) {
 		*out_buf = malloc(r);
 		if (*out_buf == NULL) {
@@ -125,7 +122,7 @@ static int cac_cac1_get_certificate(sc_card_t *card, u8 **out_buf, size_t *out_l
  * as well as set that we want the cert from the object.
  */
 static int cac_read_binary(sc_card_t *card, unsigned int idx,
-		unsigned char *buf, size_t count, unsigned long flags)
+		unsigned char *buf, size_t count, unsigned long *flags)
 {
 	cac_private_data_t * priv = CAC_DATA(card);
 	int r = 0;
@@ -139,7 +136,7 @@ static int cac_read_binary(sc_card_t *card, unsigned int idx,
 
 	/* if we didn't return it all last time, return the remainder */
 	if (priv->cached) {
-		sc_log(card->ctx, 
+		sc_log(card->ctx,
 			"returning cached value idx=%d count=%"SC_FORMAT_LEN_SIZE_T"u",
 			idx, count);
 		if (idx > priv->cache_buf_len) {
@@ -147,10 +144,10 @@ static int cac_read_binary(sc_card_t *card, unsigned int idx,
 		}
 		len = MIN(count, priv->cache_buf_len-idx);
 		memcpy(buf, &priv->cache_buf[idx], len);
-		LOG_FUNC_RETURN(card->ctx, len);
+		LOG_FUNC_RETURN(card->ctx, (int)len);
 	}
 
-	sc_log(card->ctx, 
+	sc_log(card->ctx,
 		"clearing cache idx=%d count=%"SC_FORMAT_LEN_SIZE_T"u",
 		idx, count);
 	free(priv->cache_buf);
@@ -171,16 +168,9 @@ static int cac_read_binary(sc_card_t *card, unsigned int idx,
 
 	/* if the info byte is 1, then the cert is compressed, decompress it */
 	if ((cert_type & 0x3) == 1) {
-#ifdef ENABLE_ZLIB
-		r = sc_decompress_alloc(&priv->cache_buf, &priv->cache_buf_len,
-			cert_ptr, cert_len, COMPRESSION_AUTO);
-#else
-		sc_log(card->ctx, "CAC compression not supported, no zlib");
-		r = SC_ERROR_NOT_SUPPORTED;
-#endif
-		if (r)
-			goto done;
-	} else if (cert_len > 0) {
+		*flags |= SC_FILE_FLAG_COMPRESSED_AUTO;
+	}
+	if (cert_len > 0) {
 		priv->cache_buf = malloc(cert_len);
 		if (priv->cache_buf == NULL) {
 			r = SC_ERROR_OUT_OF_MEMORY;
@@ -195,7 +185,7 @@ static int cac_read_binary(sc_card_t *card, unsigned int idx,
 	len = MIN(count, priv->cache_buf_len-idx);
 	if (len && priv->cache_buf)
 		memcpy(buf, &priv->cache_buf[idx], len);
-	r = len;
+	r = (int)len;
 done:
 	if (val)
 		free(val);
@@ -216,7 +206,8 @@ static int cac_select_file_by_type(sc_card_t *card, const sc_path_t *in_path, sc
 	struct sc_apdu apdu;
 	unsigned char buf[SC_MAX_APDU_BUFFER_SIZE];
 	unsigned char pathbuf[SC_MAX_PATH_SIZE], *path = pathbuf;
-	int r, pathlen, pathtype;
+	int r, pathtype;
+	size_t pathlen;
 	struct sc_file *file = NULL;
 	cac_private_data_t * priv = CAC_DATA(card);
 
@@ -498,9 +489,6 @@ static int cac_match_card(sc_card_t *card)
 {
 	int r;
 	SC_FUNC_CALLED(card->ctx, SC_LOG_DEBUG_VERBOSE);
-	/* Since we send an APDU, the card's logout function may be called...
-	 * however it may be in dirty memory */
-	card->ops->logout = NULL;
 
 	r = cac_find_and_initialize(card, 0);
 	return (r == SC_SUCCESS); /* never match */
@@ -529,6 +517,12 @@ static int cac_init(sc_card_t *card)
 	LOG_FUNC_RETURN(card->ctx, SC_SUCCESS);
 }
 
+static int cac_logout(sc_card_t *card)
+{
+	int index;
+	return cac_find_first_pki_applet(card, &index);
+}
+
 static struct sc_card_operations cac_ops;
 
 static struct sc_card_driver cac1_drv = {
@@ -550,6 +544,7 @@ static struct sc_card_driver * sc_get_driver(void)
 
 	cac_ops.select_file =  cac_select_file; /* need to record object type */
 	cac_ops.read_binary = cac_read_binary;
+	cac_ops.logout = cac_logout;
 
 	return &cac1_drv;
 }
