@@ -18,10 +18,10 @@
  *
  * You should have received a copy of the GNU Lesser General Public
  * License along with this library; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
  */
 
-#if HAVE_CONFIG_H
+#ifdef HAVE_CONFIG_H
 #include "config.h"
 #endif
 
@@ -72,36 +72,16 @@ typedef struct cardos_data {
 	const sc_security_env_t * sec_env;
 } cardos_data_t;
 
-/* copied from iso7816.c */
-static void fixup_transceive_length(const struct sc_card *card,
-		struct sc_apdu *apdu)
-{
-	if (card == NULL || apdu == NULL) {
-		return;
-	}
-
-	if (apdu->lc > sc_get_max_send_size(card)) {
-		/* The lower layers will automatically do chaining */
-		apdu->flags |= SC_APDU_FLAGS_CHAINING;
-	}
-
-	if (apdu->le > sc_get_max_recv_size(card)) {
-		/* The lower layers will automatically do a GET RESPONSE, if possible.
-		 * All other workarounds must be carried out by the upper layers. */
-		apdu->le = sc_get_max_recv_size(card);
-	}
-}
-
 static int cardos_match_card(sc_card_t *card)
 {
-	unsigned char atr[SC_MAX_ATR_SIZE];
+	unsigned char atr[SC_MAX_ATR_SIZE] = {0};
 	int i;
 
 	i = _sc_match_atr(card, cardos_atrs, &card->type);
 	if (i < 0)
 		return 0;
 
-	memcpy(atr, card->atr.value, sizeof(atr));
+	memcpy(atr, card->atr.value, card->atr.len);
 
 	/* Do not change card type for CIE! */
 	if (card->type == SC_CARD_TYPE_CARDOS_CIE_V1)
@@ -114,8 +94,8 @@ static int cardos_match_card(sc_card_t *card)
 		return 1;
 	if (card->type == SC_CARD_TYPE_CARDOS_M4_2) {
 		int rv;
-		sc_apdu_t apdu;
-		u8 rbuf[SC_MAX_APDU_BUFFER_SIZE];
+		sc_apdu_t apdu = {0};
+		u8 rbuf[SC_MAX_APDU_BUFFER_SIZE] = {0};
 		/* first check some additional ATR bytes */
 		if ((atr[4] != 0xff && atr[4] != 0x02) ||
 		    (atr[6] != 0x10 && atr[6] != 0x0a) ||
@@ -131,7 +111,7 @@ static int cardos_match_card(sc_card_t *card)
 		apdu.lc = 0;
 		rv = sc_transmit_apdu(card, &apdu);
 		LOG_TEST_RET(card->ctx, rv, "APDU transmit failed");
-		if (apdu.sw1 != 0x90 || apdu.sw2 != 0x00)
+		if (apdu.sw1 != 0x90 || apdu.sw2 != 0x00 || apdu.resplen < 2)
 			return 0;
 		if (apdu.resp[0] != atr[10] ||
 		    apdu.resp[1] != atr[11])
@@ -330,8 +310,8 @@ static int cardos_init(sc_card_t *card)
 #ifdef _WIN32
 			/* Windows does not support PCSC PART_10 and may have forced reader to 255/256
 			 * https://github.com/OpenSC/OpenSC/commit/eddea6f3c2d3dafc2c09eba6695c745a61b5186f
-			 * may have reset this. if so, will override and force extended 
-			 * Most, if not all, cardos cards do extended, but not chaining 
+			 * may have reset this. if so, will override and force extended
+			 * Most, if not all, cardos cards do extended, but not chaining
 			 */
 			if (card->reader->max_send_size == 255 && card->reader->max_recv_size == 256) {
 				sc_debug(card->ctx, SC_LOG_DEBUG_VERBOSE, "resetting reader to use data_field_length");
@@ -416,61 +396,61 @@ static int cardos_finish(sc_card_t *card)
 static const struct sc_card_error cardos_errors[] = {
 /* some error inside the card */
 /* i.e. nothing you can do */
-{ 0x6581, SC_ERROR_MEMORY_FAILURE,	"EEPROM error; command aborted"}, 
+{ 0x6581, SC_ERROR_MEMORY_FAILURE,	"EEPROM error; command aborted"},
 { 0x6fff, SC_ERROR_CARD_CMD_FAILED,	"internal assertion error"},
-{ 0x6700, SC_ERROR_WRONG_LENGTH,	"LC invalid"}, 
-{ 0x6985, SC_ERROR_CARD_CMD_FAILED,	"no random number available"}, 
-{ 0x6f81, SC_ERROR_CARD_CMD_FAILED,	"file invalid, maybe checksum error"}, 
-{ 0x6f82, SC_ERROR_CARD_CMD_FAILED,	"not enough memory in xram"}, 
-{ 0x6f84, SC_ERROR_CARD_CMD_FAILED,	"general protection fault"}, 
+{ 0x6700, SC_ERROR_WRONG_LENGTH,	"LC invalid"},
+{ 0x6985, SC_ERROR_CARD_CMD_FAILED,	"no random number available"},
+{ 0x6f81, SC_ERROR_CARD_CMD_FAILED,	"file invalid, maybe checksum error"},
+{ 0x6f82, SC_ERROR_CARD_CMD_FAILED,	"not enough memory in xram"},
+{ 0x6f84, SC_ERROR_CARD_CMD_FAILED,	"general protection fault"},
 
 /* the card doesn't know this combination of ins+cla+p1+p2 */
 /* i.e. command will never work */
-{ 0x6881, SC_ERROR_NO_CARD_SUPPORT,	"logical channel not supported"}, 
-{ 0x6a86, SC_ERROR_INCORRECT_PARAMETERS,"p1/p2 invalid"}, 
-{ 0x6d00, SC_ERROR_INS_NOT_SUPPORTED,	"ins invalid"}, 
-{ 0x6e00, SC_ERROR_CLASS_NOT_SUPPORTED,	"class invalid (hi nibble)"}, 
+{ 0x6881, SC_ERROR_NO_CARD_SUPPORT,	"logical channel not supported"},
+{ 0x6a86, SC_ERROR_INCORRECT_PARAMETERS,"p1/p2 invalid"},
+{ 0x6d00, SC_ERROR_INS_NOT_SUPPORTED,	"ins invalid"},
+{ 0x6e00, SC_ERROR_CLASS_NOT_SUPPORTED,	"class invalid (hi nibble)"},
 
 /* known command, but incorrectly used */
 /* i.e. command could work, but you need to change something */
-{ 0x6981, SC_ERROR_CARD_CMD_FAILED,	"command cannot be used for file structure"}, 
-{ 0x6a80, SC_ERROR_INCORRECT_PARAMETERS,"invalid parameters in data field"}, 
-{ 0x6a81, SC_ERROR_NOT_SUPPORTED,	"function/mode not supported"}, 
-{ 0x6a85, SC_ERROR_INCORRECT_PARAMETERS,"lc does not fit the tlv structure"}, 
-{ 0x6986, SC_ERROR_INCORRECT_PARAMETERS,"no current ef selected"}, 
-{ 0x6a87, SC_ERROR_INCORRECT_PARAMETERS,"lc does not fit p1/p2"}, 
-{ 0x6c00, SC_ERROR_WRONG_LENGTH,	"le does not fit the data to be sent"}, 
-{ 0x6f83, SC_ERROR_CARD_CMD_FAILED,	"command must not be used in transaction"}, 
+{ 0x6981, SC_ERROR_CARD_CMD_FAILED,	"command cannot be used for file structure"},
+{ 0x6a80, SC_ERROR_INCORRECT_PARAMETERS,"invalid parameters in data field"},
+{ 0x6a81, SC_ERROR_NOT_SUPPORTED,	"function/mode not supported"},
+{ 0x6a85, SC_ERROR_INCORRECT_PARAMETERS,"lc does not fit the tlv structure"},
+{ 0x6986, SC_ERROR_INCORRECT_PARAMETERS,"no current ef selected"},
+{ 0x6a87, SC_ERROR_INCORRECT_PARAMETERS,"lc does not fit p1/p2"},
+{ 0x6c00, SC_ERROR_WRONG_LENGTH,	"le does not fit the data to be sent"},
+{ 0x6f83, SC_ERROR_CARD_CMD_FAILED,	"command must not be used in transaction"},
 
 /* (something) not found */
-{ 0x6987, SC_ERROR_INCORRECT_PARAMETERS,"key object for sm not found"}, 
-{ 0x6f86, SC_ERROR_CARD_CMD_FAILED,	"key object not found"}, 
-{ 0x6a82, SC_ERROR_FILE_NOT_FOUND,	"file not found"}, 
-{ 0x6a83, SC_ERROR_RECORD_NOT_FOUND,	"record not found"}, 
-{ 0x6a88, SC_ERROR_CARD_CMD_FAILED,	"object not found"}, 
+{ 0x6987, SC_ERROR_INCORRECT_PARAMETERS,"key object for sm not found"},
+{ 0x6f86, SC_ERROR_CARD_CMD_FAILED,	"key object not found"},
+{ 0x6a82, SC_ERROR_FILE_NOT_FOUND,	"file not found"},
+{ 0x6a83, SC_ERROR_RECORD_NOT_FOUND,	"record not found"},
+{ 0x6a88, SC_ERROR_CARD_CMD_FAILED,	"object not found"},
 
 /* (something) invalid */
-{ 0x6884, SC_ERROR_CARD_CMD_FAILED,	"chaining error"}, 
-{ 0x6984, SC_ERROR_CARD_CMD_FAILED,	"bs object has invalid format"}, 
-{ 0x6988, SC_ERROR_INCORRECT_PARAMETERS,"key object used for sm has invalid format"}, 
+{ 0x6884, SC_ERROR_CARD_CMD_FAILED,	"chaining error"},
+{ 0x6984, SC_ERROR_CARD_CMD_FAILED,	"bs object has invalid format"},
+{ 0x6988, SC_ERROR_INCORRECT_PARAMETERS,"key object used for sm has invalid format"},
 
 /* (something) deactivated */
 { 0x6283, SC_ERROR_CARD_CMD_FAILED,	"file is deactivated"	},
-{ 0x6983, SC_ERROR_AUTH_METHOD_BLOCKED,	"bs object blocked"}, 
+{ 0x6983, SC_ERROR_AUTH_METHOD_BLOCKED,	"bs object blocked"},
 
 /* access denied */
-{ 0x6300, SC_ERROR_SECURITY_STATUS_NOT_SATISFIED,"authentication failed"}, 
-{ 0x6982, SC_ERROR_SECURITY_STATUS_NOT_SATISFIED,"required access right not granted"}, 
+{ 0x6300, SC_ERROR_SECURITY_STATUS_NOT_SATISFIED,"authentication failed"},
+{ 0x6982, SC_ERROR_SECURITY_STATUS_NOT_SATISFIED,"required access right not granted"},
 
 /* other errors */
-{ 0x6a84, SC_ERROR_CARD_CMD_FAILED,	"not enough memory"}, 
+{ 0x6a84, SC_ERROR_CARD_CMD_FAILED,	"not enough memory"},
 
 /* command ok, execution failed */
-{ 0x6f00, SC_ERROR_CARD_CMD_FAILED,	"technical error (see eToken developers guide)"}, 
+{ 0x6f00, SC_ERROR_CARD_CMD_FAILED,	"technical error (see eToken developers guide)"},
 
 /* no error, maybe a note */
-{ 0x9000, SC_SUCCESS,		NULL}, 
-{ 0x9001, SC_SUCCESS,		"success, but eeprom weakness detected"}, 
+{ 0x9000, SC_SUCCESS,		NULL},
+{ 0x9001, SC_SUCCESS,		"success, but eeprom weakness detected"},
 { 0x9850, SC_SUCCESS,		"over/underflow using in/decrease"}
 };
 
@@ -478,10 +458,10 @@ static int cardos_check_sw(sc_card_t *card, unsigned int sw1, unsigned int sw2)
 {
 	const int err_count = sizeof(cardos_errors)/sizeof(cardos_errors[0]);
 	int i;
-			        
+
 	for (i = 0; i < err_count; i++) {
 		if (cardos_errors[i].SWs == ((sw1 << 8) | sw2)) {
-			if ( cardos_errors[i].errorstr ) 
+			if ( cardos_errors[i].errorstr )
 				sc_log(card->ctx,  "%s\n",
 				 	cardos_errors[i].errorstr);
 			return cardos_errors[i].errorno;
@@ -557,7 +537,7 @@ get_next_part:
 		}
 	}
 
-	r = fids;
+	r = (int)fids;
 
 	LOG_FUNC_RETURN(card->ctx, r);
 }
@@ -653,7 +633,7 @@ static int cardos_select_file(sc_card_t *card,
 			      sc_file_t **file)
 {
 	int r;
-	
+
 	SC_FUNC_CALLED(card->ctx, SC_LOG_DEBUG_VERBOSE);
 	r = iso_ops->select_file(card, in_path, file);
 	if (r >= 0 && file)
@@ -890,7 +870,7 @@ static int cardos_create_file(sc_card_t *card, sc_file_t *file)
 			sc_log(card->ctx,  "unable to create FCP");
 			return r;
 		}
-	
+
 		sc_format_apdu(card, &apdu, SC_APDU_CASE_3_SHORT, 0xE0, 0x00, 0x00);
 		apdu.lc      = len;
 		apdu.datalen = len;
@@ -958,7 +938,7 @@ cardos_set_security_env(sc_card_t *card,
 	/* key_ref includes card mechanism and key number
 	 * But newer cards appear to get this some other way,
 	 * We can use flags passed to know what OpenSC expects from the card
-	 * and have derived what these machanisums are. 
+	 * and have derived what these machanisums are.
 	 * Newer cards may change how this is done
 	 */
 
@@ -1002,7 +982,10 @@ cardos_set_security_env(sc_card_t *card,
 			data[7] = 0x01;
 			data[8] = key_id & 0xF0;
 			apdu.lc = apdu.datalen = 9;
-		} else if (priv->sec_env->algorithm_flags & SC_ALGORITHM_RSA_PAD_PKCS1) {
+		} else if ((env->operation == SC_SEC_OPERATION_SIGN
+						&& priv->sec_env->algorithm_flags & SC_ALGORITHM_RSA_PAD_PKCS1_TYPE_01)
+					|| (env->operation == SC_SEC_OPERATION_DECIPHER
+						&& priv->sec_env->algorithm_flags & SC_ALGORITHM_RSA_PAD_PKCS1_TYPE_02)) {
 			/* TODO this may only apply to c903 cards */
 			/* TODO or only for cards without any supported_algos or EIDComplient only */
 			data[6] = 0x80;
@@ -1077,12 +1060,12 @@ do_compute_signature(sc_card_t *card, const u8 *data, size_t datalen,
 	apdu.data    = data;
 	apdu.lc      = datalen;
 	apdu.datalen = datalen;
-	fixup_transceive_length(card, &apdu);
+	iso7816_fixup_transceive_length(card, &apdu);
 	r = sc_transmit_apdu(card, &apdu);
 	LOG_TEST_RET(card->ctx, r, "APDU transmit failed");
 
 	if (apdu.sw1 == 0x90 && apdu.sw2 == 0x00)
-		SC_FUNC_RETURN(card->ctx, SC_LOG_DEBUG_VERBOSE, apdu.resplen);
+		SC_FUNC_RETURN(card->ctx, SC_LOG_DEBUG_VERBOSE, (int)apdu.resplen);
 	else
 		SC_FUNC_RETURN(card->ctx, SC_LOG_DEBUG_VERBOSE, sc_check_sw(card, apdu.sw1, apdu.sw2));
 }
@@ -1105,9 +1088,9 @@ cardos_compute_signature(sc_card_t *card, const u8 *data, size_t datalen,
 	SC_FUNC_CALLED(ctx, SC_LOG_DEBUG_VERBOSE);
 
 	/* sec_env has algorithm_flags set from sc_get_encoding_flags sec_flags
-	 * If flags are set correctly we don't need to test anything 
-	 * TODO this assumes RSA is  PSS, PKCS1 or RAW and we are passing 
-	 * the correct data. Should work for ECDSA too.    
+	 * If flags are set correctly we don't need to test anything
+	 * TODO this assumes RSA is  PSS, PKCS1 or RAW and we are passing
+	 * the correct data. Should work for ECDSA too.
 	 * use for V5 cards and TODO should for older cards too
 	 */
 	if (card->type == SC_CARD_TYPE_CARDOS_V5_0 || card->type == SC_CARD_TYPE_CARDOS_V5_3) {
@@ -1220,7 +1203,7 @@ cardos_decipher(struct sc_card *card,
 		/*
 		 * 5.3 supports RAW as well as PKCS1 and PSS
 		 * description may strip padding if card supports it
-		 * with cards that support RAW, it always appears to 
+		 * with cards that support RAW, it always appears to
 		 * drop first 00 that is start of padding.
 		 */
 
@@ -1278,7 +1261,7 @@ cardos_lifecycle_get(sc_card_t *card, int *mode)
 	LOG_TEST_RET(card->ctx, r, "Card returned error");
 
 	if (apdu.resplen < 1) {
-		LOG_TEST_RET(card->ctx, r, "Lifecycle byte not in response");
+		LOG_TEST_RET(card->ctx, SC_ERROR_UNKNOWN_DATA_RECEIVED, "Lifecycle byte not in response");
 	}
 
 	r = SC_SUCCESS;
@@ -1314,7 +1297,7 @@ cardos_lifecycle_set(sc_card_t *card, int *mode)
 	target = *mode;
 
 	r = cardos_lifecycle_get(card, &current);
-	
+
 	if (r != SC_SUCCESS)
 		return r;
 
