@@ -15,10 +15,10 @@
  *
  * You should have received a copy of the GNU Lesser General Public
  * License along with this library; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
  */
 
-#if HAVE_CONFIG_H
+#ifdef HAVE_CONFIG_H
 #include "config.h"
 #endif
 
@@ -187,7 +187,7 @@ static int flex_init(sc_card_t *card)
 	/* FIXME: Card type detection */
 	if (1) {
 		unsigned long flags;
-		
+
 		flags = SC_ALGORITHM_RSA_RAW;
 		flags |= SC_ALGORITHM_RSA_HASH_NONE;
 		if (card->flags & FLAG_KEYGEN)
@@ -272,7 +272,7 @@ cryptoflex_process_file_attrs(sc_card_t *card, sc_file_t *file,
 	const u8 *p = buf + 2;
 	u8 b1, b2;
 	int is_mf = 0;
-	
+
 	if (buflen < 14)
 		return -1;
 	b1 = *p++;
@@ -470,7 +470,7 @@ static void cache_path(sc_card_t *card, const sc_path_t *path,
 	int result)
 {
 	sc_path_t *curpath = &card->cache.current_path;
-	
+
 	if (result < 0) {
 		curpath->len = 0;
 		return;
@@ -625,7 +625,7 @@ static int cryptoflex_list_files(sc_card_t *card, u8 *buf, size_t buflen)
 	u8 rbuf[4];
 	int r;
 	size_t count = 0;
-	
+
 	sc_format_apdu(card, &apdu, SC_APDU_CASE_2_SHORT, 0xA8, 0, 0);
 	apdu.cla = 0xF0;
 	apdu.le = 4;
@@ -641,7 +641,7 @@ static int cryptoflex_list_files(sc_card_t *card, u8 *buf, size_t buflen)
 		if (r)
 			return r;
 		if (apdu.resplen != 4) {
-			sc_log(card->ctx, 
+			sc_log(card->ctx,
 				 "expected 4 bytes, got %"SC_FORMAT_LEN_SIZE_T"u.\n",
 				 apdu.resplen);
 			return SC_ERROR_UNKNOWN_DATA_RECEIVED;
@@ -651,7 +651,7 @@ static int cryptoflex_list_files(sc_card_t *card, u8 *buf, size_t buflen)
 		count += 2;
 		buflen -= 2;
 	}
-	return count;
+	return (int)count;
 }
 
 /*
@@ -661,9 +661,9 @@ static int cyberflex_list_files(sc_card_t *card, u8 *buf, size_t buflen)
 {
 	sc_apdu_t apdu;
 	u8 rbuf[6];
-	int r;
-	size_t count = 0, p2 = 0;
-	
+	int r, p2 = 0;
+	size_t count = 0;
+
 	while (buflen > 2) {
 		sc_format_apdu(card, &apdu, SC_APDU_CASE_2_SHORT, 0xA8, 0, ++p2);
 		apdu.le = 6;
@@ -678,7 +678,7 @@ static int cyberflex_list_files(sc_card_t *card, u8 *buf, size_t buflen)
 		if (r)
 			return r;
 		if (apdu.resplen != 6) {
-			sc_log(card->ctx, 
+			sc_log(card->ctx,
 				 "expected 6 bytes, got %"SC_FORMAT_LEN_SIZE_T"u.\n",
 				 apdu.resplen);
 			return SC_ERROR_UNKNOWN_DATA_RECEIVED;
@@ -688,7 +688,7 @@ static int cyberflex_list_files(sc_card_t *card, u8 *buf, size_t buflen)
 		count += 2;
 		buflen -= 2;
 	}
-	return count;
+	return (int)count;
 }
 
 static int flex_delete_file(sc_card_t *card, const sc_path_t *path)
@@ -707,7 +707,7 @@ static int flex_delete_file(sc_card_t *card, const sc_path_t *path)
 	apdu.data = path->value;
 	apdu.lc = 2;
 	apdu.datalen = 2;
-	
+
 	r = sc_transmit_apdu(card, &apdu);
 	LOG_TEST_RET(card->ctx, r, "APDU transmit failed");
 	return sc_check_sw(card, apdu.sw1, apdu.sw2);
@@ -849,7 +849,7 @@ cyberflex_construct_file_attrs(sc_card_t *card, const sc_file_t *file,
 		break;
 	}
 
-	sc_log(card->ctx, 
+	sc_log(card->ctx,
 		 "Creating %02x:%02x, size %"SC_FORMAT_LEN_SIZE_T"u %02"SC_FORMAT_LEN_SIZE_T"x:%02"SC_FORMAT_LEN_SIZE_T"x\n",
 		 file->id >> 8,
 		 file->id & 0xFF,
@@ -883,7 +883,7 @@ cyberflex_construct_file_attrs(sc_card_t *card, const sc_file_t *file,
 		}
 	p[5] = 0x01;	/* status?? */
 	p[6] = p[7] = 0;
-	
+
 	*buflen = 16;
 
 	p[8] = p[9] = p[11] = 0xFF;
@@ -897,7 +897,7 @@ static int flex_create_file(sc_card_t *card, sc_file_t *file)
 	size_t sendlen;
 	int r, rec_nr;
 	sc_apdu_t apdu;
-	
+
 	/* Build the file attrs. These are not the real FCI bytes
 	 * in the standard sense, but its a convenient way of
 	 * abstracting the Cryptoflex/Cyberflex differences */
@@ -907,7 +907,7 @@ static int flex_create_file(sc_card_t *card, sc_file_t *file)
 		return SC_ERROR_INVALID_ARGUMENTS;
 	}
 	if (file->type != SC_FILE_TYPE_DF && file->ef_structure != SC_FILE_EF_TRANSPARENT)
-		rec_nr = file->record_count;
+		rec_nr = (int)file->record_count;
 	else
 		rec_nr = 0;
 	sc_format_apdu(card, &apdu, SC_APDU_CASE_3_SHORT, 0xE0, 0x00, rec_nr);
@@ -923,18 +923,18 @@ static int flex_create_file(sc_card_t *card, sc_file_t *file)
 	LOG_TEST_RET(card->ctx, r, "Card returned error");
 	if (card->cache.valid) {
 		u8 file_id[2];
-		
+
 		file_id[0] = file->id >> 8;
 		file_id[1] = file->id & 0xFF;
 		if (card->cache.current_path.len != 0)
 			sc_append_path_id(&card->cache.current_path, file_id, 2);
-	}		
+	}
 	return 0;
 }
 
 static int flex_set_security_env(sc_card_t *card,
 				 const sc_security_env_t *env,
-				 int se_num)   
+				 int se_num)
 {
 	struct flex_private_data *prv = (struct flex_private_data *) card->drv_data;
 
@@ -986,9 +986,9 @@ cryptoflex_compute_signature(sc_card_t *card, const u8 *data,
 	u8 sbuf[SC_MAX_APDU_BUFFER_SIZE];
 	int r;
 	size_t i, i2;
-	
+
 	if (data_len != 64 && data_len != 96 && data_len != 128  && data_len != 256) {
-		sc_log(card->ctx, 
+		sc_log(card->ctx,
 			 "Illegal input length: %"SC_FORMAT_LEN_SIZE_T"u\n",
 			 data_len);
 		return SC_ERROR_INVALID_ARGUMENTS;
@@ -1032,7 +1032,7 @@ cryptoflex_compute_signature(sc_card_t *card, const u8 *data,
 	LOG_TEST_RET(card->ctx, r, "Card returned error");
 	for (i = 0; i < apdu.resplen; i++)
 		out[i] = sbuf[apdu.resplen-1-i];
-	return apdu.resplen;
+	return (int)apdu.resplen;
 }
 
 static int
@@ -1043,13 +1043,13 @@ cyberflex_compute_signature(sc_card_t *card, const u8 *data,
 	sc_apdu_t apdu;
 	u8 alg_id, key_id;
 	int r;
-	
+
 	switch (data_len) {
 	case 64:  alg_id = 0xC4; break;
 	case 96:  alg_id = 0xC6; break;
 	case 128: alg_id = 0xC8; break;
 	default:
-		sc_log(card->ctx, 
+		sc_log(card->ctx,
 			 "Illegal input length: %"SC_FORMAT_LEN_SIZE_T"u\n",
 			 data_len);
 		return SC_ERROR_INVALID_ARGUMENTS;
@@ -1071,7 +1071,7 @@ cyberflex_compute_signature(sc_card_t *card, const u8 *data,
 	LOG_TEST_RET(card->ctx, r, "APDU transmit failed");
 	r = sc_check_sw(card, apdu.sw1, apdu.sw2);
 	LOG_TEST_RET(card->ctx, r, "Card returned error");
-	return apdu.resplen;
+	return (int)apdu.resplen;
 }
 
 static int flex_decipher(sc_card_t *card,
@@ -1116,14 +1116,14 @@ static int flex_generate_key(sc_card_t *card, struct sc_cardctl_cryptoflex_genke
 	sc_apdu_t apdu;
 	u8 sbuf[SC_MAX_APDU_BUFFER_SIZE];
 	int r, p1, p2;
-	
+
 	switch (data->key_bits) {
 	case  512:	p2 = 0x40; break;
 	case  768:	p2 = 0x60; break;
 	case 1024:	p2 = 0x80; break;
 	case 2048:	p2 = 0x00; break;
 	default:
-		sc_log(card->ctx,  "Illegal key length: %d\n", data->key_bits);
+		sc_log(card->ctx,  "Illegal key length: %zu\n", data->key_bits);
 		return SC_ERROR_INVALID_ARGUMENTS;
 	}
 
@@ -1147,7 +1147,7 @@ static int flex_generate_key(sc_card_t *card, struct sc_cardctl_cryptoflex_genke
 	r = sc_check_sw(card, apdu.sw1, apdu.sw2);
 	LOG_TEST_RET(card->ctx, r, "Card returned error");
 
-	data->pubkey_len = apdu.resplen;
+	data->pubkey_len = (unsigned)apdu.resplen;
 	return 0;
 }
 
@@ -1182,7 +1182,7 @@ static int flex_get_serialnr(sc_card_t *card, sc_serial_number_t *serial)
 	r = sc_read_binary(card, 0, buf, len, 0);
 	if (r < 0)
 		return r;
-	card->serialnr.len = len;	
+	card->serialnr.len = len;
 	memcpy(card->serialnr.value, buf, len);
 
 	memcpy(serial, &card->serialnr, sizeof(*serial));
