@@ -15,7 +15,7 @@
  *
  * You should have received a copy of the GNU Lesser General Public
  * License along with this library; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
  */
 
 /*
@@ -137,11 +137,10 @@ static struct gids_aid gids_aids[] = {
 // stolen from cardmod.h for the cardcf file
 typedef struct _CARD_CACHE_FILE_FORMAT
 {
-    unsigned char bVersion;			// Cache version
-    unsigned char bPinsFreshness;		// Card PIN
-    unsigned short wContainersFreshness;
-    unsigned short wFilesFreshness;
-
+	unsigned char bVersion;	      // Cache version
+	unsigned char bPinsFreshness; // Card PIN
+	unsigned short wContainersFreshness;
+	unsigned short wFilesFreshness;
 } CARD_CACHE_FILE_FORMAT, *PCARD_CACHE_FILE_FORMAT;
 
 struct gids_private_data {
@@ -156,25 +155,6 @@ struct gids_private_data {
 	size_t buffersize;
 };
 
-static void fixup_transceive_length(const struct sc_card *card,
-		struct sc_apdu *apdu)
-{
-	if (card == NULL || apdu == NULL) {
-		return;
-	}
-
-	if (apdu->lc > sc_get_max_send_size(card)) {
-		/* The lower layers will automatically do chaining */
-		apdu->flags |= SC_APDU_FLAGS_CHAINING;
-	}
-
-	if (apdu->le > sc_get_max_recv_size(card)) {
-		/* The lower layers will automatically do a GET RESPONSE, if possible.
-		 * All other workarounds must be carried out by the upper layers. */
-		apdu->le = sc_get_max_recv_size(card);
-	}
-}
-
 // LOW LEVEL API
 ///////////////////////////////////////////
 
@@ -189,12 +169,12 @@ static int gids_get_identifiers(sc_card_t* card, u8* masterfile, size_t masterfi
 		if (strcmp(directory, records[i].directory) == 0 && strcmp(filename, records[i].filename) == 0) {
 			*fileIdentifier = records[i].fileIdentifier;
 			*dataObjectIdentifier = records[i].dataObjectIdentifier;
-			sc_log(card->ctx, 
+			sc_log(card->ctx,
 		"Identifiers of %s %s is fileIdentifier=%x, dataObjectIdentifier=%x\n", directory, filename, *fileIdentifier, *dataObjectIdentifier);
 			return 0;
 		}
 	}
-	sc_log(card->ctx,  "file %s %s not found\n", directory, filename);
+	sc_log(card->ctx, "file %s %s not found\n", directory, filename);
 	return SC_ERROR_FILE_NOT_FOUND;
 }
 
@@ -231,9 +211,10 @@ static int gids_get_DO(sc_card_t* card, int fileIdentifier, int dataObjectIdenti
 	size_t datasize = 0;
 	const u8* p;
 	u8 buffer[MAX_GIDS_FILE_SIZE];
+	size_t buffer_len = sizeof(buffer);
 
 	SC_FUNC_CALLED(card->ctx, SC_LOG_DEBUG_VERBOSE);
-	sc_log(card->ctx, 
+	sc_log(card->ctx,
 		 "Got args: fileIdentifier=%x, dataObjectIdentifier=%x, response=%p, responselen=%"SC_FORMAT_LEN_SIZE_T"u\n",
 		 fileIdentifier, dataObjectIdentifier, response,
 		 responselen ? *responselen : 0);
@@ -244,14 +225,15 @@ static int gids_get_DO(sc_card_t* card, int fileIdentifier, int dataObjectIdenti
 	apdu.data = data;
 	apdu.datalen = 04;
 	apdu.resp = buffer;
-	apdu.resplen = sizeof(buffer);
+	apdu.resplen = buffer_len;
 	apdu.le = 256;
 
 	r = sc_transmit_apdu(card, &apdu);
 	LOG_TEST_RET(card->ctx, r, "gids get data failed");
-	LOG_TEST_RET(card->ctx,  sc_check_sw(card, apdu.sw1, apdu.sw2), "invalid return");
+	LOG_TEST_RET(card->ctx, sc_check_sw(card, apdu.sw1, apdu.sw2), "invalid return");
+	buffer_len = apdu.resplen;
 
-	p = sc_asn1_find_tag(card->ctx, buffer, sizeof(buffer), dataObjectIdentifier, &datasize);
+	p = sc_asn1_find_tag(card->ctx, buffer, buffer_len, dataObjectIdentifier, &datasize);
 	if (!p) {
 		LOG_FUNC_RETURN(card->ctx, SC_ERROR_FILE_NOT_FOUND);
 	}
@@ -272,7 +254,7 @@ static int gids_put_DO(sc_card_t* card, int fileIdentifier, int dataObjectIdenti
 	u8 buffer[SC_MAX_EXT_APDU_BUFFER_SIZE];
 	u8* p = buffer;
 	SC_FUNC_CALLED(card->ctx, SC_LOG_DEBUG_VERBOSE);
-	sc_log(card->ctx, 
+	sc_log(card->ctx,
 		 "Got args: fileIdentifier=%x, dataObjectIdentifier=%x, data=%p, datalen=%"SC_FORMAT_LEN_SIZE_T"u\n",
 		 fileIdentifier, dataObjectIdentifier, data, datalen);
 
@@ -288,7 +270,7 @@ static int gids_put_DO(sc_card_t* card, int fileIdentifier, int dataObjectIdenti
 
 	r = sc_transmit_apdu(card, &apdu);
 	LOG_TEST_RET(card->ctx, r, "gids put data failed");
-	LOG_TEST_RET(card->ctx,  sc_check_sw(card, apdu.sw1, apdu.sw2), "invalid return");
+	LOG_TEST_RET(card->ctx, sc_check_sw(card, apdu.sw1, apdu.sw2), "invalid return");
 
 	return SC_SUCCESS;
 }
@@ -300,7 +282,7 @@ static int gids_select_aid(sc_card_t* card, u8* aid, size_t aidlen, u8* response
 	int r;
 
 	SC_FUNC_CALLED(card->ctx, SC_LOG_DEBUG_VERBOSE);
-	sc_log(card->ctx, 
+	sc_log(card->ctx,
 		 "Got args: aid=%p, aidlen=%"SC_FORMAT_LEN_SIZE_T"u, response=%p, responselen=%"SC_FORMAT_LEN_SIZE_T"u\n",
 		 aid, aidlen, response, responselen ? *responselen : 0);
 
@@ -311,13 +293,13 @@ static int gids_select_aid(sc_card_t* card, u8* aid, size_t aidlen, u8* response
 	apdu.datalen = aidlen;
 	apdu.resp = response;
 	apdu.resplen = responselen ? *responselen : 0;
-	apdu.le = response == NULL ? 0 : 256; /* could be 21  for fci */
+	apdu.le = response == NULL ? 0 : 256; /* could be 21 for fci */
 
 	r = sc_transmit_apdu(card, &apdu);
 	if (responselen)
 		*responselen = apdu.resplen;
 	LOG_TEST_RET(card->ctx, r, "gids select failed");
-	SC_FUNC_RETURN(card->ctx, SC_LOG_DEBUG_VERBOSE,  sc_check_sw(card, apdu.sw1, apdu.sw2));
+	SC_FUNC_RETURN(card->ctx, SC_LOG_DEBUG_VERBOSE, sc_check_sw(card, apdu.sw1, apdu.sw2));
 }
 
 // DIRECT FILE MANIPULATION
@@ -465,7 +447,7 @@ static int gids_create_file(sc_card_t *card, char* directory, char* filename) {
 	struct gids_private_data* privatedata = (struct gids_private_data*) card->drv_data;
 	int fileIdentifier, dataObjectIdentifier;
 	size_t records;
-	int offset;
+	size_t offset;
 	gids_mf_record_t* record;
 
 	r = gids_find_available_DO(card, privatedata->masterfile, privatedata->masterfilesize, &fileIdentifier, &dataObjectIdentifier);
@@ -477,7 +459,7 @@ static int gids_create_file(sc_card_t *card, char* directory, char* filename) {
 		SC_FUNC_RETURN(card->ctx, SC_LOG_DEBUG_VERBOSE, SC_ERROR_NOT_ENOUGH_MEMORY);
 	}
 
-	records = ((privatedata->masterfilesize -1)  / sizeof(gids_mf_record_t));
+	records = ((privatedata->masterfilesize - 1) / sizeof(gids_mf_record_t));
 	offset = 1 + sizeof(gids_mf_record_t) * records;
 	memcpy(masterfilebuffer + offset + sizeof(gids_mf_record_t), masterfilebuffer + offset,
 		privatedata->masterfilesize - offset);
@@ -573,7 +555,7 @@ static int gids_get_pin_status(sc_card_t *card, int pinreference, int *tries_lef
 			*max_tries = p[0];
 	}
 
-	sc_log(card->ctx, 
+	sc_log(card->ctx,
 		"Pin information for PIN 0x%x: triesleft=%d trieslimit=%d\n", pinreference, (tries_left?*tries_left:-1), (max_tries?*max_tries:-1));
 	LOG_FUNC_RETURN(card->ctx, SC_SUCCESS);
 }
@@ -866,7 +848,7 @@ gids_decipher(struct sc_card *card,
 	 * P1:  0x80  Resp: Plain value
 	 * P2:  0x86  Cmd: Padding indicator byte followed by cryptogram
 	 * Implementation by Microsoft indicates that Padding indicator
-	 * must not be sent. It may only be needed if Secure Messaging 
+	 * must not be sent. It may only be needed if Secure Messaging
 	 * is used. This driver does not support SM.
 	 */
 	sc_format_apdu(card, &apdu, SC_APDU_CASE_4, 0x2A, 0x80, 0x86);
@@ -878,12 +860,12 @@ gids_decipher(struct sc_card *card,
 	apdu.lc = crgram_len;
 	apdu.datalen = crgram_len;
 
-	fixup_transceive_length(card, &apdu);
+	iso7816_fixup_transceive_length(card, &apdu);
 	r = sc_transmit_apdu(card, &apdu);
 	LOG_TEST_RET(card->ctx, r, "APDU transmit failed");
 
 	if (apdu.sw1 == 0x90 && apdu.sw2 == 0x00)
-		LOG_FUNC_RETURN(card->ctx, apdu.resplen);
+		LOG_FUNC_RETURN(card->ctx, (int)apdu.resplen);
 
 	LOG_FUNC_RETURN(card->ctx, sc_check_sw(card, apdu.sw1, apdu.sw2));
 }
@@ -902,7 +884,7 @@ static int gids_logout(sc_card_t *card)
 	r = sc_transmit_apdu(card, &apdu);
 	LOG_TEST_RET(card->ctx, r, "APDU transmit failed");
 
-	SC_FUNC_RETURN(card->ctx, SC_LOG_DEBUG_VERBOSE,  sc_check_sw(card, apdu.sw1, apdu.sw2));
+	SC_FUNC_RETURN(card->ctx, SC_LOG_DEBUG_VERBOSE, sc_check_sw(card, apdu.sw1, apdu.sw2));
 }
 
 // read a public key
@@ -924,7 +906,7 @@ static int gids_read_public_key (struct sc_card *card , unsigned int algorithm,
 	size_t buffersize = sizeof(buffer);
 
 	SC_FUNC_CALLED(card->ctx, SC_LOG_DEBUG_VERBOSE);
-	sc_log(card->ctx, 
+	sc_log(card->ctx,
 		 "Got args: key_reference=%x, response=%p, responselen=%"SC_FORMAT_LEN_SIZE_T"u\n",
 		 key_reference, response, responselen ? *responselen : 0);
 
@@ -939,12 +921,12 @@ static int gids_read_public_key (struct sc_card *card , unsigned int algorithm,
 
 	r = sc_transmit_apdu(card, &apdu);
 	LOG_TEST_RET(card->ctx, r, "gids read public key failed");
-	LOG_TEST_RET(card->ctx,  sc_check_sw(card, apdu.sw1, apdu.sw2), "invalid return");
+	LOG_TEST_RET(card->ctx, sc_check_sw(card, apdu.sw1, apdu.sw2), "invalid return");
 	buffersize = apdu.resplen;
 
 	keytemplate = sc_asn1_find_tag(card->ctx, buffer, buffersize, GIDS_PUBKEY_TAG, &tlen);
 	if (keytemplate == NULL) {
-		sc_log(card->ctx,  "invalid public key data: missing tag");
+		sc_log(card->ctx, "invalid public key data: missing tag");
 		LOG_FUNC_RETURN(card->ctx, SC_ERROR_INTERNAL);
 	}
 
@@ -968,7 +950,7 @@ static int gids_read_public_key (struct sc_card *card , unsigned int algorithm,
 		r = sc_pkcs15_encode_pubkey_rsa(card->ctx, &rsa_key, response, responselen);
 		LOG_TEST_RET(card->ctx, r, "failed to read public key: cannot encode RSA public key");
 	} else {
-		sc_log(card->ctx,  "it is not a known public key");
+		sc_log(card->ctx, "it is not a known public key");
 		LOG_FUNC_RETURN(card->ctx, SC_ERROR_INTERNAL);
 	}
 
@@ -1024,7 +1006,7 @@ static int gids_select_file(sc_card_t *card, const struct sc_path *in_path,
 
 static int gids_get_pin_policy(struct sc_card *card, struct sc_pin_cmd_data *data) {
 	int r;
-	if (data->pin_type != SC_AC_CHV)   {
+	if (data->pin_type != SC_AC_CHV) {
 		LOG_FUNC_RETURN(card->ctx, SC_ERROR_INVALID_ARGUMENTS);
 	}
 	r = gids_get_pin_status(card, data->pin_reference, &(data->pin1.tries_left), &(data->pin1.max_tries));
@@ -1048,7 +1030,7 @@ gids_pin_cmd(struct sc_card *card, struct sc_pin_cmd_data *data, int *tries_left
 
 // used to read existing certificates
 static int gids_read_binary(sc_card_t *card, unsigned int offset,
-		unsigned char *buf, size_t count, unsigned long flags) {
+		unsigned char *buf, size_t count, unsigned long *flags) {
 	struct gids_private_data *data = (struct gids_private_data *) card->drv_data;
 	struct sc_context *ctx = card->ctx;
 	int r;
@@ -1069,21 +1051,13 @@ static int gids_read_binary(sc_card_t *card, unsigned int offset,
 			LOG_FUNC_RETURN(card->ctx, SC_ERROR_INVALID_DATA);
 		}
 		if (buffer[0] == 1 && buffer[1] == 0) {
-			size_t expectedsize = buffer[2] + buffer[3] * 0x100;
-			data->buffersize = sizeof(data->buffer);
-			r = sc_decompress(data->buffer, &(data->buffersize), buffer+4, buffersize-4, COMPRESSION_ZLIB);
-			if (r != SC_SUCCESS) {
-				sc_log(card->ctx,  "Zlib error: %d", r);
-				LOG_FUNC_RETURN(card->ctx, r);
-			}
-			if (data->buffersize != expectedsize) {
-				sc_log(card->ctx, 
-					 "expected size: %"SC_FORMAT_LEN_SIZE_T"u real size: %"SC_FORMAT_LEN_SIZE_T"u",
-					 expectedsize, data->buffersize);
-				LOG_FUNC_RETURN(card->ctx, SC_ERROR_INVALID_DATA);
-			}
+			if (flags)
+				*flags |= SC_FILE_FLAG_COMPRESSED_ZLIB;
+			/* compressed data are starting on position buffer + 4 */
+			data->buffersize = sizeof(data->buffer) - 4;
+			memcpy(data->buffer, buffer + 4, buffersize);
 		} else {
-			sc_log(card->ctx,  "unknown compression method %d", buffer[0] + (buffer[1] <<8));
+			sc_log(card->ctx, "unknown compression method %d", buffer[0] + (buffer[1] << 8));
 			LOG_FUNC_RETURN(card->ctx, SC_ERROR_INVALID_DATA);
 		}
 		data->state = GIDS_STATE_READ_DATA_PRESENT;
@@ -1192,11 +1166,11 @@ gids_select_key_reference(sc_card_t *card, sc_pkcs15_prkey_info_t* key_info) {
 		// key was specified. Search if the key can be used
 		size_t i = key_info->key_reference - GIDS_FIRST_KEY_IDENTIFIER;
 		if (i > GIDS_MAX_CONTAINER) {
-			sc_log(card->ctx,  "invalid key ref %d", key_info->key_reference);
+			sc_log(card->ctx, "invalid key ref %d", key_info->key_reference);
 			LOG_FUNC_RETURN(card->ctx, SC_ERROR_INVALID_ARGUMENTS);
 		}
 		if (i > recordsnum) {
-			sc_log(card->ctx, 
+			sc_log(card->ctx,
 				 "container num is not allowed %"SC_FORMAT_LEN_SIZE_T"u %"SC_FORMAT_LEN_SIZE_T"u",
 				 i, recordsnum);
 			LOG_FUNC_RETURN(card->ctx, SC_ERROR_INVALID_ARGUMENTS);
@@ -1504,11 +1478,11 @@ static int gids_import_key(sc_card_t *card, sc_pkcs15_object_t *object, sc_pkcs1
 	assert((object->type & SC_PKCS15_TYPE_CLASS_MASK) == SC_PKCS15_TYPE_PRKEY);
 
 	if (object->type != SC_PKCS15_TYPE_PRKEY_RSA || key->algorithm != SC_ALGORITHM_RSA) {
-		sc_log(card->ctx,  "GIDS supports RSA keys only (but may support ECC one day).");
+		sc_log(card->ctx, "GIDS supports RSA keys only (but may support ECC one day).");
 		return SC_ERROR_NOT_SUPPORTED;
 	}
 	if (!key->u.rsa.dmp1.len || !key->u.rsa.dmq1.len || !key->u.rsa.iqmp.len) {
-		sc_log(card->ctx,  "GIDS needs dmp1 & dmq1 & iqmp");
+		sc_log(card->ctx, "GIDS needs dmp1 & dmq1 & iqmp");
 		return SC_ERROR_NOT_SUPPORTED;
 	}
 	sc_format_asn1_entry(asn1_rsa_priv_coefficients_gids + 0, &version, NULL, 1);
@@ -1778,14 +1752,14 @@ static int gids_initialize_create_file(sc_card_t *card, u8* command, size_t comm
 
 	r = sc_transmit_apdu(card, &apdu);
 	LOG_TEST_RET(card->ctx, r, "APDU1 transmit failed");
-	LOG_TEST_RET(card->ctx,  sc_check_sw(card, apdu.sw1, apdu.sw2), "invalid return");
+	LOG_TEST_RET(card->ctx, sc_check_sw(card, apdu.sw1, apdu.sw2), "invalid return");
 
 	// activate file
 	sc_format_apdu(card, &apdu, SC_APDU_CASE_1, INS_ACTIVATE_FILE, 0x00, 0x00);
 
 	r = sc_transmit_apdu(card, &apdu);
 	LOG_TEST_RET(card->ctx, r, "APDU2 transmit failed");
-	LOG_TEST_RET(card->ctx,  sc_check_sw(card, apdu.sw1, apdu.sw2), "invalid return");
+	LOG_TEST_RET(card->ctx, sc_check_sw(card, apdu.sw1, apdu.sw2), "invalid return");
 	LOG_FUNC_RETURN(card->ctx, r);
 }
 
@@ -1909,9 +1883,10 @@ static int gids_initialize(sc_card_t *card, sc_cardctl_gids_init_param_t* param)
 	}
 	if (i < 0) {
 		// set a random cardid if not set
-		r = RAND_bytes(param->cardid, sizeof(param->cardid));
-		LOG_TEST_RET(card->ctx, r, "unable to set a random serial number");
-
+		if (RAND_bytes(param->cardid, sizeof(param->cardid)) != 1) {
+			sc_log_openssl(card->ctx);
+			LOG_TEST_RET(card->ctx, SC_ERROR_INTERNAL, "unable to set a random serial number");
+		}
 	}
 #endif
 	r = gids_put_DO(card, CARDID_FI, CARDID_DO, param->cardid, sizeof(param->cardid));
@@ -1925,12 +1900,12 @@ static int gids_initialize(sc_card_t *card, sc_cardctl_gids_init_param_t* param)
 
 	r = sc_transmit_apdu(card, &apdu);
 	LOG_TEST_RET(card->ctx, r, "APDU transmit failed");
-	LOG_TEST_RET(card->ctx,  sc_check_sw(card, apdu.sw1, apdu.sw2), "invalid return");
+	LOG_TEST_RET(card->ctx, sc_check_sw(card, apdu.sw1, apdu.sw2), "invalid return");
 	// activate file
 	sc_format_apdu(card, &apdu, SC_APDU_CASE_1, INS_ACTIVATE_FILE, 0x00, 0x00);
 	r = sc_transmit_apdu(card, &apdu);
 	LOG_TEST_RET(card->ctx, r, "APDU transmit failed");
-	LOG_TEST_RET(card->ctx,  sc_check_sw(card, apdu.sw1, apdu.sw2), "invalid return");
+	LOG_TEST_RET(card->ctx, sc_check_sw(card, apdu.sw1, apdu.sw2), "invalid return");
 	LOG_FUNC_RETURN(card->ctx, r);
 }
 
@@ -1959,14 +1934,9 @@ static int gids_authenticate_admin(sc_card_t *card, u8* key) {
 	u8 buffer3[16+16+8];
 	int buffer3size = 40;
 	sc_apdu_t apdu;
-	const EVP_CIPHER *cipher;
+	EVP_CIPHER *cipher;
 
 	SC_FUNC_CALLED(card->ctx, SC_LOG_DEBUG_VERBOSE);
-	// this is CBC instead of ECB
-	cipher = EVP_des_ede3_cbc();
-	if (!cipher) {
-		LOG_FUNC_RETURN(card->ctx, SC_ERROR_INTERNAL);
-	}
 
 	// select the admin key
 	sc_format_apdu(card, &apdu, SC_APDU_CASE_3, INS_MANAGE_SECURITY_ENVIRONMENT, 0xC1, 0xA4);
@@ -1975,11 +1945,13 @@ static int gids_authenticate_admin(sc_card_t *card, u8* key) {
 	apdu.datalen = 3;
 	r = sc_transmit_apdu(card, &apdu);
 	LOG_TEST_RET(card->ctx, r, "APDU transmit failed");
-	LOG_TEST_RET(card->ctx,  sc_check_sw(card, apdu.sw1, apdu.sw2), "invalid return");
+	LOG_TEST_RET(card->ctx, sc_check_sw(card, apdu.sw1, apdu.sw2), "invalid return");
 
 	// generate a challenge
-	r = RAND_bytes(randomR1, 16);
-	LOG_TEST_RET(card->ctx, r, "unable to set computer random");
+	if (RAND_bytes(randomR1, 16) != 1) {
+		sc_log_openssl(card->ctx);
+		LOG_TEST_RET(card->ctx, SC_ERROR_INTERNAL, "unable to set computer random");
+	}
 
 	// send it to the card
 	sc_format_apdu(card, &apdu, SC_APDU_CASE_4, INS_GENERAL_AUTHENTICATE, 0x00, 0x00);
@@ -1991,11 +1963,14 @@ static int gids_authenticate_admin(sc_card_t *card, u8* key) {
 	apdu.le = 256;
 	r = sc_transmit_apdu(card, &apdu);
 	LOG_TEST_RET(card->ctx, r, "APDU transmit failed");
-	LOG_TEST_RET(card->ctx,  sc_check_sw(card, apdu.sw1, apdu.sw2), "invalid return");
+	LOG_TEST_RET(card->ctx, sc_check_sw(card, apdu.sw1, apdu.sw2), "invalid return");
 
 	// compute the half size of the mutual authentication secret
-	r = RAND_bytes(z1, 7);
-	LOG_TEST_RET(card->ctx, r, "unable to set computer random");
+	if (RAND_bytes(z1, 7) != 1) {
+		sc_log_openssl(card->ctx);
+		LOG_TEST_RET(card->ctx, SC_ERROR_INTERNAL, "unable to set computer random");
+	}
+
 	// set the padding
 	z1[7] = 0x80;
 
@@ -2006,25 +1981,34 @@ static int gids_authenticate_admin(sc_card_t *card, u8* key) {
 	// init crypto
 	ctx = EVP_CIPHER_CTX_new();
 	if (ctx == NULL) {
-	    LOG_FUNC_RETURN(card->ctx, SC_ERROR_INTERNAL);
+		sc_log_openssl(card->ctx);
+		LOG_FUNC_RETURN(card->ctx, SC_ERROR_INTERNAL);
 	}
-
+	cipher = sc_evp_cipher(card->ctx, "DES-EDE3-CBC");
 	if (!EVP_EncryptInit(ctx, cipher, key, NULL)) {
+		sc_log_openssl(card->ctx);
 		EVP_CIPHER_CTX_free(ctx);
+		sc_evp_cipher_free(cipher);
 		LOG_FUNC_RETURN(card->ctx, SC_ERROR_INTERNAL);
 	}
 	EVP_CIPHER_CTX_set_padding(ctx,0);
 	if (!EVP_EncryptUpdate(ctx, buffer2, &buffer2size, buffer, sizeof(buffer))) {
+		sc_log_openssl(card->ctx);
 		EVP_CIPHER_CTX_free(ctx);
+		sc_evp_cipher_free(cipher);
 		LOG_FUNC_RETURN(card->ctx, SC_ERROR_INTERNAL);
 	}
 
-	if(!EVP_EncryptFinal(ctx, buffer2+buffer2size, &buffer2size)) {
+	if (!EVP_EncryptFinal(ctx, buffer2 + buffer2size, &buffer2size)) {
+		sc_log_openssl(card->ctx);
 		EVP_CIPHER_CTX_free(ctx);
+		sc_evp_cipher_free(cipher);
 		LOG_FUNC_RETURN(card->ctx, SC_ERROR_INTERNAL);
 	}
 	EVP_CIPHER_CTX_free(ctx);
 	ctx = NULL;
+	sc_evp_cipher_free(cipher);
+	cipher = NULL;
 	// send it to the card
 	sc_format_apdu(card, &apdu, SC_APDU_CASE_4, INS_GENERAL_AUTHENTICATE, 0x00, 0x00);
 	apdu.lc = sizeof(apduSendReponse);
@@ -2035,48 +2019,66 @@ static int gids_authenticate_admin(sc_card_t *card, u8* key) {
 	apdu.le = 256;
 	r = sc_transmit_apdu(card, &apdu);
 	LOG_TEST_RET(card->ctx, r, "APDU transmit failed");
-	LOG_TEST_RET(card->ctx,  sc_check_sw(card, apdu.sw1, apdu.sw2), "invalid return");
-	
+	LOG_TEST_RET(card->ctx, sc_check_sw(card, apdu.sw1, apdu.sw2), "invalid return");
+
 	if (apdu.resplen != 44)
 	{
-		sc_log(card->ctx,  "Expecting a response len of 44 - found %d",(int) apdu.resplen);
+		sc_log(card->ctx, "Expecting a response len of 44 - found %d", (int)apdu.resplen);
 		LOG_FUNC_RETURN(card->ctx, SC_ERROR_INTERNAL);
 	}
 	// init crypto
 	ctx = EVP_CIPHER_CTX_new();
 	if (ctx == NULL) {
-	    LOG_FUNC_RETURN(card->ctx, SC_ERROR_INTERNAL);
+		LOG_FUNC_RETURN(card->ctx, SC_ERROR_INTERNAL);
 	}
+	cipher = sc_evp_cipher(card->ctx, "DES-EDE3-CBC");
 	if (!EVP_DecryptInit(ctx, cipher, key, NULL)) {
+		sc_log_openssl(card->ctx);
+		sc_evp_cipher_free(cipher);
 		EVP_CIPHER_CTX_free(ctx);
 		LOG_FUNC_RETURN(card->ctx, SC_ERROR_INTERNAL);
 	}
 	EVP_CIPHER_CTX_set_padding(ctx,0);
-	if (!EVP_DecryptUpdate(ctx, buffer3, &buffer3size, apdu.resp + 4, apdu.resplen - 4)) {
-		sc_log(card->ctx,  "unable to decrypt data");
+	if (!EVP_DecryptUpdate(ctx, buffer3, &buffer3size, apdu.resp + 4, (int)apdu.resplen - 4)) {
+		sc_log_openssl(card->ctx);
+		sc_log(card->ctx, "unable to decrypt data");
+		sc_evp_cipher_free(cipher);
 		EVP_CIPHER_CTX_free(ctx);
 		LOG_FUNC_RETURN(card->ctx, SC_ERROR_PIN_CODE_INCORRECT);
 	}
-	if(!EVP_DecryptFinal(ctx, buffer3+buffer3size, &buffer3size)) {
-		sc_log(card->ctx,  "unable to decrypt final data");
+	if (!EVP_DecryptFinal(ctx, buffer3 + buffer3size, &buffer3size)) {
+		sc_log_openssl(card->ctx);
+		sc_log(card->ctx, "unable to decrypt final data");
+		sc_evp_cipher_free(cipher);
 		EVP_CIPHER_CTX_free(ctx);
 		LOG_FUNC_RETURN(card->ctx, SC_ERROR_PIN_CODE_INCORRECT);
 	}
-	sc_log(card->ctx,  "data has been decrypted using the key");
+	sc_log(card->ctx, "data has been decrypted using the key");
 	if (memcmp(buffer3, randomR1, 16) != 0) {
-		sc_log(card->ctx,  "R1 doesn't match");
+		sc_log_openssl(card->ctx);
+		sc_log(card->ctx, "R1 doesn't match");
+		sc_evp_cipher_free(cipher);
+		EVP_CIPHER_CTX_free(ctx);
 		LOG_FUNC_RETURN(card->ctx, SC_ERROR_PIN_CODE_INCORRECT);
 	}
 	if (memcmp(buffer3 + 16, randomR2, 16) != 0) {
-		sc_log(card->ctx,  "R2 doesn't match");
+		sc_log_openssl(card->ctx);
+		sc_log(card->ctx, "R2 doesn't match");
+		sc_evp_cipher_free(cipher);
+		EVP_CIPHER_CTX_free(ctx);
 		LOG_FUNC_RETURN(card->ctx, SC_ERROR_PIN_CODE_INCORRECT);
 	}
 	if (buffer[39] != 0x80) {
-		sc_log(card->ctx,  "Padding not found");
+		sc_log_openssl(card->ctx);
+		sc_log(card->ctx, "Padding not found");
+		sc_evp_cipher_free(cipher);
+		EVP_CIPHER_CTX_free(ctx);
 		LOG_FUNC_RETURN(card->ctx, SC_ERROR_PIN_CODE_INCORRECT);
 	}
 	EVP_CIPHER_CTX_free(ctx);
 	ctx = NULL;
+	sc_evp_cipher_free(cipher);
+	cipher = NULL;
 
 	LOG_FUNC_RETURN(card->ctx, SC_SUCCESS);
 #endif
