@@ -18,12 +18,12 @@
  *
  * You should have received a copy of the GNU Lesser General Public
  * License along with this library; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
  *
  * best view with tabstop=4
  */
 
-#if HAVE_CONFIG_H
+#ifdef HAVE_CONFIG_H
 #include "config.h"
 #endif
 
@@ -31,10 +31,7 @@
 #include <stdlib.h>
 #include <string.h>
 #include <ctype.h>
-#include <openssl/sha.h>
 #include <openssl/evp.h>
-#include <openssl/rsa.h>
-#include <openssl/opensslv.h>
 
 #include "internal.h"
 #include "cardctl.h"
@@ -64,7 +61,7 @@ static const struct sc_atr_table oberthur_atrs[] = {
 };
 
 struct auth_senv {
-	unsigned int algorithm;
+	unsigned long algorithm;
 	int key_file_id;
 	size_t key_size;
 };
@@ -83,7 +80,7 @@ struct auth_update_component_info {
 	enum SC_CARDCTL_OBERTHUR_KEY_TYPE  type;
 	unsigned int    component;
 	unsigned char   *data;
-	unsigned int    len;
+	size_t    len;
 };
 
 
@@ -105,7 +102,7 @@ static const char *nameAidAuthentIC_V5 = "AuthentIC v5";
 #define PUBKEY_2048_ASN1_SIZE	0x10E
 
 static unsigned char rsa_der[PUBKEY_2048_ASN1_SIZE];
-static int rsa_der_len = 0;
+static size_t rsa_der_len = 0;
 
 static struct sc_file *auth_current_ef = NULL,  *auth_current_df = NULL;
 static struct sc_card_operations auth_ops;
@@ -148,7 +145,7 @@ auth_select_aid(struct sc_card *card)
 {
 	struct sc_apdu apdu;
 	unsigned char apdu_resp[SC_MAX_APDU_BUFFER_SIZE];
-	struct auth_private_data *data =  (struct auth_private_data *) card->drv_data;
+	struct auth_private_data *data = (struct auth_private_data *)card->drv_data;
 	int rv, ii;
 	struct sc_path tmp_path;
 
@@ -165,6 +162,9 @@ auth_select_aid(struct sc_card *card)
 
 	rv = sc_transmit_apdu(card, &apdu);
 	LOG_TEST_RET(card->ctx, rv, "APDU transmit failed");
+	if (apdu.resplen < 20) {
+		LOG_TEST_RET(card->ctx, SC_ERROR_UNKNOWN_DATA_RECEIVED, "Serial number has incorrect length");
+	}
 	card->serialnr.len = 4;
 	memcpy(card->serialnr.value, apdu.resp+15, 4);
 
@@ -534,12 +534,17 @@ auth_select_file(struct sc_card *card, const struct sc_path *in_path,
 				memcpy(&tmp_path, &auth_current_df->path,  sizeof(struct sc_path));
 				tmp_path.type = SC_PATH_TYPE_PARENT;
 
+				if (file_out) {
+					sc_file_free(*file_out);
+					*file_out = NULL;
+				}
+
 				rv = auth_select_file (card, &tmp_path, file_out);
 				LOG_TEST_RET(card->ctx, rv, "select file failed");
 			}
 		}
 
-		if (path.len - offs > 0)   {
+		if (path.len > offs)   {
 			struct sc_path tmp_path;
 
 			memset(&tmp_path, 0, sizeof(struct sc_path));
@@ -549,17 +554,25 @@ auth_select_file(struct sc_card *card, const struct sc_path *in_path,
 			for (ii=0; ii < path.len - offs; ii+=2)   {
 				memcpy(tmp_path.value, path.value + offs + ii, 2);
 
+				if (file_out) {
+					sc_file_free(*file_out);
+					*file_out = NULL;
+				}
+
 				rv = auth_select_file(card, &tmp_path, file_out);
 				LOG_TEST_RET(card->ctx, rv, "select file failed");
 			}
 		}
 		else if (path.len - offs == 0 && file_out)  {
-			if (sc_compare_path(&path, &auth_current_df->path))
+			if (sc_compare_path(&path, &auth_current_df->path) && file_out) {
+				sc_file_free(*file_out);
 				sc_file_dup(file_out, auth_current_df);
-			else  if (auth_current_ef)
+			} else  if (auth_current_ef && file_out) {
+				sc_file_free(*file_out);
 				sc_file_dup(file_out, auth_current_ef);
-			else
+			} else {
 				LOG_TEST_RET(card->ctx, SC_ERROR_INTERNAL, "No current EF");
+			}
 		}
 	}
 
@@ -593,7 +606,7 @@ auth_list_files(struct sc_card *card, unsigned char *buf, size_t buflen)
 	buflen = buflen < apdu.resplen ? buflen : apdu.resplen;
 	memcpy(buf, rbuf, buflen);
 
-	LOG_FUNC_RETURN(card->ctx, buflen);
+	LOG_FUNC_RETURN(card->ctx, (int)buflen);
 }
 
 
@@ -724,7 +737,8 @@ encode_file_structure_V5(struct sc_card *card, const struct sc_file *file,
 				 unsigned char *buf, size_t *buflen)
 {
 	size_t ii;
-	int rv=0, size;
+	int rv = 0;
+	size_t size;
 	unsigned char *p = buf;
 	unsigned char  ops[8] = {0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF};
 
@@ -950,7 +964,7 @@ auth_create_file(struct sc_card *card, struct sc_file *file)
 	LOG_TEST_RET(card->ctx, rv, "File structure encoding failed");
 
 	if (file->type != SC_FILE_TYPE_DF && file->ef_structure != SC_FILE_EF_TRANSPARENT)
-		rec_nr = file->record_count;
+		rec_nr = (int)file->record_count;
 	else
 		rec_nr = 0;
 
@@ -976,6 +990,7 @@ auth_create_file(struct sc_card *card, struct sc_file *file)
 		tmp_path.len = 2;
 		rv = auth_select_file(card, &tmp_path, &df_file);
 		sc_log(card->ctx, "rv %i", rv);
+		sc_file_free(df_file);
 	}
 
 	sc_file_free(auth_current_ef);
@@ -1005,7 +1020,7 @@ auth_set_security_env(struct sc_card *card,
 
 	LOG_FUNC_CALLED(card->ctx);
 	sc_log(card->ctx,
-	       "op %i; path %s; key_ref 0x%X; algos 0x%X; flags 0x%lX",
+	       "op %i; path %s; key_ref 0x%X; algos 0x%lX; flags 0x%lX",
 	       env->operation, sc_print_path(&env->file_ref), env->key_ref[0],
 	       env->algorithm_flags, env->flags);
 
@@ -1018,7 +1033,7 @@ auth_set_security_env(struct sc_card *card,
 	case SC_ALGORITHM_DES:
 	case SC_ALGORITHM_3DES:
 		sc_log(card->ctx,
-		       "algo SC_ALGORITHM_xDES: ref %X, flags %lX",
+		       "algo SC_ALGORITHM_xDES: ref %lX, flags %lX",
 		       env->algorithm_ref, env->flags);
 
 		if (env->operation == SC_SEC_OPERATION_DECIPHER)   {
@@ -1133,7 +1148,7 @@ auth_compute_signature(struct sc_card *card, const unsigned char *in, size_t ile
 
 	memcpy(out, apdu.resp, apdu.resplen);
 
-	LOG_FUNC_RETURN(card->ctx, apdu.resplen);
+	LOG_FUNC_RETURN(card->ctx, (int)apdu.resplen);
 }
 
 
@@ -1143,7 +1158,8 @@ auth_decipher(struct sc_card *card, const unsigned char *in, size_t inlen,
 {
 	struct sc_apdu apdu;
 	unsigned char resp[SC_MAX_APDU_BUFFER_SIZE];
-	int rv, _inlen = inlen;
+	int rv;
+	size_t _inlen = inlen;
 
 	LOG_FUNC_CALLED(card->ctx);
 	sc_log(card->ctx,
@@ -1200,7 +1216,7 @@ auth_decipher(struct sc_card *card, const unsigned char *in, size_t inlen,
 		outlen = apdu.resplen;
 
 	memcpy(out, apdu.resp, outlen);
-	rv = outlen;
+	rv = (int)outlen;
 
 done:
 	LOG_FUNC_RETURN(card->ctx, rv);
@@ -1230,7 +1246,7 @@ auth_encode_exponent(unsigned long exponent, unsigned char *buff, size_t buff_le
 	if (ii==buff_len)
 		return 0;
 	else
-		return ii;
+		return (int)ii;
 }
 
 
@@ -1274,9 +1290,9 @@ auth_generate_key(struct sc_card *card, int use_sm,
 	apdu.datalen = rv + 4;
 
 	rv = sc_transmit_apdu(card, &apdu);
-	LOG_TEST_RET(card->ctx, rv, "APDU transmit failed");
+	LOG_TEST_GOTO_ERR(card->ctx, rv, "APDU transmit failed");
 	rv = sc_check_sw(card, apdu.sw1, apdu.sw2);
-	LOG_TEST_RET(card->ctx, rv, "Card returned error");
+	LOG_TEST_GOTO_ERR(card->ctx, rv, "Card returned error");
 
 	memset(&tmp_path, 0, sizeof(struct sc_path));
 	tmp_path.type = SC_PATH_TYPE_FILE_ID;
@@ -1284,17 +1300,19 @@ auth_generate_key(struct sc_card *card, int use_sm,
 	memcpy(tmp_path.value, sbuf, 2);
 
 	rv = auth_select_file(card, &tmp_path, NULL);
-	LOG_TEST_RET(card->ctx, rv, "cannot select public key");
+	LOG_TEST_GOTO_ERR(card->ctx, rv, "cannot select public key");
 
 	rv = auth_read_component(card, SC_CARDCTL_OBERTHUR_KEY_RSA_PUBLIC,
 			1, apdu.resp, data->key_bits/8);
-	LOG_TEST_RET(card->ctx, rv, "auth_read_component() returned error");
+	LOG_TEST_GOTO_ERR(card->ctx, rv, "auth_read_component() returned error");
 
 	apdu.resplen = rv;
 
 	if (data->pubkey)   {
-		if (data->pubkey_len < apdu.resplen)
-			LOG_FUNC_RETURN(card->ctx, SC_ERROR_INVALID_ARGUMENTS);
+		if (data->pubkey_len < apdu.resplen) {
+			rv = SC_ERROR_INVALID_ARGUMENTS;
+			LOG_TEST_GOTO_ERR(card->ctx, rv, "invalid length received");
+		}
 
 		memcpy(data->pubkey,apdu.resp,apdu.resplen);
 	}
@@ -1305,6 +1323,9 @@ auth_generate_key(struct sc_card *card, int use_sm,
 	sc_log(card->ctx, "resulted public key len %"SC_FORMAT_LEN_SIZE_T"u",
 	       apdu.resplen);
 	LOG_FUNC_RETURN(card->ctx, SC_SUCCESS);
+err:
+	free(apdu.resp);
+	return rv;
 }
 
 
@@ -1320,7 +1341,7 @@ auth_update_component(struct sc_card *card, struct auth_update_component_info *a
 	if (args->len > sizeof(sbuf) || args->len > 0x100)
 		LOG_FUNC_RETURN(card->ctx, SC_ERROR_INVALID_ARGUMENTS);
 
-	sc_log(card->ctx, "nn %i; len %i", args->component, args->len);
+	sc_log(card->ctx, "nn %i; len %zu", args->component, args->len);
 	ins = 0xD8;
 	p1 = args->component;
 	p2 = 0x04;
@@ -1336,22 +1357,32 @@ auth_update_component(struct sc_card *card, struct auth_update_component_info *a
 		const unsigned char in[8] = {0,0,0,0,0,0,0,0};
 		unsigned char out[8];
 		EVP_CIPHER_CTX  * ctx = NULL;
+		EVP_CIPHER *alg = NULL;
 
 		if (args->len!=8 && args->len!=24)
 			LOG_FUNC_RETURN(card->ctx, SC_ERROR_INVALID_ARGUMENTS);
 
 		ctx = EVP_CIPHER_CTX_new();
-		if (ctx == NULL) 
+		if (ctx == NULL)
 		    LOG_FUNC_RETURN(card->ctx, SC_ERROR_OUT_OF_MEMORY);
 
 		p2 = 0;
 		if (args->len == 24)
-			EVP_EncryptInit_ex(ctx, EVP_des_ede(), NULL, args->data, NULL);
+			alg = sc_evp_cipher(card->ctx, "DES-EDE");
 		else
-			EVP_EncryptInit_ex(ctx, EVP_des_ecb(), NULL, args->data, NULL);
+			alg = sc_evp_cipher(card->ctx, "DES-ECB");
+		rv = EVP_EncryptInit_ex(ctx, alg, NULL, args->data, NULL);
+		if (rv == 0) {
+			sc_evp_cipher_free(alg);
+			sc_log_openssl(card->ctx);
+			sc_log(card->ctx, "OpenSSL encryption error.");
+			LOG_FUNC_RETURN(card->ctx, SC_ERROR_INTERNAL);
+		}
 		rv = EVP_EncryptUpdate(ctx, out, &outl, in, 8);
 		EVP_CIPHER_CTX_free(ctx);
+		sc_evp_cipher_free(alg);
 		if (rv == 0) {
+			sc_log_openssl(card->ctx);
 			sc_log(card->ctx, "OpenSSL encryption error.");
 			LOG_FUNC_RETURN(card->ctx, SC_ERROR_INTERNAL);
 		}
@@ -1501,7 +1532,7 @@ auth_read_component(struct sc_card *card, enum SC_CARDCTL_OBERTHUR_KEY_TYPE type
 		LOG_FUNC_RETURN(card->ctx, SC_ERROR_WRONG_LENGTH);
 
 	memcpy(out, apdu.resp, apdu.resplen);
-	LOG_FUNC_RETURN(card->ctx, apdu.resplen);
+	LOG_FUNC_RETURN(card->ctx, (int)apdu.resplen);
 }
 
 
@@ -1802,8 +1833,11 @@ auth_pin_reset_oberthur_style(struct sc_card *card, unsigned int type,
 	rv = iso_ops->select_file(card, &tmp_path, &tmp_file);
 	LOG_TEST_RET(card->ctx, rv, "select PUK file");
 
-	if (!tmp_file || tmp_file->size < OBERTHUR_AUTH_MAX_LENGTH_PUK)
+	if (!tmp_file || tmp_file->size < OBERTHUR_AUTH_MAX_LENGTH_PUK) {
+		sc_file_free(tmp_file);
 		LOG_TEST_RET(card->ctx, SC_ERROR_FILE_TOO_SMALL, "Oberthur style 'PIN RESET' failed");
+	}
+	sc_file_free(tmp_file);
 
 	rv = iso_ops->read_binary(card, 0, puk, OBERTHUR_AUTH_MAX_LENGTH_PUK, 0);
 	LOG_TEST_RET(card->ctx, rv, "read PUK file error");
@@ -1895,7 +1929,7 @@ auth_pin_cmd(struct sc_card *card, struct sc_pin_cmd_data *data, int *tries_left
 	if (data->pin_type != SC_AC_CHV)
 		LOG_TEST_RET(card->ctx, SC_ERROR_NOT_SUPPORTED, "auth_pin_cmd() unsupported PIN type");
 
-	sc_log(card->ctx, "PIN CMD:%i; reference:%i; pin1:%p/%i, pin2:%p/%i", data->cmd,
+	sc_log(card->ctx, "PIN CMD:%i; reference:%i; pin1:%p/%zu, pin2:%p/%zu", data->cmd,
 			data->pin_reference, data->pin1.data, data->pin1.len,
 			data->pin2.data, data->pin2.len);
 	switch (data->cmd) {
@@ -2034,7 +2068,7 @@ write_publickey (struct sc_card *card, unsigned int offset,
 
 	sc_log(card->ctx, "der_size %"SC_FORMAT_LEN_SIZE_T"u", der_size);
 	if (offset + len < der_size + 2)
-		LOG_FUNC_RETURN(card->ctx, len);
+		LOG_FUNC_RETURN(card->ctx, (int)len);
 
 	rv = sc_pkcs15_decode_pubkey_rsa(card->ctx, &key, rsa_der, rsa_der_len);
 	rsa_der_len = 0;
@@ -2057,7 +2091,7 @@ write_publickey (struct sc_card *card, unsigned int offset,
 	rv = auth_update_component(card, &args);
 	LOG_TEST_RET(card->ctx, rv, "Update component failed");
 
-	LOG_FUNC_RETURN(card->ctx, len);
+	LOG_FUNC_RETURN(card->ctx, (int)len);
 }
 
 
@@ -2104,9 +2138,10 @@ auth_update_binary(struct sc_card *card, unsigned int offset,
 
 static int
 auth_read_binary(struct sc_card *card, unsigned int offset,
-		unsigned char *buf, size_t count, unsigned long flags)
+		unsigned char *buf, size_t count, unsigned long *flags)
 {
 	int rv;
+	size_t sz;
 	struct sc_pkcs15_bignum bn[2];
 	unsigned char *out = NULL;
 	bn[0].data = NULL;
@@ -2119,7 +2154,7 @@ auth_read_binary(struct sc_card *card, unsigned int offset,
 
 	sc_log(card->ctx,
 	       "offset %i; size %"SC_FORMAT_LEN_SIZE_T"u; flags 0x%lX",
-	       offset, count, flags);
+	       offset, count, flags ? *flags : 0);
 	sc_log(card->ctx,"last selected : magic %X; ef %X",
 			auth_current_ef->magic, auth_current_ef->ef_structure);
 
@@ -2175,10 +2210,11 @@ auth_read_binary(struct sc_card *card, unsigned int offset,
 				rv = SC_ERROR_UNKNOWN_DATA_RECEIVED;
 				goto err;
 			}
-			rv = ((out_len - offset) > count) ? count : (out_len - offset);
-			memcpy(buf, out + offset, rv);
+			sz = MIN(out_len - offset, count);
+			memcpy(buf, out + offset, sz);
 
-			sc_log_hex(card->ctx, "write_publickey", buf, rv);
+			sc_log_hex(card->ctx, "write_publickey", buf, sz);
+			rv = (int)sz;
 		}
 	}
 	else {
@@ -2195,7 +2231,7 @@ err:
 
 
 static int
-auth_read_record(struct sc_card *card, unsigned int nr_rec,
+auth_read_record(struct sc_card *card, unsigned int nr_rec, unsigned int idx,
 		unsigned char *buf, size_t count, unsigned long flags)
 {
 	struct sc_apdu apdu;
@@ -2206,6 +2242,9 @@ auth_read_record(struct sc_card *card, unsigned int nr_rec,
 	       "auth_read_record(): nr_rec %i; count %"SC_FORMAT_LEN_SIZE_T"u",
 	       nr_rec, count);
 
+	if (nr_rec > 0xFF || idx != 0)
+		LOG_FUNC_RETURN(card->ctx, SC_ERROR_INVALID_ARGUMENTS);
+
 	sc_format_apdu(card, &apdu, SC_APDU_CASE_2_SHORT, 0xB2, nr_rec, 0);
 	apdu.p2 = (flags & SC_RECORD_EF_ID_MASK) << 3;
 	if (flags & SC_RECORD_BY_REC_NR)
@@ -2224,7 +2263,7 @@ auth_read_record(struct sc_card *card, unsigned int nr_rec,
 	rv = sc_check_sw(card, apdu.sw1, apdu.sw2);
 	LOG_TEST_RET(card->ctx, rv, "Card returned error");
 
-	LOG_FUNC_RETURN(card->ctx, apdu.resplen);
+	LOG_FUNC_RETURN(card->ctx, (int)apdu.resplen);
 }
 
 
