@@ -16,10 +16,10 @@
  *
  * You should have received a copy of the GNU Lesser General Public
  * License along with this library; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
  */
 
-#if HAVE_CONFIG_H
+#ifdef HAVE_CONFIG_H
 #include "config.h"
 #endif
 
@@ -31,6 +31,7 @@
 #include "internal.h"
 #include "iso7816.h"
 
+// clang-format off
 static const struct sc_atr_table starcos_atrs[] = {
 	{ "3B:B7:94:00:c0:24:31:fe:65:53:50:4b:32:33:90:00:b4", NULL, NULL, SC_CARD_TYPE_STARCOS_GENERIC, 0, NULL },
 	{ "3B:B7:94:00:81:31:fe:65:53:50:4b:32:33:90:00:d1", NULL, NULL, SC_CARD_TYPE_STARCOS_GENERIC, 0, NULL },
@@ -48,6 +49,7 @@ static const struct sc_atr_table starcos_atrs[] = {
 	{ "3b:df:96:ff:81:31:fe:45:80:5b:44:45:2e:42:41:5f:53:43:33:35:32:81:05:b5", NULL, NULL, SC_CARD_TYPE_STARCOS_V3_5_ESIGN, 0, NULL },
 	{ NULL, NULL, NULL, 0, 0, NULL }
 };
+// clang-format on
 
 static struct sc_card_operations starcos_ops;
 static struct sc_card_operations *iso_ops = NULL;
@@ -81,7 +83,7 @@ static const struct sc_card_error starcos_errors[] =
 typedef struct starcos_ex_data_st {
 	int    sec_ops;	/* the currently selected security operation,
 			 * i.e. SC_SEC_OPERATION_AUTHENTICATE etc. */
-	unsigned int    fix_digestInfo;
+	unsigned long    fix_digestInfo;
 	unsigned int    pin_encoding;
 } starcos_ex_data;
 
@@ -199,7 +201,7 @@ static int starcos_determine_pin_format34(sc_card_t *card, unsigned int * pin_fo
 	rv = sc_select_file(card, &path, &file);
 	LOG_TEST_RET(ctx, rv, "Cannot select EF.PWDD file");
 
-	if ( (rv = sc_read_record(card, rec_no, buf, sizeof(buf), SC_RECORD_BY_REC_NR)) > 0 ) {
+	if ( (rv = sc_read_record(card, rec_no, 0, buf, sizeof(buf), SC_RECORD_BY_REC_NR)) > 0 ) {
 		starcos_ctrl_ref_template ctrl_ref_template;
 		memset((void*)&ctrl_ref_template, 0, sizeof(ctrl_ref_template));
 		rv = starcos_parse_supported_sec_mechanisms(card, buf, rv, &ctrl_ref_template);
@@ -236,7 +238,7 @@ static int starcos_determine_pin_format35(sc_card_t *card, unsigned int * pin_fo
 	rv = sc_select_file(card, &path, &file);
 	LOG_TEST_RET(ctx, rv, "Cannot select EF.KEYD file");
 
-	while ( (rv = sc_read_record(card, rec_no++, buf, sizeof(buf), SC_RECORD_BY_REC_NR)) > 0 ) {
+	while ( (rv = sc_read_record(card, rec_no++, 0, buf, sizeof(buf), SC_RECORD_BY_REC_NR)) > 0 ) {
 		if ( buf[0] != TAG_STARCOS35_PIN_REFERENCE ) continue;
 
 		memset((void*)&ctrl_ref_template, 0, sizeof(ctrl_ref_template));
@@ -781,6 +783,7 @@ static int starcos_select_aid(sc_card_t *card,
 		file->namelen = len;
 		file->id = 0x0000;
 		file->magic = SC_FILE_MAGIC;
+
 		*file_out = file;
 	}
 	SC_FUNC_RETURN(card->ctx, SC_LOG_DEBUG_VERBOSE, SC_SUCCESS);
@@ -810,7 +813,11 @@ static int starcos_select_fid(sc_card_t *card,
 	if ( IS_V3x(card) ) {
 		if (id_hi == 0x3f && id_lo == 0x0) {
 			apdu.p1 = 0x0;
-			apdu.p2 = 0x0;
+			apdu.p2 = 0x0C;
+			apdu.le = 0;
+			apdu.resplen = 0;
+			apdu.resp = NULL;
+			apdu.cse = SC_APDU_CASE_3_SHORT;
 			isMF = 1;
 		} else if (file_out || is_file) {
 			// last component (i.e. file or path)
@@ -890,7 +897,7 @@ static int starcos_select_fid(sc_card_t *card,
 		file->id   = (id_hi << 8) + id_lo;
 		file->path = card->cache.current_path;
 
-		if (bIsDF) {
+		if (bIsDF || isMF) {
 			/* we have a DF */
 			file->type = SC_FILE_TYPE_DF;
 			file->ef_structure = SC_FILE_EF_UNKNOWN;
@@ -932,9 +939,6 @@ static int starcos_select_file(sc_card_t *card,
 	int    r, pathtype;
 	size_t i, pathlen;
 	char pbuf[SC_MAX_PATH_STRING_SIZE];
-	/* option for path caching, it is deactivated by default,
-	   but it can be enabled by setting cache_valid to option: card->cache.valid */
-	int    cache_valid = 0;
 
 	SC_FUNC_CALLED(card->ctx, SC_LOG_DEBUG_VERBOSE);
 
@@ -943,11 +947,9 @@ static int starcos_select_file(sc_card_t *card,
 		pbuf[0] = '\0';
 
 	sc_log(card->ctx,
-		 "current path (%s, %s): %s (len: %"SC_FORMAT_LEN_SIZE_T"u)\n",
+		 "current path (%s): %s (len: %"SC_FORMAT_LEN_SIZE_T"u)\n",
 		 card->cache.current_path.type == SC_PATH_TYPE_DF_NAME ?
-		 "aid" : "path",
-		 cache_valid ? "valid" : "invalid", pbuf,
-		 card->cache.current_path.len);
+		 "aid" : "path", pbuf, card->cache.current_path.len);
 
 	if ( in_path->len > sizeof(pathbuf) ) {
 		SC_FUNC_RETURN(card->ctx, SC_LOG_DEBUG_VERBOSE, SC_ERROR_BUFFER_TOO_SMALL);
@@ -965,13 +967,8 @@ static int starcos_select_file(sc_card_t *card,
 			pathlen = in_path->aid.len;
 			pathtype = SC_PATH_TYPE_DF_NAME;
 		} else {
-			if (!cache_valid
-				|| card->cache.current_path.type != SC_PATH_TYPE_DF_NAME
-				|| card->cache.current_path.len != pathlen
-				|| memcmp(card->cache.current_path.value, in_path->aid.value, in_path->aid.len) != 0 ) {
-				r = starcos_select_aid(card, in_path->aid.value, in_path->aid.len, file_out);
-				LOG_TEST_RET(card->ctx, r, "Could not select AID!");
-			}
+			r = starcos_select_aid(card, in_path->aid.value, in_path->aid.len, NULL);
+			LOG_TEST_RET(card->ctx, r, "Could not select AID!");
 
 			if (pathtype == SC_PATH_TYPE_DF_NAME) {
 				pathtype = SC_PATH_TYPE_FILE_ID;
@@ -984,26 +981,19 @@ static int starcos_select_file(sc_card_t *card,
 		/* Select with 2byte File-ID */
 		if (pathlen != 2)
 			SC_FUNC_RETURN(card->ctx, SC_LOG_DEBUG_VERBOSE,SC_ERROR_INVALID_ARGUMENTS);
-		return starcos_select_fid(card, path[0], path[1], file_out, 1);
+		r = starcos_select_fid(card, path[0],
+				path[1], path[0] == 0x3F && path[1] == 0x00 ? NULL : file_out, 1);
+		SC_FUNC_RETURN(card->ctx, SC_LOG_DEBUG_VERBOSE, r);
 	}
 	else if (pathtype == SC_PATH_TYPE_DF_NAME)
-    {	/* SELECT DF with AID */
+	{	/* SELECT DF with AID */
 		/* Select with 1-16byte Application-ID */
-		if (cache_valid
-		    && card->cache.current_path.type == SC_PATH_TYPE_DF_NAME
-		    && card->cache.current_path.len == pathlen
-		    && memcmp(card->cache.current_path.value, pathbuf, pathlen) == 0 )
-		{
-			sc_log(card->ctx,  "cache hit\n");
-			SC_FUNC_RETURN(card->ctx, SC_LOG_DEBUG_VERBOSE, SC_SUCCESS);
-		}
-		else
-			return starcos_select_aid(card, pathbuf, pathlen, file_out);
+		r = starcos_select_aid(card, pathbuf, pathlen, file_out);
+		SC_FUNC_RETURN(card->ctx, SC_LOG_DEBUG_VERBOSE, r);
 	}
 	else if (pathtype == SC_PATH_TYPE_PATH)
 	{
 		u8 n_pathbuf[SC_MAX_PATH_SIZE];
-		int bMatch = -1;
 
 		/* Select with path (sequence of File-IDs) */
 		/* Starcos (S 2.1 and SPK 2.3) only supports one
@@ -1029,82 +1019,13 @@ static int starcos_select_file(sc_card_t *card,
 			}
 		}
 
-		/* check current working directory */
-		if (cache_valid
-		    && card->cache.current_path.type == SC_PATH_TYPE_PATH
-		    && card->cache.current_path.len >= 2
-		    && card->cache.current_path.len <= pathlen )
+		for ( i=0; i<pathlen-2; i+=2 )
 		{
-			bMatch = 0;
-			for (i=0; i < card->cache.current_path.len; i+=2)
-				if (card->cache.current_path.value[i] == path[i]
-				    && card->cache.current_path.value[i+1] == path[i+1] )
-					bMatch += 2;
-
-			if ((IS_V3x(card))
-					&& bMatch > 0 && (size_t) bMatch < card->cache.current_path.len) {
-				/* we're in the wrong folder, start traversing from root */
-				bMatch = 0;
-				card->cache.current_path.len = 0;
-			}
-		}
-
-		if ( cache_valid && bMatch >= 0 )
-		{
-			if ( pathlen - bMatch == 2 )
-				/* we are in the right directory */
-				return starcos_select_fid(card, path[bMatch], path[bMatch+1], file_out, 1);
-			else if ( pathlen - bMatch > 2 )
-			{
-				/* two more steps to go */
-				sc_path_t new_path;
-
-				/* first step: change directory */
-				r = starcos_select_fid(card, path[bMatch], path[bMatch+1], NULL, 0);
-				LOG_TEST_RET(card->ctx, r, "SELECT FILE (DF-ID) failed");
-
-				memset(&new_path, 0, sizeof(sc_path_t));
-				new_path.type = SC_PATH_TYPE_PATH;
-				new_path.len  = pathlen - bMatch-2;
-				memcpy(new_path.value, &(path[bMatch+2]), new_path.len);
-				/* final step: select file */
-				return starcos_select_file(card, &new_path, file_out);
-			}
-			else /* if (bMatch - pathlen == 0) */
-			{
-				/* done: we are already in the
-				 * requested directory */
-				sc_log(card->ctx,
-					"cache hit\n");
-				/* copy file info (if necessary) */
-				if (file_out) {
-					sc_file_t *file = sc_file_new();
-					if (!file)
-						LOG_FUNC_RETURN(card->ctx, SC_ERROR_OUT_OF_MEMORY);
-					file->id = (path[pathlen-2] << 8) +
-						   path[pathlen-1];
-					file->path = card->cache.current_path;
-					file->type = SC_FILE_TYPE_DF;
-					file->ef_structure = SC_FILE_EF_UNKNOWN;
-					file->size = 0;
-					file->namelen = 0;
-					file->magic = SC_FILE_MAGIC;
-					*file_out = file;
-				}
-				/* nothing left to do */
-				return SC_SUCCESS;
-			}
-		}
-		else
-		{
-			/* no usable cache */
-			for ( i=0; i<pathlen-2; i+=2 )
-			{
-				r = starcos_select_fid(card, path[i], path[i+1], NULL, 0);
-				LOG_TEST_RET(card->ctx, r, "SELECT FILE (DF-ID) failed");
-			}
-			return starcos_select_fid(card, path[pathlen-2], path[pathlen-1], file_out, 1);
+			r = starcos_select_fid(card, path[i], path[i+1], NULL, 0);
+			LOG_TEST_RET(card->ctx, r, "SELECT FILE (DF-ID) failed");
 		}
+		r = starcos_select_fid(card, path[pathlen-2], path[pathlen-1], file_out, 1);
+		SC_FUNC_RETURN(card->ctx, SC_LOG_DEBUG_VERBOSE, r);
 	}
 	else
 		SC_FUNC_RETURN(card->ctx, SC_LOG_DEBUG_VERBOSE, SC_ERROR_INVALID_ARGUMENTS);
@@ -1181,7 +1102,7 @@ static int starcos_process_acl(sc_card_t *card, sc_file_t *file,
 		*p++ = data->data.mf.header[14];
 		/* if sm is required use combined mode */
 		if (file->acl[SC_AC_OP_CREATE] && (sc_file_get_acl_entry(file, SC_AC_OP_CREATE))->method & SC_AC_PRO)
-			tmp = 0x03;	/* combinde mode */
+			tmp = 0x03;	/* combined mode */
 		else
 			tmp = 0x00;	/* no sm */
 		*p++ = tmp;	/* use the same sm mode for all ops */
@@ -1201,7 +1122,7 @@ static int starcos_process_acl(sc_card_t *card, sc_file_t *file,
 			memcpy(p, file->name, (u8)file->namelen);
 			p   += 16;
 		} else {
-			/* (mis)use the fid as aid */
+			/* use the fid as aid */
 			*p++ = 2;
 			memset(p, 0, 16);
 			*p++ = (file->id >> 8) & 0xff;
@@ -1474,7 +1395,7 @@ static int starcos_erase_card(sc_card_t *card)
 {	/* restore the delivery state */
 	int r;
 	u8  sbuf[2];
-	sc_apdu_t apdu;
+	sc_apdu_t apdu = {0};
 
 	sbuf[0] = 0x3f;
 	sbuf[1] = 0x00;
@@ -1721,7 +1642,8 @@ static int starcos_set_security_env(sc_card_t *card,
 		if (apdu.sw1 != 0x90 || apdu.sw2 != 0x00)
 			SC_FUNC_RETURN(card->ctx, SC_LOG_DEBUG_VERBOSE, sc_check_sw(card, apdu.sw1, apdu.sw2));
 
-		if (env->algorithm_flags == SC_ALGORITHM_RSA_PAD_PKCS1) {
+		if ((operation == SC_SEC_OPERATION_SIGN && env->algorithm_flags == SC_ALGORITHM_RSA_PAD_PKCS1_TYPE_01)
+			|| (operation == SC_SEC_OPERATION_DECIPHER && env->algorithm_flags == SC_ALGORITHM_RSA_PAD_PKCS1_TYPE_02)) {
 			// input data will be already padded
 			ex_data->fix_digestInfo = 0;
 		} else {
@@ -1743,7 +1665,7 @@ static int starcos_set_security_env(sc_card_t *card,
 	}
 	pp = p;
 	if (operation == SC_SEC_OPERATION_DECIPHER){
-		if (env->algorithm_flags & SC_ALGORITHM_RSA_PAD_PKCS1) {
+		if (env->algorithm_flags & SC_ALGORITHM_RSA_PAD_PKCS1_TYPE_02) {
 			*p++ = 0x80;
 			*p++ = 0x01;
 			*p++ = 0x02;
@@ -1763,7 +1685,7 @@ static int starcos_set_security_env(sc_card_t *card,
 	}
 	/* try COMPUTE SIGNATURE */
 	if (operation == SC_SEC_OPERATION_SIGN && (
-	    env->algorithm_flags & SC_ALGORITHM_RSA_PAD_PKCS1 ||
+	    env->algorithm_flags & SC_ALGORITHM_RSA_PAD_PKCS1_TYPE_01 ||
 	    env->algorithm_flags & SC_ALGORITHM_RSA_PAD_ISO9796)) {
 		if (env->flags & SC_SEC_ENV_ALG_REF_PRESENT) {
 			*p++ = 0x80;
@@ -1774,7 +1696,7 @@ static int starcos_set_security_env(sc_card_t *card,
 			/* set the method to use based on the algorithm_flags */
 			*p++ = 0x80;
 			*p++ = 0x01;
-			if (env->algorithm_flags & SC_ALGORITHM_RSA_PAD_PKCS1) {
+			if (env->algorithm_flags & SC_ALGORITHM_RSA_PAD_PKCS1_TYPE_01) {
 				if (env->algorithm_flags & SC_ALGORITHM_RSA_HASH_SHA1)
 					*p++ = 0x12;
 				else if (env->algorithm_flags & SC_ALGORITHM_RSA_HASH_RIPEMD160)
@@ -1924,7 +1846,7 @@ static int starcos_compute_signature(sc_card_t *card,
 			if ( out != apdu.resp ) {
 				memcpy(out, apdu.resp, len);
 			}
-			SC_FUNC_RETURN(card->ctx, SC_LOG_DEBUG_VERBOSE, len);
+			SC_FUNC_RETURN(card->ctx, SC_LOG_DEBUG_VERBOSE, (int)len);
 		}
 	} else if (ex_data->sec_ops == SC_SEC_OPERATION_AUTHENTICATE) {
 		size_t tmp_len;
@@ -1958,7 +1880,7 @@ static int starcos_compute_signature(sc_card_t *card,
 			size_t len = apdu.resplen > outlen ? outlen : apdu.resplen;
 
 			memcpy(out, apdu.resp, len);
-			SC_FUNC_RETURN(card->ctx, SC_LOG_DEBUG_VERBOSE, len);
+			SC_FUNC_RETURN(card->ctx, SC_LOG_DEBUG_VERBOSE, (int)len);
 		}
 	} else
 		SC_FUNC_RETURN(card->ctx, SC_LOG_DEBUG_VERBOSE, SC_ERROR_INVALID_ARGUMENTS);
@@ -2028,7 +1950,7 @@ static int starcos_decipher(struct sc_card *card,
 		LOG_TEST_RET(card->ctx, r, "APDU transmit failed");
 
 		if (apdu.sw1 == 0x90 && apdu.sw2 == 0x00)
-			r = apdu.resplen;
+			r = (int)apdu.resplen;
 		else
 			r = sc_check_sw(card, apdu.sw1, apdu.sw2);
 	} else {
@@ -2052,7 +1974,7 @@ static int starcos_check_sw(sc_card_t *card, unsigned int sw1, unsigned int sw2)
 	sc_log(card->ctx,
 		"sw1 = 0x%02x, sw2 = 0x%02x\n", sw1, sw2);
 
-	if (sw1 == 0x90)
+	if (sw1 == 0x90 && sw2 == 0x00)
 		return SC_SUCCESS;
 	if (sw1 == 0x63 && (sw2 & ~0x0fU) == 0xc0 )
 	{
@@ -2150,18 +2072,9 @@ static int starcos_card_ctl(sc_card_t *card, unsigned long cmd, void *ptr)
 	}
 }
 
-/**
- * starcos_logout_v3_x()
- * StarCOS 3.x cards will not clear the security status by selecting MF.
- * Returning NOT_SUPPORTED would cause card reset, effectively invalidating
- * the security status.
- */
 static int starcos_logout_v3_x(sc_card_t *card)
 {
-	int r = SC_ERROR_NOT_SUPPORTED;
-	SC_FUNC_CALLED(card->ctx, SC_LOG_DEBUG_NORMAL);
-
-	SC_FUNC_RETURN(card->ctx, SC_LOG_DEBUG_VERBOSE, r);
+	return SC_ERROR_NOT_SUPPORTED;
 }
 
 static int starcos_logout(sc_card_t *card)
