@@ -16,7 +16,7 @@
  *
  * You should have received a copy of the GNU Lesser General Public
  * License along with this library; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
  */
 
 #ifndef _OPENSC_CARDCTL_H
@@ -41,21 +41,12 @@ enum {
 	SC_CARDCTL_LIFECYCLE_GET,
 	SC_CARDCTL_LIFECYCLE_SET,
 	SC_CARDCTL_GET_SERIALNR,
+	SC_CARDCTL_GET_CHANGE_COUNTER,
 	SC_CARDCTL_GET_SE_INFO,
 	SC_CARDCTL_GET_CHV_REFERENCE_IN_SE,
 	SC_CARDCTL_PKCS11_INIT_TOKEN,
 	SC_CARDCTL_PKCS11_INIT_PIN,
-
-	/*
-	 * GPK specific calls
-	 */
-	SC_CARDCTL_GPK_BASE = _CTL_PREFIX('G', 'P', 'K'),
-	SC_CARDCTL_GPK_VARIANT,
-	SC_CARDCTL_GPK_LOCK,
-	SC_CARDCTL_GPK_PKINIT,
-	SC_CARDCTL_GPK_PKLOAD,
-	SC_CARDCTL_GPK_IS_LOCKED,
-	SC_CARDCTL_GPK_GENERATE_KEY,
+	SC_CARDCTL_GET_MODEL,
 
 	/*
 	 * Cryptoflex specific calls
@@ -66,7 +57,7 @@ enum {
 	/*
 	 * TCOS specific calls
 	 */
-	SC_CARDCTL_TCOS_BASE = _CTL_PREFIX('T','C','S'),
+	SC_CARDCTL_TCOS_BASE = _CTL_PREFIX('T', 'C', 'S'),
 	SC_CARDCTL_TCOS_SETPERM,
 
 	/*
@@ -106,21 +97,10 @@ enum {
 	SC_CARDCTL_SETCOS_GENERATE_STORE_KEY,
 	SC_CARDCTL_SETCOS_ACTIVATE_FILE,
 
-	/*
-	 * Incrypto34 specific calls
-	 */
-	SC_CARDCTL_INCRYPTO34_BASE = _CTL_PREFIX('I', '3', '4'),
-	SC_CARDCTL_INCRYPTO34_PUT_DATA_FCI,
-	SC_CARDCTL_INCRYPTO34_PUT_DATA_OCI,
-	SC_CARDCTL_INCRYPTO34_PUT_DATA_SECI,
-	SC_CARDCTL_INCRYPTO34_GENERATE_KEY,
-	SC_CARDCTL_INCRYPTO34_CHANGE_KEY_DATA,
-	SC_CARDCTL_INCRYPTO34_ERASE_FILES,
-	
 	/*
 	 * Muscle specific calls
 	 */
-	SC_CARDCTL_MUSCLE_BASE = _CTL_PREFIX('M','S','C'),
+	SC_CARDCTL_MUSCLE_BASE = _CTL_PREFIX('M', 'S', 'C'),
 	SC_CARDCTL_MUSCLE_GENERATE_KEY,
 	SC_CARDCTL_MUSCLE_EXTRACT_KEY,
 	SC_CARDCTL_MUSCLE_IMPORT_KEY,
@@ -129,26 +109,26 @@ enum {
 	/*
 	 * ASEPCOS specific calls
 	 */
-	SC_CARDCTL_ASEPCOS_BASE = _CTL_PREFIX('A','S','E'),
+	SC_CARDCTL_ASEPCOS_BASE = _CTL_PREFIX('A', 'S', 'E'),
 	SC_CARDCTL_ASEPCOS_CHANGE_KEY,
 	SC_CARDCTL_ASEPCOS_AKN2FILEID,
 	SC_CARDCTL_ASEPCOS_SET_SATTR,
 	SC_CARDCTL_ASEPCOS_ACTIVATE_FILE,
 
- 	/*
+	/*
 	 * ruToken specific calls
 	 */
- 	SC_CARDCTL_RUTOKEN_BASE = _CTL_PREFIX('R', 'T', 'K'),
- 	/*  PUT_DATA  */
- 	SC_CARDCTL_RUTOKEN_CREATE_DO,
- 	SC_CARDCTL_RUTOKEN_CHANGE_DO,
- 	SC_CARDCTL_RUTOKEN_GENERATE_KEY_DO,
- 	SC_CARDCTL_RUTOKEN_DELETE_DO,
- 	SC_CARDCTL_RUTOKEN_GET_INFO,
- 	/* NON STANDARD  */
- 	SC_CARDCTL_RUTOKEN_GET_DO_INFO,
- 	SC_CARDCTL_RUTOKEN_GOST_ENCIPHER, 
- 	SC_CARDCTL_RUTOKEN_GOST_DECIPHER,
+	SC_CARDCTL_RUTOKEN_BASE = _CTL_PREFIX('R', 'T', 'K'),
+	/*  PUT_DATA  */
+	SC_CARDCTL_RUTOKEN_CREATE_DO,
+	SC_CARDCTL_RUTOKEN_CHANGE_DO,
+	SC_CARDCTL_RUTOKEN_GENERATE_KEY_DO,
+	SC_CARDCTL_RUTOKEN_DELETE_DO,
+	SC_CARDCTL_RUTOKEN_GET_INFO,
+	/* NON STANDARD  */
+	SC_CARDCTL_RUTOKEN_GET_DO_INFO,
+	SC_CARDCTL_RUTOKEN_GOST_ENCIPHER,
+	SC_CARDCTL_RUTOKEN_GOST_DECIPHER,
 	SC_CARDCTL_RUTOKEN_FORMAT_INIT,
 	SC_CARDCTL_RUTOKEN_FORMAT_END,
 
@@ -171,8 +151,8 @@ enum {
 	SC_CARDCTL_RTECP_GENERATE_KEY,
 
 	/*
-	* Westcos specific
-	*/
+	 * Westcos specific
+	 */
 	SC_CARDCTL_WESTCOS_FREEZE = _CTL_PREFIX('W', 'T', 'C'),
 	SC_CARDCTL_WESTCOS_CREATE_MF,
 	SC_CARDCTL_WESTCOS_COMMIT,
@@ -212,14 +192,14 @@ enum {
 	SC_CARDCTL_CAC_FINAL_GET_CERT_OBJECTS,
 	SC_CARDCTL_CAC_GET_ACA_PATH,
 
-        /*
+	/*
 	 * AuthentIC v3
 	 */
-        SC_CARDCTL_AUTHENTIC_BASE = _CTL_PREFIX('A','V','3'),
-        SC_CARDCTL_AUTHENTIC_SDO_CREATE,
-        SC_CARDCTL_AUTHENTIC_SDO_DELETE,
-        SC_CARDCTL_AUTHENTIC_SDO_STORE,
-        SC_CARDCTL_AUTHENTIC_SDO_GENERATE,
+	SC_CARDCTL_AUTHENTIC_BASE = _CTL_PREFIX('A', 'V', '3'),
+	SC_CARDCTL_AUTHENTIC_SDO_CREATE,
+	SC_CARDCTL_AUTHENTIC_SDO_DELETE,
+	SC_CARDCTL_AUTHENTIC_SDO_STORE,
+	SC_CARDCTL_AUTHENTIC_SDO_GENERATE,
 
 	/*
 	 * Coolkey specific calls
@@ -232,13 +212,13 @@ enum {
 	SC_CARDCTL_COOLKEY_GET_TOKEN_INFO,
 	SC_CARDCTL_COOLKEY_FIND_OBJECT,
 
-        /*
+	/*
 	 * IAS/ECC
 	 */
-	SC_CARDCTL_IASECC_BASE = _CTL_PREFIX('E','C','C'),
+	SC_CARDCTL_IASECC_BASE = _CTL_PREFIX('E', 'C', 'C'),
 	SC_CARDCTL_IASECC_GET_FREE_KEY_REFERENCE,
-	SC_CARDCTL_IASECC_SDO_MAGIC = _CTL_PREFIX('S','D','O') | 'M',
-	SC_CARDCTL_IASECC_SDO_MAGIC_PUT_DATA = _CTL_PREFIX('S','D','O') | 'P',
+	SC_CARDCTL_IASECC_SDO_MAGIC = _CTL_PREFIX('S', 'D', 'O') | 'M',
+	SC_CARDCTL_IASECC_SDO_MAGIC_PUT_DATA = _CTL_PREFIX('S', 'D', 'O') | 'P',
 	SC_CARDCTL_IASECC_SDO_PUT_DATA,
 	SC_CARDCTL_IASECC_SDO_KEY_RSA_PUT_DATA,
 	SC_CARDCTL_IASECC_SDO_GET_DATA,
@@ -269,21 +249,21 @@ enum {
 	/*
 	 * DNIe specific calls
 	 */
-    SC_CARDCTL_DNIE_BASE = _CTL_PREFIX('D', 'N', 'I'),
+	SC_CARDCTL_DNIE_BASE = _CTL_PREFIX('D', 'N', 'I'),
 	SC_CARDCTL_DNIE_GENERATE_KEY,
 	SC_CARDCTL_DNIE_GET_INFO,
 
 	/*
 	 * isoApplet Java Card Applet
 	 */
-	SC_CARDCTL_ISOAPPLET_BASE = _CTL_PREFIX('I','S','O'),
+	SC_CARDCTL_ISOAPPLET_BASE = _CTL_PREFIX('I', 'S', 'O'),
 	SC_CARDCTL_ISOAPPLET_GENERATE_KEY,
 	SC_CARDCTL_ISOAPPLET_IMPORT_KEY,
 
 	/*
 	 * GIDS cards
 	 */
-	SC_CARDCTL_GIDS_BASE = _CTL_PREFIX('G','I','D'),
+	SC_CARDCTL_GIDS_BASE = _CTL_PREFIX('G', 'I', 'D'),
 	SC_CARDCTL_GIDS_GET_ALL_CONTAINERS,
 	SC_CARDCTL_GIDS_GET_CONTAINER_DETAIL,
 	SC_CARDCTL_GIDS_SELECT_KEY_REFERENCE,
@@ -305,6 +285,7 @@ enum {
 	SC_CARDCTL_IDPRIME_GET_NEXT_OBJECT,
 	SC_CARDCTL_IDPRIME_FINAL_GET_OBJECTS,
 	SC_CARDCTL_IDPRIME_GET_TOKEN_NAME,
+	SC_CARDCTL_IDPRIME_GET_PIN_ID,
 
 };
 
@@ -351,41 +332,6 @@ struct sc_cardctl_parsed_token_info {
 	struct sc_pkcs15_tokeninfo * tokeninfo;
 };
 
-/*
- * GPK lock file.
- * Parent DF of file must be selected.
- */
-struct sc_cardctl_gpk_lock {
-	struct sc_file *	file;
-	unsigned int		operation;
-};
-
-/*
- * GPK initialize private key file.
- * Parent DF must be selected.
- */
-struct sc_cardctl_gpk_pkinit {
-	struct sc_file *	file;
-	unsigned int		privlen;
-};
-
-/*
- * GPK load private key portion.
- */
-struct sc_cardctl_gpk_pkload {
-	struct sc_file *	file;
-	u8 *			data;
-	unsigned int		len;
-	unsigned int		datalen;
-};
-
-struct sc_cardctl_gpk_genkey {
-	unsigned int		fid;
-	unsigned int		privlen;
-	unsigned char *		pubkey;
-	unsigned int		pubkey_len;
-};
-
 /*
  * Siemens CardOS PIN info
  */
@@ -396,7 +342,7 @@ struct sc_cardctl_cardos_obj_info {
 
 struct sc_cardctl_cardos_genkey_info {
 	unsigned int	key_id;
-	unsigned int	key_bits;
+	size_t	key_bits;
 	unsigned short	fid;
 };
 
@@ -409,28 +355,12 @@ struct sc_cardctl_cardos_pass_algo_flags {
 	unsigned long ext_flags; /* for EC keys */
 };
 
-/*
- * Incrypto34 PIN info
- */
-struct sc_cardctl_incrypto34_obj_info {
-	u8 *		data;
-	size_t		len;
-	unsigned int	key_id;
-	unsigned int	key_class;
-};
-
-struct sc_cardctl_incrypto34_genkey_info {
-	unsigned int	key_id;
-	unsigned int	key_bits;
-	unsigned short	fid;
-};
-
 /*
  * Cryptoflex info
  */
 struct sc_cardctl_cryptoflex_genkey_info {
 	unsigned int	key_num;
-	unsigned int	key_bits;
+	size_t	key_bits;
 	unsigned long	exponent;
 	unsigned char *	pubkey;
 	unsigned int	pubkey_len;
@@ -488,13 +418,13 @@ enum SC_CARDCTL_OBERTHUR_KEY_TYPE {
 
 struct sc_cardctl_oberthur_genkey_info {
 	unsigned int    id_prv, id_pub;
-	unsigned int    key_bits;
+	size_t    key_bits;
 	unsigned long   exponent;
 	unsigned char * pubkey;
-	unsigned int    pubkey_len;
+	size_t    pubkey_len;
 
 	int     method;     /* SC_AC_XXX */
-	int     key_ref;    /* key reference */		
+	int     key_ref;    /* key reference */
 };
 
 struct sc_cardctl_oberthur_updatekey_info {
@@ -504,14 +434,14 @@ struct sc_cardctl_oberthur_updatekey_info {
 	unsigned int    data_len;
 
 	unsigned char   id[256];
-	unsigned int    id_len;
+	size_t    id_len;
 };
 
 struct sc_cardctl_oberthur_createpin_info {
 	unsigned int type;
 	unsigned int ref;
 	const unsigned char *pin;
-	unsigned int pin_len;
+	size_t pin_len;
 	unsigned int pin_tries;
 	const unsigned char *puk;
 	unsigned int puk_len;
@@ -531,12 +461,12 @@ struct sc_cardctl_setcos_data_obj {
 
 struct sc_cardctl_setcos_gen_store_key_info {
 	int             op_type;
-	unsigned int    mod_len;     /* in bits */
-	unsigned int    pubexp_len;  /* in bits */
+	size_t    mod_len;     /* in bits */
+	size_t    pubexp_len;  /* in bits */
 	unsigned char  *pubexp;
-	unsigned int    primep_len;  /* in bits */
+	size_t    primep_len;  /* in bits */
 	unsigned char  *primep;
-	unsigned int    primeq_len;  /* in bits */
+	size_t    primeq_len;  /* in bits */
 	unsigned char  *primeq;
 };
 
@@ -545,7 +475,7 @@ struct sc_cardctl_setcos_gen_store_key_info {
  */
 typedef struct sc_cardctl_muscle_gen_key_info {
 	int 	keyType;
-	int 	keySize;
+	size_t 	keySize;
 	int 	privateKeyLocation;
 	int 	publicKeyLocation;
 } sc_cardctl_muscle_gen_key_info_t;
@@ -554,7 +484,7 @@ typedef struct sc_cardctl_muscle_gen_key_info {
 typedef struct sc_cardctl_muscle_key_info {
 	int 	keyType;
 	int 	keyLocation;
-	int 	keySize;
+	size_t 	keySize;
 	size_t 	modLength;
 	u8* 	modValue;
 	size_t 	expLength;
@@ -618,7 +548,7 @@ typedef struct {
  *  RuToken types and constants
  */
 
-#define SC_RUTOKEN_DO_PART_BODY_LEN    199    
+#define SC_RUTOKEN_DO_PART_BODY_LEN    199
 #define SC_RUTOKEN_DO_HDR_LEN  32
 
 /*   DO Types  */
@@ -645,7 +575,7 @@ typedef struct {
 
 /*  DO ID  */
 #define SC_RUTOKEN_ID_CURDF_RESID_FLAG   0x80        /*  DO placed in current DF  */
-                                            
+
 #define SC_RUTOKEN_DEF_ID_GCHV_ADMIN       0x01      /*  ID DO ADMIN  */
 #define SC_RUTOKEN_DEF_ID_GCHV_USER        0x02      /*  ID DO USER  */
 
@@ -727,7 +657,7 @@ struct sc_rutoken_decipherinfo {
 
 /*
  * EnterSafe stuff
- * 
+ *
  */
 
 #define	SC_ENTERSAFE_MF_DATA	0x01
@@ -762,7 +692,7 @@ typedef struct sc_entersafe_create_data_st {
 			   u8 init_key[16];
 		  } df;
 		  struct {
-			   u8 file_id[2];	
+			   u8 file_id[2];
 			   u8 size[2];
 			   u8 attr[2];
 			   u8 name;
@@ -831,6 +761,7 @@ typedef struct sc_epass2003_gen_key_data_st {
 	 int pukey_id;
 	 size_t key_length;
 	 u8 *modulus;
+	 size_t modulus_len;
 } sc_epass2003_gen_key_data;
 
 
@@ -887,20 +818,20 @@ typedef struct sc_rtecp_genkey_data {
 
 	struct sc_cardctl_myeid_gen_store_key_info {
 		int             op_type;
-		unsigned int	key_type;			/* value of SC_CARDCTL_MYEID_KEY_TYPE */ 
-		size_t    key_len_bits;   
+		unsigned int	key_type;			/* value of SC_CARDCTL_MYEID_KEY_TYPE */
+		size_t    key_len_bits;
 		unsigned char  *mod;
-		size_t    pubexp_len;  
+		size_t    pubexp_len;
 		unsigned char  *pubexp;
-		size_t    primep_len;  
+		size_t    primep_len;
 		unsigned char  *primep;
-		size_t    primeq_len;  
+		size_t    primeq_len;
 		unsigned char  *primeq;
-		size_t    dp1_len;  
+		size_t    dp1_len;
 		unsigned char  *dp1;
-		size_t    dq1_len;  
+		size_t    dq1_len;
 		unsigned char  *dq1;
-		size_t    invq_len;  
+		size_t    invq_len;
 		unsigned char  *invq;
 		/* new for MyEID > 3.6.0 */
 		unsigned char  *d;                  /* EC private key / Symmetric key */
@@ -914,15 +845,16 @@ typedef struct sc_rtecp_genkey_data {
  */
 typedef struct sc_cardctl_piv_genkey_info_st {
 	unsigned int	key_num;
-	unsigned int	key_algid;	/* RSA 5, 6, 7; EC 11, 14 */ 
+	unsigned int	key_algid;	/* RSA 5, 6, 7; EC 11, 14 */
 	unsigned int	key_bits;	/* RSA */
-	unsigned long	exponent;	/* RSA */
+	unsigned char *	exponent;	/* RSA */
+	size_t	exponent_len;	/* RSA */
 	unsigned char *	pubkey;		/* RSA */
-	unsigned int	pubkey_len;	/* RSA */
+	size_t	pubkey_len;	/* RSA */
 	unsigned char * ecparam;        /* EC */
 	unsigned int    ecparam_len;    /* EC */
 	unsigned char * ecpoint;        /* EC */
-	unsigned int    ecpoint_len;    /* EC */
+	size_t    ecpoint_len;    /* EC */
 
 } sc_cardctl_piv_genkey_info_t;
 
@@ -965,7 +897,7 @@ typedef struct sc_cardctl_openpgp_keygen_info {
 			size_t ecpoint_len;
 			struct sc_object_id oid;
 			u8 oid_len;
-			unsigned int key_length;
+			size_t key_length;
 		} ec;
 	} u;
 } sc_cardctl_openpgp_keygen_info_t;
@@ -1056,6 +988,7 @@ typedef struct sc_cardctl_sc_hsm_pka_register {
  */
 
 #define SC_ISOAPPLET_ALG_REF_RSA_GEN_2048 0xF3
+#define SC_ISOAPPLET_ALG_REF_RSA_GEN_4096 0xF5
 #define SC_ISOAPPLET_ALG_REF_EC_GEN 0xEC
 
 typedef struct sc_cardctl_isoApplet_ec_parameters {
