@@ -1,10 +1,10 @@
 /**
  * cwa14890.c: Implementation of Secure Messaging according CWA-14890-1 and CWA-14890-2 standards.
- * 
+ *
  * Copyright (C) 2010 Juan Antonio Martinez <jonsito@terra.es>
  *
  * This work is derived from many sources at OpenSC Project site,
- * (see references) and the information made public by Spanish 
+ * (see references) and the information made public by Spanish
  * Direccion General de la Policia y de la Guardia Civil
  *
  * This library is free software; you can redistribute it and/or
@@ -19,7 +19,7 @@
  *
  * You should have received a copy of the GNU Lesser General Public
  * License along with this library; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
  */
 
 #define __CWA14890_C__
@@ -49,10 +49,6 @@
 # include <openssl/provider.h>
 #endif
 
-#if OPENSSL_VERSION_NUMBER >= 0x30000000L
-	static OSSL_PROVIDER *legacy_provider = NULL;
-#endif
-
 #define MAX_RESP_BUFFER_SIZE 2048
 
 /**
@@ -142,7 +138,7 @@ static int cwa_increase_ssc(sc_card_t * card)
 /**
  * ISO 7816 padding.
  *
- * Adds an 0x80 at the end of buffer and as many zeroes to get len 
+ * Adds an 0x80 at the end of buffer and as many zeroes to get len
  * multiple of 8
  * Buffer must be long enough to store additional bytes
  *
@@ -362,17 +358,24 @@ static int cwa_verify_icc_certificates(sc_card_t * card,
 	/* verify sub_ca_cert against root_ca_key */
 	res = X509_verify(sub_ca_cert, root_ca_key);
 	if (!res) {
+		sc_log_openssl(ctx);
 		msg = "Cannot verify icc Sub-CA certificate";
 		res = SC_ERROR_SM_AUTHENTICATION_FAILED;
 		goto verify_icc_certificates_end;
 	}
 
 	/* extract sub_ca_key from sub_ca_cert */
-	sub_ca_key = X509_get_pubkey(sub_ca_cert);
+	if (!(sub_ca_key = X509_get_pubkey(sub_ca_cert))) {
+		sc_log_openssl(ctx);
+		msg = "Cannot extract public key icc Sub-CA certificate";
+		res = SC_ERROR_INTERNAL;
+		goto verify_icc_certificates_end;
+	}
 
 	/* verify icc_cert against sub_ca_key */
 	res = X509_verify(icc_cert, sub_ca_key);
 	if (!res) {
+		sc_log_openssl(ctx);
 		msg = "Cannot verify icc certificate";
 		res = SC_ERROR_SM_AUTHENTICATION_FAILED;
 		goto verify_icc_certificates_end;
@@ -385,15 +388,16 @@ static int cwa_verify_icc_certificates(sc_card_t * card,
 		EVP_PKEY_free(root_ca_key);
 	if (sub_ca_key)
 		EVP_PKEY_free(sub_ca_key);
-	if (res != SC_SUCCESS)
+	if (res != SC_SUCCESS) {
 		sc_log(ctx, "%s", msg);
+	}
 	LOG_FUNC_RETURN(ctx, res);
 }
 
 /**
  * Verify CVC certificates in SM establishment process.
  *
- * This is done by mean of 00 2A 00 AE 
+ * This is done by mean of 00 2A 00 AE
  * (Perform Security Operation: Verify Certificate )
  *
  * @param card pointer to card data
@@ -434,7 +438,7 @@ static int cwa_verify_cvc_certificate(sc_card_t * card,
  * Standard set_security_env() method has sc_security_env->buffer limited
  * to 8 bytes; so cannot send some of required SM commands.
  *
- * @param card pointer to card data 
+ * @param card pointer to card data
  * @param p1 apdu P1 parameter
  * @param p2 apdu P2 parameter
  * @param buffer raw data to be inserted in apdu
@@ -472,7 +476,7 @@ static int cwa_set_security_env(sc_card_t * card,
  *
  * Internal (Card) authentication (let the card verify sent ifd certs)
  *
- * @param card pointer to card data 
+ * @param card pointer to card data
  * @param sig signature buffer
  * @param dig_len signature buffer length
  * @param data data to be sent in apdu
@@ -513,7 +517,7 @@ static int cwa_internal_auth(sc_card_t * card, u8 * sig, size_t sig_len, u8 * da
 
 /**
  * Compose signature data for external auth according CWA-14890.
- * 
+ *
  * This code prepares data to be sent to ICC for external
  * authentication procedure
  *
@@ -541,11 +545,11 @@ static int cwa_prepare_external_auth(sc_card_t * card,
 	   PRND2 || - (74 bytes) random data to make buffer 128 bytes length
 	   Kifd  || - (32 bytes)- ifd random generated key
 	   sha1_hash(
-	   PRND2   ||  
-	   Kifd    || 
+	   PRND2   ||
+	   Kifd    ||
 	   RND.ICC || - (8 bytes) response to get_challenge() cmd
 	   SN.ICC  - (8 bytes) serial number from get_serialnr() cmd
-	   ) || 
+	   ) ||
 	   0xBC - iso 9796-2 padding
 	   ) - total: 128 bytes
 
@@ -601,8 +605,13 @@ static int cwa_prepare_external_auth(sc_card_t * card,
 
 	/* compose buffer data */
 	buf3[0] = 0x6A;		/* iso padding */
-	RAND_bytes(buf3 + 1, 74);	/* pRND */
-	RAND_bytes(sm->ifd.k, 32);	/* Kifd */
+	if (RAND_bytes(buf3 + 1, 74) != 1 ||		  /* pRND */
+			RAND_bytes(sm->ifd.k, 32) != 1) { /* Kifd */
+		sc_log_openssl(ctx);
+		msg = "prepare external auth: random data error";
+		res = SC_ERROR_INTERNAL;
+		goto prepare_external_auth_end;
+	}
 	memcpy(buf3 + 1 + 74, sm->ifd.k, 32);	/* copy Kifd into buffer */
 	/* prepare data to be hashed */
 	memcpy(sha_buf, buf3 + 1, 74);	/* copy pRND into sha_buf */
@@ -617,9 +626,10 @@ static int cwa_prepare_external_auth(sc_card_t * card,
 	/* decrypt with ifd private key */
 	pctx = EVP_PKEY_CTX_new(ifd_privkey, NULL);
 	if (!pctx ||
-		EVP_PKEY_decrypt_init(pctx) != 1 ||
-		EVP_PKEY_CTX_set_rsa_padding(pctx, RSA_NO_PADDING) != 1 ||
+			EVP_PKEY_decrypt_init(pctx) != 1 ||
+			EVP_PKEY_CTX_set_rsa_padding(pctx, RSA_NO_PADDING) != 1 ||
 		EVP_PKEY_decrypt(pctx, buf2, &len2, buf3, 128) != 1) {
+		sc_log_openssl(ctx);
 		msg = "Prepare external auth: ifd_privk decrypt failed";
 		res = SC_ERROR_SM_ENCRYPT_FAILED;
 		EVP_PKEY_CTX_free(pctx);
@@ -629,9 +639,10 @@ static int cwa_prepare_external_auth(sc_card_t * card,
 	pctx = NULL;
 
 	/* evaluate value of minsig and store into buf3 */
-	bn = BN_bin2bn(buf2, len2, NULL);
+	bn = BN_bin2bn(buf2, (int)len2, NULL);
 	bnsub = BN_new();
 	if (!bn || !bnsub) {
+		sc_log_openssl(ctx);
 		msg = "Prepare external auth: BN creation failed";
 		res = SC_ERROR_INTERNAL;
 		goto prepare_external_auth_end;
@@ -640,6 +651,7 @@ static int cwa_prepare_external_auth(sc_card_t * card,
 	RSA_get0_key(rsa_ifd_privkey, &ifd_privkey_n, NULL, NULL);
 #else
 	if (EVP_PKEY_get_bn_param(ifd_privkey, OSSL_PKEY_PARAM_RSA_N, &ifd_privkey_n) != 1) {
+		sc_log_openssl(ctx);
 		msg = "Prepare external auth: BN get param failed";
 		res = SC_ERROR_INTERNAL;
 		goto prepare_external_auth_end;
@@ -648,18 +660,21 @@ static int cwa_prepare_external_auth(sc_card_t * card,
 
 	res = BN_sub(bnsub, ifd_privkey_n, bn);	/* eval N.IFD-SIG */
 	if (res == 0) {		/* 1:success 0 fail */
+		sc_log_openssl(ctx);
 		msg = "Prepare external auth: BN sigmin evaluation failed";
 		res = SC_ERROR_INTERNAL;
 		goto prepare_external_auth_end;
 	}
 	bnres = (BN_cmp(bn, bnsub) < 0) ? bn : bnsub;	/* choose min(SIG,N.IFD-SIG) */
 	if (BN_num_bytes(bnres) > 128) {
+		sc_log_openssl(ctx);
 		msg = "Prepare external auth: BN sigmin result is too big";
 		res = SC_ERROR_INTERNAL;
 		goto prepare_external_auth_end;
 	}
 	len3 = BN_bn2bin(bnres, buf3);	/* convert result back into buf3 */
 	if (len3 <= 0) {
+		sc_log_openssl(ctx);
 		msg = "Prepare external auth: BN to buffer conversion failed";
 		res = SC_ERROR_INTERNAL;
 		goto prepare_external_auth_end;
@@ -668,10 +683,11 @@ static int cwa_prepare_external_auth(sc_card_t * card,
 	/* re-encrypt result with icc public key */
 	pctx = EVP_PKEY_CTX_new(icc_pubkey, NULL);
 	if (!pctx ||
-		EVP_PKEY_encrypt_init(pctx) != 1 ||
-		EVP_PKEY_CTX_set_rsa_padding(pctx, RSA_NO_PADDING) != 1 ||
-		EVP_PKEY_encrypt(pctx, buf1, &len1, buf3, 128) != 1 ||
-		(size_t) len1 != sig_len) {
+			EVP_PKEY_encrypt_init(pctx) != 1 ||
+			EVP_PKEY_CTX_set_rsa_padding(pctx, RSA_NO_PADDING) != 1 ||
+			EVP_PKEY_encrypt(pctx, buf1, &len1, buf3, 128) != 1 ||
+			(size_t)len1 != sig_len) {
+		sc_log_openssl(ctx);
 		msg = "Prepare external auth: icc_pubk encrypt failed";
 		res = SC_ERROR_SM_ENCRYPT_FAILED;
 		EVP_PKEY_CTX_free(pctx);
@@ -684,10 +700,8 @@ static int cwa_prepare_external_auth(sc_card_t * card,
 	res = SC_SUCCESS;
 
  prepare_external_auth_end:
-	if (bn)
-		BN_free(bn);
-	if (bnsub)
-		BN_free(bnsub);
+	BN_free(bn);
+	BN_free(bnsub);
 	if (buf1) {
 		sc_mem_clear(buf1, 128);
 		free(buf1);
@@ -704,17 +718,14 @@ static int cwa_prepare_external_auth(sc_card_t * card,
 		sc_mem_clear(sha_buf, 74 + 32 + 8 + 1 + 7);
 		free(sha_buf);
 	}
-	if (sha_data) {
-		free(sha_data);
-	}
+	free(sha_data);
 #if OPENSSL_VERSION_NUMBER >= 0x30000000L
-	if (ifd_privkey_n) {
-		BN_clear_free(ifd_privkey_n);
-	}
+	BN_clear_free(ifd_privkey_n);
 #endif
 
-	if (res != SC_SUCCESS)
+	if (res != SC_SUCCESS) {
 		sc_log(ctx, "%s", msg);
+	}
 	LOG_FUNC_RETURN(ctx, res);
 }
 
@@ -723,7 +734,7 @@ static int cwa_prepare_external_auth(sc_card_t * card,
  *
  * Perform external (IFD) authenticate procedure (8.4.1.2)
  *
- * @param card pointer to card data 
+ * @param card pointer to card data
  * @param sig signature buffer
  * @param sig signature buffer length
  * @return SC_SUCCESS if OK: else error code
@@ -823,9 +834,7 @@ static int cwa_compute_session_keys(sc_card_t * card)
 		sc_mem_clear(data, 32 + 4);
 		free(data);
 	}
-	if (sha_data) {
-		free(sha_data);
-	}
+	free(sha_data);
 	if (res != SC_SUCCESS)
 		sc_log(ctx, "%s", msg);
 	else {
@@ -866,14 +875,12 @@ static int cwa_compare_signature(u8 * data, size_t dlen, u8 * ifd_data)
 	if (memcmp(data + 127 - SHA_DIGEST_LENGTH, sha, SHA_DIGEST_LENGTH) == 0)
 		res = SC_SUCCESS;
  compare_signature_end:
-	if (buf)
-		free(buf);
-	if (sha)
-		free(sha);
+	free(buf);
+	free(sha);
 	return res;
 }
 
-/** 
+/**
  * check the result of internal_authenticate operation.
  *
  * Checks icc received data from internal auth procedure against
@@ -943,7 +950,7 @@ static int cwa_verify_internal_auth(sc_card_t * card,
 		goto verify_internal_done;
 	}
 
-	/* 
+	/*
 	   We have received data with this format:
 	   sigbuf = E[PK.IFD.AUT](SIGMIN)
 	   SIGMIN = min ( SIG, N.ICC-SIG )
@@ -952,7 +959,7 @@ static int cwa_verify_internal_auth(sc_card_t * card,
 	   PRND1 ||
 	   Kicc  ||
 	   sha1_hash(PRND1 || Kicc || RND.IFD || SN.IFD) ||
-	   0xBC 
+	   0xBC
 	   )
 	   So we should reverse the process and try to get valid results
 	 */
@@ -960,9 +967,10 @@ static int cwa_verify_internal_auth(sc_card_t * card,
 	/* decrypt data with our ifd priv key */
 	pctx = EVP_PKEY_CTX_new(ifd_privkey, NULL);
 	if (!pctx ||
-		EVP_PKEY_decrypt_init(pctx) != 1 ||
-		EVP_PKEY_CTX_set_rsa_padding(pctx, RSA_NO_PADDING) != 1 ||
-		EVP_PKEY_decrypt(pctx, buf1, &len1, sig, sig_len) != 1) {
+			EVP_PKEY_decrypt_init(pctx) != 1 ||
+			EVP_PKEY_CTX_set_rsa_padding(pctx, RSA_NO_PADDING) != 1 ||
+			EVP_PKEY_decrypt(pctx, buf1, &len1, sig, sig_len) != 1) {
+		sc_log_openssl(ctx);
 		msg = "Verify Signature: decrypt with ifd privk failed";
 		res = SC_ERROR_SM_ENCRYPT_FAILED;
 		EVP_PKEY_CTX_free(pctx);
@@ -976,28 +984,29 @@ static int cwa_verify_internal_auth(sc_card_t * card,
 	/* evaluate DS[SK.ICC.AUTH](SIG) trying to decrypt with icc pubk */
 	pctx = EVP_PKEY_CTX_new(icc_pubkey, NULL);
 	if (!pctx ||
-		EVP_PKEY_encrypt_init(pctx) != 1 ||
-		EVP_PKEY_CTX_set_rsa_padding(pctx, RSA_NO_PADDING) != 1 ||
-		EVP_PKEY_encrypt(pctx, buf3, &len3, buf1, len1) != 1) {
+			EVP_PKEY_encrypt_init(pctx) != 1 ||
+			EVP_PKEY_CTX_set_rsa_padding(pctx, RSA_NO_PADDING) != 1 ||
+			EVP_PKEY_encrypt(pctx, buf3, &len3, buf1, len1) != 1) {
 		EVP_PKEY_CTX_free(pctx);
+		sc_log_openssl(ctx);
 		goto verify_nicc_sig;	/* evaluate N.ICC-SIG and retry */
 	}
 
 	EVP_PKEY_CTX_free(pctx);
-	pctx = NULL;
 
 	res = cwa_compare_signature(buf3, len3, ifdbuf);
 	if (res == SC_SUCCESS)
 		goto verify_internal_ok;
 
  verify_nicc_sig:
-	/* 
-	 * Arriving here means need to evaluate N.ICC-SIG 
+	/*
+	 * Arriving here means need to evaluate N.ICC-SIG
 	 * So convert buffers to bignums to operate
 	 */
-	bn = BN_bin2bn(buf1, len1, NULL);	/* create BN data */
+	bn = BN_bin2bn(buf1, (int)len1, NULL);	/* create BN data */
 	sigbn = BN_new();
 	if (!bn || !sigbn) {
+		sc_log_openssl(ctx);
 		msg = "Verify Signature: cannot bignums creation error";
 		res = SC_ERROR_OUT_OF_MEMORY;
 		goto verify_internal_done;
@@ -1006,19 +1015,22 @@ static int cwa_verify_internal_auth(sc_card_t * card,
 	RSA_get0_key(rsa_icc_pubkey, &icc_pubkey_n, NULL, NULL);
 #else
 	if (EVP_PKEY_get_bn_param(icc_pubkey, OSSL_PKEY_PARAM_RSA_N, &icc_pubkey_n) != 1) {
+		sc_log_openssl(ctx);
 		msg = "Verify Signature: BN get param failed";
 		res = SC_ERROR_INTERNAL;
-		goto verify_internal_ok;
+		goto verify_internal_done;
 	}
 #endif
 	res = BN_sub(sigbn, icc_pubkey_n, bn);	/* eval N.ICC-SIG */
 	if (!res) {
+		sc_log_openssl(ctx);
 		msg = "Verify Signature: evaluation of N.ICC-SIG failed";
 		res = SC_ERROR_INTERNAL;
 		goto verify_internal_done;
 	}
 	len2 = BN_bn2bin(sigbn, buf2);	/* copy result to buffer */
 	if (len2 <= 0) {
+		sc_log_openssl(ctx);
 		msg = "Verify Signature: cannot convert bignum to buffer";
 		res = SC_ERROR_INTERNAL;
 		goto verify_internal_done;
@@ -1030,6 +1042,7 @@ static int cwa_verify_internal_auth(sc_card_t * card,
 		EVP_PKEY_encrypt_init(pctx) != 1 ||
 		EVP_PKEY_CTX_set_rsa_padding(pctx, RSA_NO_PADDING) != 1 ||
 		EVP_PKEY_encrypt(pctx, buf3, &len3, buf2, len2) != 1) {
+		sc_log_openssl(ctx);
 		msg = "Verify Signature: cannot get valid SIG data";
 		res = SC_ERROR_INVALID_DATA;
 		EVP_PKEY_CTX_free(pctx);
@@ -1049,22 +1062,17 @@ static int cwa_verify_internal_auth(sc_card_t * card,
 	memcpy(sm->icc.k, buf3 + 1 + 74, 32);	/* extract Kicc from buf3 */
 	res = SC_SUCCESS;
  verify_internal_done:
-	if (buf1)
-		free(buf1);
-	if (buf2)
-		free(buf2);
-	if (buf3)
-		free(buf3);
-	if (bn)
-		BN_free(bn);
-	if (sigbn)
-		BN_free(sigbn);
+	free(buf1);
+	free(buf2);
+	free(buf3);
+	BN_free(bn);
+	BN_free(sigbn);
 #if OPENSSL_VERSION_NUMBER >= 0x30000000L
-	if (icc_pubkey_n)
-		BN_clear_free(icc_pubkey_n);
+	BN_clear_free(icc_pubkey_n);
 #endif
-	if (res != SC_SUCCESS)
+	if (res != SC_SUCCESS) {
 		sc_log(ctx, "%s", msg);
+	}
 	LOG_FUNC_RETURN(ctx, res);
 }
 
@@ -1166,7 +1174,7 @@ int cwa_create_secure_channel(sc_card_t * card,
 		goto csc_end;
 	}
 
-	/* 
+	/*
 	 * Notice that this code inverts ICC and IFD certificate standard
 	 * checking sequence.
 	 */
@@ -1213,7 +1221,12 @@ int cwa_create_secure_channel(sc_card_t * card,
 	}
 
 	/* Extract public key from ICC certificate */
-	icc_pubkey = X509_get_pubkey(icc_cert);
+	if (!(icc_pubkey = X509_get_pubkey(icc_cert))) {
+		res = SC_ERROR_INTERNAL;
+		sc_log_openssl(ctx);
+		msg = "Cannot extract public key from ICC certificate";
+		goto csc_end;
+	}
 
 	/* Select Root CA in card for ifd certificate verification */
 	sc_log(ctx,
@@ -1340,7 +1353,7 @@ int cwa_create_secure_channel(sc_card_t * card,
 		goto csc_end;
 	}
 
-	/* Internal (Card) authentication (let the card verify sent ifd certs) 
+	/* Internal (Card) authentication (let the card verify sent ifd certs)
 	   SN.IFD equals 8 lsb bytes of ifd.pubk ref according cwa14890 sec 8.4.1 */
 	sc_log(ctx, "Step 8.4.1.10: Perform Internal authentication");
 	res = provider->cwa_get_sn_ifd(card);
@@ -1348,7 +1361,12 @@ int cwa_create_secure_channel(sc_card_t * card,
 		msg = "Cannot get ifd serial number from provider";
 		goto csc_end;
 	}
-	RAND_bytes(sm->ifd.rnd, 8);	/* generate 8 random bytes */
+	/* generate 8 random bytes */
+	if (RAND_bytes(sm->ifd.rnd, 8) != 1) {
+		msg = "Cannot generate random data";
+		res = SC_ERROR_INTERNAL;
+		goto csc_end;
+	}
 	memcpy(rndbuf, sm->ifd.rnd, 8);	/* insert RND.IFD into rndbuf */
 	memcpy(rndbuf + 8, sm->ifd.sn, 8);	/* insert SN.IFD into rndbuf */
 	res = cwa_internal_auth(card, sig, 128, rndbuf, 16);
@@ -1423,14 +1441,10 @@ int cwa_create_secure_channel(sc_card_t * card,
 	res = SC_SUCCESS;
  csc_end:
 	free(tlv);
-	if (icc_cert)
-		X509_free(icc_cert);
-	if (ca_cert)
-		X509_free(ca_cert);
-	if (icc_pubkey)
-		EVP_PKEY_free(icc_pubkey);
-	if (ifd_privkey)
-		EVP_PKEY_free(ifd_privkey);
+	X509_free(icc_cert);
+	X509_free(ca_cert);
+	EVP_PKEY_free(icc_pubkey);
+	EVP_PKEY_free(ifd_privkey);
 	/* setup SM state according result */
 	if (res != SC_SUCCESS) {
 		sc_log(ctx, "%s", msg);
@@ -1475,6 +1489,7 @@ int cwa_encode_apdu(sc_card_t * card,
 	u8 *cryptbuf = NULL;
 
 	EVP_CIPHER_CTX *cctx = NULL;
+	EVP_CIPHER *alg = NULL;
 	unsigned char *key = NULL;
 	int tmplen = 0;
 
@@ -1553,22 +1568,25 @@ int cwa_encode_apdu(sc_card_t * card,
 	/* if no data, skip data encryption step */
 	if (from->lc != 0) {
 		unsigned char iv[8] = { 0, 0, 0, 0, 0, 0, 0, 0 };
-		int dlen = from->lc;
+		int dlen = (int)from->lc;
 		size_t len = dlen;
 
 		/* pad message */
 		memcpy(msgbuf, from->data, dlen);
 		cwa_iso7816_padding(msgbuf, &len);
-		dlen = len;
+		dlen = (int)len;
 
 		/* start kriptbuff with iso padding indicator */
 		*cryptbuf = 0x01;
 		key = sm_session->session_enc;
 
-		if (EVP_EncryptInit_ex(cctx, EVP_des_ede_cbc(), NULL, key, iv) != 1 ||
-			EVP_CIPHER_CTX_set_padding(cctx, 0) != 1 ||
-			EVP_EncryptUpdate(cctx, cryptbuf + 1, &dlen, msgbuf, dlen) != 1 ||
-			EVP_EncryptFinal_ex(cctx, cryptbuf + 1 + dlen, &tmplen) != 1) {
+		alg = sc_evp_cipher(card->ctx, "DES-EDE-CBC");
+
+		if (!alg ||
+				EVP_EncryptInit_ex(cctx, alg, NULL, key, iv) != 1 ||
+				EVP_CIPHER_CTX_set_padding(cctx, 0) != 1 ||
+				EVP_EncryptUpdate(cctx, cryptbuf + 1, &dlen, msgbuf, dlen) != 1 ||
+				EVP_EncryptFinal_ex(cctx, cryptbuf + 1 + dlen, &tmplen) != 1) {
 			msg = "Error in encrypting APDU";
 			res = SC_ERROR_INTERNAL;
 			goto encode_end;
@@ -1582,24 +1600,28 @@ int cwa_encode_apdu(sc_card_t * card,
 			goto encode_end;
 		}
 	} else if ((0xff & from->le) > 0) {
-
-	/* if le byte is declared, compose and add Le TLV */
-	/* FIXME: For DNIe we must not send the le bytes
-	  when le == 256 but this goes against the standard
-	  and might break other cards reusing this code */
-        /* NOTE: In FNMT MultiPKCS11 code this is an if, i.e.,
-           the le is only sent if no data (lc) is set.
-           In DNIe 3.0 pin verification sending both TLV return
-           69 88 "SM Data Object incorrect". For the moment it is
-           fixed sendind le=0 in pin verification apdu */
-	    u8 le = 0xff & from->le;
-	    res = cwa_compose_tlv(card, 0x97, 1, &le, &ccbuf, &cclen);
-	    if (res != SC_SUCCESS) {
-		msg = "Encode APDU compose_tlv(0x97) failed";
-		goto encode_end;
-	    }
+		/* if le byte is declared, compose and add Le TLV */
+		/* FIXME: For DNIe we must not send the le bytes
+		   when le == 256 but this goes against the standard
+		   and might break other cards reusing this code */
+		/* NOTE: In FNMT MultiPKCS11 code this is an if, i.e.,
+		   the le is only sent if no data (lc) is set.
+		   In DNIe 3.0 pin verification sending both TLV return
+		   69 88 "SM Data Object incorrect". For the moment it is
+		   fixed sendind le=0 in pin verification apdu */
+		u8 le = 0xff & from->le;
+		res = cwa_compose_tlv(card, 0x97, 1, &le, &ccbuf, &cclen);
+		if (res != SC_SUCCESS) {
+			msg = "Encode APDU compose_tlv(0x97) failed";
+			goto encode_end;
+		}
 	}
 	/* copy current data to apdu buffer (skip header and header padding) */
+	if (cclen < 8) {
+		res = SC_ERROR_INTERNAL;
+		msg = "Incorrect checksum length";
+		goto encode_end;
+	}
 	memcpy(apdubuf, ccbuf + 8, cclen - 8);
 	apdulen = cclen - 8;
 	/* pad again ccbuffer to compute CC */
@@ -1618,22 +1640,15 @@ int cwa_encode_apdu(sc_card_t * card,
 	tmplen = 0;
 	key = sm_session->session_mac;
 
-#if OPENSSL_VERSION_NUMBER >= 0x30000000L
-	if (!legacy_provider) {
-		if (!(legacy_provider = OSSL_PROVIDER_try_load(NULL, "legacy", 1))) {
-			msg = "Failed to load legacy provider";
-			res = SC_ERROR_INTERNAL;
-			goto encode_end;
-		}
-	}
-#endif
-
-	if (EVP_EncryptInit_ex(cctx, EVP_des_ecb(), NULL, key, NULL) != 1 ||
-		EVP_CIPHER_CTX_set_padding(cctx, 0) != 1) {
+	sc_evp_cipher_free(alg);
+	alg = sc_evp_cipher(card->ctx, "DES-ECB");
+	if (!alg ||
+			EVP_EncryptInit_ex(cctx, alg, NULL, key, NULL) != 1 ||
+			EVP_CIPHER_CTX_set_padding(cctx, 0) != 1) {
 		msg = "Error in DES ECB encryption";
 		res = SC_ERROR_INTERNAL;
 		goto encode_end;
-	 }
+	}
 
 	for (i = 0; i < cclen; i += 8) {	/* divide data in 8 byte blocks */
 		/* compute DES */
@@ -1653,10 +1668,14 @@ int cwa_encode_apdu(sc_card_t * card,
 	}
 
 	/* and apply 3DES to result */
-	if (EVP_EncryptInit_ex(cctx, EVP_des_ede_ecb(), NULL, key, NULL) != 1 ||
-		EVP_CIPHER_CTX_set_padding(cctx, 0) != 1 ||
-		EVP_EncryptUpdate(cctx, macbuf, &tmplen, macbuf, 8) != 1 ||
-		EVP_EncryptFinal_ex(cctx, macbuf + tmplen, &tmplen) != 1) {
+	sc_evp_cipher_free(alg);
+	alg = sc_evp_cipher(card->ctx, "DES-EDE-ECB");
+
+	if (!alg ||
+			EVP_EncryptInit_ex(cctx, alg, NULL, key, NULL) != 1 ||
+			EVP_CIPHER_CTX_set_padding(cctx, 0) != 1 ||
+			EVP_EncryptUpdate(cctx, macbuf, &tmplen, macbuf, 8) != 1 ||
+			EVP_EncryptFinal_ex(cctx, macbuf + tmplen, &tmplen) != 1) {
 		msg = "Error in 3DEC ECB encryption";
 		res = SC_ERROR_INTERNAL;
 		goto encode_end;
@@ -1682,15 +1701,16 @@ int cwa_encode_apdu(sc_card_t * card,
 
 err:
 encode_end:
-	if (apdubuf)
-		free(apdubuf);
+	free(apdubuf);
 	if (from->resp != to->resp)
 		free(to->resp);
 encode_end_apdu_valid:
-	if (cctx)
-		EVP_CIPHER_CTX_free(cctx);
-	if (msg)
+	sc_evp_cipher_free(alg);
+	EVP_CIPHER_CTX_free(cctx);
+	if (msg) {
+		sc_log_openssl(ctx);
 		sc_log(ctx, "%s", msg);
+	}
 	free(msgbuf);
 	free(cryptbuf);
 	free(ccbuf);
@@ -1732,11 +1752,14 @@ int cwa_decode_response(sc_card_t * card,
 	struct sm_cwa_session * sm_session = &card->sm_ctx.info.session.cwa;
 
 	EVP_CIPHER_CTX *cctx = NULL;
+	EVP_CIPHER *alg = NULL;
 	unsigned char *key = NULL;
 	int tmplen = 0;
 
-	if ((cctx = EVP_CIPHER_CTX_new()) == NULL)
+	if ((cctx = EVP_CIPHER_CTX_new()) == NULL) {
+		sc_log_openssl(ctx);
 		return SC_ERROR_INTERNAL;
+	}
 
 	/* mandatory check */
 	if (!card || !card->ctx || !provider)
@@ -1860,27 +1883,21 @@ int cwa_decode_response(sc_card_t * card,
 	/* set up key for mac computing */
 	key = sm_session->session_mac;
 
-#if OPENSSL_VERSION_NUMBER >= 0x30000000L
-	if (!legacy_provider) {
-		if (!(legacy_provider = OSSL_PROVIDER_try_load(NULL, "legacy", 1))) {
-			msg = "Failed to load legacy provider";
-			res = SC_ERROR_INTERNAL;
-			goto response_decode_end;
-		}
-	}
-#endif
-
-	if (EVP_EncryptInit_ex(cctx, EVP_des_ecb(), NULL, key, NULL) != 1 ||
-		EVP_CIPHER_CTX_set_padding(cctx, 0) != 1) {
+	alg = sc_evp_cipher(card->ctx, "DES-ECB");
+	if (!alg ||
+			EVP_EncryptInit_ex(cctx, alg, NULL, key, NULL) != 1 ||
+			EVP_CIPHER_CTX_set_padding(cctx, 0) != 1) {
+		sc_log_openssl(ctx);
 		msg = "Error in DES ECB encryption";
 		res = SC_ERROR_INTERNAL;
 		goto response_decode_end;
-	 }
+	}
 
 	memcpy(macbuf, sm_session->ssc, 8);	/* start with computed SSC */
 	for (i = 0; i < cclen; i += 8) {	/* divide data in 8 byte blocks */
 		/* compute DES */
 		if (EVP_EncryptUpdate(cctx, macbuf, &tmplen, macbuf, 8) != 1) {
+			sc_log_openssl(ctx);
 			msg = "Error in DES ECB encryption";
 			res = SC_ERROR_INTERNAL;
 			goto response_decode_end;
@@ -1890,16 +1907,22 @@ int cwa_decode_response(sc_card_t * card,
 			macbuf[j] ^= ccbuf[i + j];
 	}
 	if (EVP_EncryptFinal_ex(cctx, macbuf + tmplen, &tmplen) != 1) {
+		sc_log_openssl(ctx);
 		msg = "Error in DES ECB encryption";
 		res = SC_ERROR_INTERNAL;
 		goto response_decode_end;
 	}
 
 	/* finally apply 3DES to result */
-	if (EVP_EncryptInit_ex(cctx, EVP_des_ede_ecb(), NULL, key, NULL) != 1 ||
-		EVP_CIPHER_CTX_set_padding(cctx, 0) != 1 ||
-		EVP_EncryptUpdate(cctx, macbuf, &tmplen, macbuf, 8) != 1 ||
-		EVP_EncryptFinal_ex(cctx, macbuf + tmplen, &tmplen) != 1) {
+	sc_evp_cipher_free(alg);
+	alg = sc_evp_cipher(card->ctx, "DES-EDE-ECB");
+
+	if (!alg ||
+			EVP_EncryptInit_ex(cctx, alg, NULL, key, NULL) != 1 ||
+			EVP_CIPHER_CTX_set_padding(cctx, 0) != 1 ||
+			EVP_EncryptUpdate(cctx, macbuf, &tmplen, macbuf, 8) != 1 ||
+			EVP_EncryptFinal_ex(cctx, macbuf + tmplen, &tmplen) != 1) {
+		sc_log_openssl(ctx);
 		msg = "Error in 3DEC ECB encryption";
 		res = SC_ERROR_INTERNAL;
 		goto response_decode_end;
@@ -1934,7 +1957,7 @@ int cwa_decode_response(sc_card_t * card,
 	/* if encoded data, decode and store into apdu response */
 	else if (e_tlv->buf) {	/* encoded data */
 		unsigned char iv[8] = { 0, 0, 0, 0, 0, 0, 0, 0 };
-		int dlen = apdu->resplen;
+		int dlen = (int)apdu->resplen;
 		/* check data len */
 		if ((e_tlv->len < 9) || ((e_tlv->len - 1) % 8) != 0) {
 			msg = "Invalid length for Encoded data TLV";
@@ -1952,10 +1975,15 @@ int cwa_decode_response(sc_card_t * card,
 
 		/* decrypt into response buffer
 		 * by using 3DES CBC by mean of kenc and iv={0,...0} */
-		if (EVP_DecryptInit_ex(cctx, EVP_des_ede_cbc(), NULL, key, iv) != 1 ||
-			EVP_CIPHER_CTX_set_padding(cctx, 0) != 1 ||
-			EVP_DecryptUpdate(cctx, apdu->resp, &dlen, &e_tlv->data[1], e_tlv->len - 1) != 1 ||
-			EVP_DecryptFinal_ex(cctx, apdu->resp + dlen, &tmplen) != 1) {
+		sc_evp_cipher_free(alg);
+		alg = sc_evp_cipher(card->ctx, "DES-EDE-CBC");
+
+		if (!alg ||
+				EVP_DecryptInit_ex(cctx, alg, NULL, key, iv) != 1 ||
+				EVP_CIPHER_CTX_set_padding(cctx, 0) != 1 ||
+				EVP_DecryptUpdate(cctx, apdu->resp, &dlen, &e_tlv->data[1], (int)(e_tlv->len - 1)) != 1 ||
+				EVP_DecryptFinal_ex(cctx, apdu->resp + dlen, &tmplen) != 1) {
+			sc_log_openssl(ctx);
 			res = SC_ERROR_INTERNAL;
 			msg = "Can not decrypt 3DES CBC";
 			goto response_decode_end;
@@ -1981,11 +2009,10 @@ int cwa_decode_response(sc_card_t * card,
 	res = SC_SUCCESS;
 
  response_decode_end:
+	sc_evp_cipher_free(alg);
  	EVP_CIPHER_CTX_free(cctx);
-	if (buffer)
-		free(buffer);
-	if (ccbuf)
-		free(ccbuf);
+	free(buffer);
+	free(ccbuf);
 	if (msg) {
 		sc_log(ctx, "%s", msg);
 	} else {
