@@ -15,10 +15,10 @@
  *
  * You should have received a copy of the GNU Lesser General Public
  * License along with this library; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
  */
 
-#if HAVE_CONFIG_H
+#ifdef HAVE_CONFIG_H
 #include "config.h"
 #endif
 
@@ -98,8 +98,8 @@ parse_dir_record(sc_card_t *card, u8 ** buf, size_t *buflen, int rec_nr)
 		sc_bin_to_hex(aid.value, aid.len, aid_str, sizeof(aid_str), 0);
 		blocks = scconf_find_blocks(card->ctx->conf, conf_block, "application", aid_str);
 		if (blocks)   {
-			ignore_app = (blocks[0] && scconf_get_str(blocks[0], "disable", 0));
-                        free(blocks);
+			ignore_app = (blocks[0] && scconf_get_bool(blocks[0], "disable", 0));
+			free(blocks);
 		 }
 
 		if (ignore_app)   {
@@ -227,7 +227,7 @@ int sc_enum_apps(sc_card_t *card)
 		/* Arbitrary set '16' as maximal number of records to check out:
 		 * to avoid endless loop because of some incomplete cards/drivers */
 		for (rec_nr = 1; rec_nr < 16; rec_nr++) {
-			r = sc_read_record(card, rec_nr, buf, sizeof(buf), SC_RECORD_BY_REC_NR);
+			r = sc_read_record(card, rec_nr, 0, buf, sizeof(buf), SC_RECORD_BY_REC_NR);
 			if (r == SC_ERROR_RECORD_NOT_FOUND)
 				break;
 			LOG_TEST_RET(ctx, r, "read_record() failed");
@@ -361,7 +361,7 @@ static int update_single_record(sc_card_t *card, sc_app_info_t *app)
 	if (r)
 		return r;
 	if (app->rec_nr > 0)
-		r = sc_update_record(card, (unsigned int)app->rec_nr, rec, rec_size, SC_RECORD_BY_REC_NR);
+		r = sc_update_record(card, (unsigned int)app->rec_nr, 0, rec, rec_size, SC_RECORD_BY_REC_NR);
 	else if (app->rec_nr == 0) {
 		/* create new record entry */
 		r = sc_append_record(card, rec, rec_size, 0);
@@ -375,7 +375,7 @@ static int update_single_record(sc_card_t *card, sc_app_info_t *app)
 				if (card->app[i]->rec_nr > rec_nr)
 					rec_nr = card->app[i]->rec_nr;
 			rec_nr++;
-			r = sc_update_record(card, (unsigned int)rec_nr, rec, rec_size, SC_RECORD_BY_REC_NR);
+			r = sc_update_record(card, (unsigned int)rec_nr, 0, rec, rec_size, SC_RECORD_BY_REC_NR);
 		}
 	} else {
 		sc_log(card->ctx, "invalid record number\n");
