@@ -17,10 +17,10 @@
  *
  * You should have received a copy of the GNU Lesser General Public
  * License along with this library; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
  */
 
-#if HAVE_CONFIG_H
+#ifdef HAVE_CONFIG_H
 #include "config.h"
 #endif
 
@@ -64,7 +64,7 @@ gp_select_aid(struct sc_card *card, const struct sc_aid *aid)
 	if (rv < 0)
 		return rv;
 
-	return apdu.resplen;
+	return (int)apdu.resplen;
 }
 
 /* Select the Open Platform Card Manager */
@@ -117,5 +117,5 @@ gp_get_cplc_data(struct sc_card *card, global_platform_cplc_data_t *cplc_data)
 	if ((size_t)apdu.resplen < sizeof(global_platform_cplc_data_t)) {
 		LOG_FUNC_RETURN(card->ctx, SC_ERROR_CORRUPTED_DATA);
 	}
-	LOG_FUNC_RETURN(card->ctx, apdu.resplen);
+	LOG_FUNC_RETURN(card->ctx, (int)apdu.resplen);
 }
