@@ -15,7 +15,7 @@
  *
  * You should have received a copy of the GNU Lesser General Public
  * License along with this library; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
  */
 
 #include "config.h"
@@ -30,8 +30,8 @@
 #include "iso7816.h"
 #include "sm/sm-iso.h"
 
-
-static void fixup_transceive_length(const struct sc_card *card,
+void
+iso7816_fixup_transceive_length(const struct sc_card *card,
 		struct sc_apdu *apdu)
 {
 	if (card == NULL || apdu == NULL) {
@@ -117,12 +117,12 @@ iso7816_check_sw(struct sc_card *card, unsigned int sw1, unsigned int sw2)
 		sc_log(card->ctx, "Wrong length; correct length is %d", sw2);
 		return SC_ERROR_WRONG_LENGTH;
 	}
-	if (sw1 == 0x90)
+	if (sw1 == 0x90 && sw2 == 0x00)
 		return SC_SUCCESS;
-        if (sw1 == 0x63U && (sw2 & ~0x0fU) == 0xc0U ) {
-             sc_log(card->ctx, "PIN not verified (remaining tries: %d)", (sw2 & 0x0f));
-             return SC_ERROR_PIN_CODE_INCORRECT;
-        }
+	if (sw1 == 0x63U && (sw2 & ~0x0fU) == 0xc0U) {
+		sc_log(card->ctx, "PIN not verified (remaining tries: %d)", (sw2 & 0x0f));
+		return SC_ERROR_PIN_CODE_INCORRECT;
+	}
 	for (i = 0; i < err_count; i++)   {
 		if (iso7816_errors[i].SWs == ((sw1 << 8) | sw2)) {
 			sc_log(card->ctx, "%s", iso7816_errors[i].errorstr);
@@ -136,7 +136,7 @@ iso7816_check_sw(struct sc_card *card, unsigned int sw1, unsigned int sw2)
 
 
 static int
-iso7816_read_binary(struct sc_card *card, unsigned int idx, u8 *buf, size_t count, unsigned long flags)
+iso7816_read_binary(struct sc_card *card, unsigned int idx, u8 *buf, size_t count, unsigned long *flags)
 {
 	struct sc_context *ctx = card->ctx;
 	struct sc_apdu apdu;
@@ -152,44 +152,131 @@ iso7816_read_binary(struct sc_card *card, unsigned int idx, u8 *buf, size_t coun
 	apdu.resplen = count;
 	apdu.resp = buf;
 
-	fixup_transceive_length(card, &apdu);
+	iso7816_fixup_transceive_length(card, &apdu);
 	r = sc_transmit_apdu(card, &apdu);
 	LOG_TEST_RET(ctx, r, "APDU transmit failed");
 
 	r = sc_check_sw(card, apdu.sw1, apdu.sw2);
 	if (r == SC_ERROR_FILE_END_REACHED)
-		LOG_FUNC_RETURN(ctx, apdu.resplen);
+		LOG_FUNC_RETURN(ctx, (int)apdu.resplen);
 	LOG_TEST_RET(ctx, r, "Check SW error");
 
-	LOG_FUNC_RETURN(ctx, apdu.resplen);
+	LOG_FUNC_RETURN(ctx, (int)apdu.resplen);
 }
 
 
+const struct sc_asn1_entry c_asn1_do_data[] = {
+	{ "Offset Data Object", SC_ASN1_OCTET_STRING, SC_ASN1_APP|0x14, SC_ASN1_OPTIONAL, NULL, NULL },
+	{ "Discretionary Data Object", SC_ASN1_OCTET_STRING, SC_ASN1_APP|0x13, SC_ASN1_ALLOC|SC_ASN1_UNSIGNED|SC_ASN1_OPTIONAL, NULL, NULL },
+	{ NULL, 0, 0, 0, NULL, NULL }
+};
+
+int encode_do_data(struct sc_context *ctx,
+		unsigned int idx, const unsigned char *data, size_t data_len,
+		u8 **out, size_t *outlen)
+{
+	unsigned char offset_buffer[2];
+	size_t offset_buffer_len = sizeof offset_buffer;
+	struct sc_asn1_entry asn1_do_data[sizeof c_asn1_do_data / sizeof *c_asn1_do_data];
+	sc_copy_asn1_entry(c_asn1_do_data, asn1_do_data);
+
+	if (idx > 0xFFFF)
+		LOG_TEST_RET(ctx, SC_ERROR_INTERNAL, "Offset beyond 0xFFFF not supported");
+	offset_buffer[0] = (u8) (idx >> 8);
+	offset_buffer[1] = (u8) (idx & 0x00FF);
+	sc_format_asn1_entry(asn1_do_data + 0, offset_buffer, &offset_buffer_len, 1);
+
+	if (data && data_len) {
+		sc_format_asn1_entry(asn1_do_data + 1, (void *) &data, &data_len, 1);
+	} else {
+		sc_format_asn1_entry(asn1_do_data + 1, NULL, NULL, 0);
+	}
+
+	LOG_TEST_RET(ctx,
+			sc_asn1_encode(ctx, asn1_do_data, out, outlen),
+			"sc_asn1_encode() failed");
+
+	return SC_SUCCESS;
+}
+
+int decode_do_data(struct sc_context *ctx,
+		const unsigned char *encoded_data, size_t encoded_data_len,
+		u8 **out, size_t *outlen)
+{
+	struct sc_asn1_entry asn1_do_data[sizeof c_asn1_do_data / sizeof *c_asn1_do_data];
+	sc_copy_asn1_entry(c_asn1_do_data, asn1_do_data);
+
+	sc_format_asn1_entry(asn1_do_data + 0, NULL, NULL, 0);
+	sc_format_asn1_entry(asn1_do_data + 1, out, outlen, 0);
+
+	LOG_TEST_RET(ctx,
+			sc_asn1_decode(ctx, asn1_do_data, encoded_data, encoded_data_len, NULL, NULL),
+			"sc_asn1_decode() failed");
+
+	if (!(asn1_do_data[1].flags & SC_ASN1_PRESENT))
+		return SC_ERROR_INVALID_ASN1_OBJECT;
+
+	return SC_SUCCESS;
+}
+
 static int
-iso7816_read_record(struct sc_card *card,
-		unsigned int rec_nr, u8 *buf, size_t count, unsigned long flags)
+iso7816_read_record(struct sc_card *card, unsigned int rec_nr, unsigned int idx,
+		u8 *buf, size_t count, unsigned long flags)
 {
 	struct sc_apdu apdu;
 	int r;
+	/* XXX maybe use some bigger buffer */
+	unsigned char resp[SC_MAX_APDU_RESP_SIZE];
+	unsigned char *encoded_data = NULL, *decoded_data = NULL;
+	size_t encoded_data_len = 0, decoded_data_len = 0;
 
 	if (rec_nr > 0xFF)
 		LOG_FUNC_RETURN(card->ctx, SC_ERROR_INVALID_ARGUMENTS);
 
-	sc_format_apdu(card, &apdu, SC_APDU_CASE_2, 0xB2, rec_nr, 0);
-	apdu.le = count;
-	apdu.resplen = count;
-	apdu.resp = buf;
+	if (idx == 0) {
+		sc_format_apdu(card, &apdu, SC_APDU_CASE_2, 0xB2, rec_nr, 0);
+		apdu.le = count;
+		apdu.resplen = count;
+		apdu.resp = buf;
+	} else {
+		r = encode_do_data(card->ctx, idx, NULL, 0, &encoded_data, &encoded_data_len);
+		LOG_TEST_GOTO_ERR(card->ctx, r, "Could not encode data objects");
+
+		sc_format_apdu(card, &apdu, SC_APDU_CASE_4, 0xB3, rec_nr, 0);
+		apdu.lc = encoded_data_len;
+		apdu.datalen = encoded_data_len;
+		apdu.data = encoded_data;
+		apdu.le = sizeof resp;
+		apdu.resplen = sizeof resp;
+		apdu.resp = resp;
+	}
 	apdu.p2 = (flags & SC_RECORD_EF_ID_MASK) << 3;
 	if (flags & SC_RECORD_BY_REC_NR)
 		apdu.p2 |= 0x04;
 
-	fixup_transceive_length(card, &apdu);
+	iso7816_fixup_transceive_length(card, &apdu);
 	r = sc_transmit_apdu(card, &apdu);
-	LOG_TEST_RET(card->ctx, r, "APDU transmit failed");
-	if (apdu.resplen == 0)
-		LOG_FUNC_RETURN(card->ctx, sc_check_sw(card, apdu.sw1, apdu.sw2));
+	LOG_TEST_GOTO_ERR(card->ctx, r, "APDU transmit failed");
+	r = sc_check_sw(card, apdu.sw1, apdu.sw2);
+	LOG_TEST_GOTO_ERR(card->ctx, r, "Card returned error");
 
-	LOG_FUNC_RETURN(card->ctx, apdu.resplen);
+	if (idx == 0) {
+		r = (int)apdu.resplen;
+	} else {
+		r = decode_do_data(card->ctx, apdu.resp, apdu.resplen,
+				&decoded_data, &decoded_data_len);
+		LOG_TEST_GOTO_ERR(card->ctx, r, "Could not decode data objects");
+		if (decoded_data_len <= count) {
+			count = decoded_data_len;
+		}
+		memcpy(buf, decoded_data, count);
+		r = (int)count;
+	}
+
+err:
+	free(encoded_data);
+	free(decoded_data);
+	LOG_FUNC_RETURN(card->ctx, r);
 }
 
 
@@ -208,13 +295,13 @@ iso7816_write_record(struct sc_card *card, unsigned int rec_nr,
 	if (flags & SC_RECORD_BY_REC_NR)
 		apdu.p2 |= 0x04;
 
-	fixup_transceive_length(card, &apdu);
+	iso7816_fixup_transceive_length(card, &apdu);
 	r = sc_transmit_apdu(card, &apdu);
 	LOG_TEST_RET(card->ctx, r, "APDU transmit failed");
 	r = sc_check_sw(card, apdu.sw1, apdu.sw2);
 	LOG_TEST_RET(card->ctx, r, "Card returned error");
 
-	LOG_FUNC_RETURN(card->ctx, count);
+	LOG_FUNC_RETURN(card->ctx, (int)count);
 }
 
 
@@ -231,41 +318,58 @@ iso7816_append_record(struct sc_card *card,
 	apdu.data = buf;
 	apdu.p2 = (flags & SC_RECORD_EF_ID_MASK) << 3;
 
-	fixup_transceive_length(card, &apdu);
+	iso7816_fixup_transceive_length(card, &apdu);
 	r = sc_transmit_apdu(card, &apdu);
 	LOG_TEST_RET(card->ctx, r, "APDU transmit failed");
 	r = sc_check_sw(card, apdu.sw1, apdu.sw2);
 	LOG_TEST_RET(card->ctx, r, "Card returned error");
 
-	LOG_FUNC_RETURN(card->ctx, count);
+	LOG_FUNC_RETURN(card->ctx, (int)count);
 }
 
 
 static int
-iso7816_update_record(struct sc_card *card, unsigned int rec_nr,
+iso7816_update_record(struct sc_card *card, unsigned int rec_nr, unsigned int idx,
 		const u8 *buf, size_t count, unsigned long flags)
 {
 	struct sc_apdu apdu;
 	int r;
+	unsigned char *encoded_data = NULL;
+	size_t encoded_data_len = 0;
 
-	sc_format_apdu(card, &apdu, SC_APDU_CASE_3, 0xDC, rec_nr, 0);
-	apdu.lc = count;
-	apdu.datalen = count;
-	apdu.data = buf;
+	if (rec_nr > 0xFF)
+		LOG_FUNC_RETURN(card->ctx, SC_ERROR_INVALID_ARGUMENTS);
+
+	if (idx == 0) {
+		sc_format_apdu(card, &apdu, SC_APDU_CASE_3, 0xDC, rec_nr, 0);
+		apdu.lc = count;
+		apdu.datalen = count;
+		apdu.data = buf;
+	} else {
+		r = encode_do_data(card->ctx, idx, buf, count, &encoded_data, &encoded_data_len);
+		LOG_TEST_GOTO_ERR(card->ctx, r, "Could not encode data objects");
+
+		sc_format_apdu(card, &apdu, SC_APDU_CASE_3, 0xDD, rec_nr, 0);
+		apdu.lc = encoded_data_len;
+		apdu.datalen = encoded_data_len;
+		apdu.data = encoded_data;
+	}
 	apdu.p2 = (flags & SC_RECORD_EF_ID_MASK) << 3;
 	if (flags & SC_RECORD_BY_REC_NR)
 		apdu.p2 |= 0x04;
 
-	fixup_transceive_length(card, &apdu);
+	iso7816_fixup_transceive_length(card, &apdu);
 	r = sc_transmit_apdu(card, &apdu);
-	LOG_TEST_RET(card->ctx, r, "APDU transmit failed");
+	LOG_TEST_GOTO_ERR(card->ctx, r, "APDU transmit failed");
 	r = sc_check_sw(card, apdu.sw1, apdu.sw2);
-	LOG_TEST_RET(card->ctx, r, "Card returned error");
+	LOG_TEST_GOTO_ERR(card->ctx, r, "Card returned error");
+	r = (int)count;
 
-	LOG_FUNC_RETURN(card->ctx, count);
+err:
+	free(encoded_data);
+	LOG_FUNC_RETURN(card->ctx, r);
 }
 
-
 static int
 iso7816_write_binary(struct sc_card *card,
 		unsigned int idx, const u8 *buf, size_t count, unsigned long flags)
@@ -284,13 +388,13 @@ iso7816_write_binary(struct sc_card *card,
 	apdu.datalen = count;
 	apdu.data = buf;
 
-	fixup_transceive_length(card, &apdu);
+	iso7816_fixup_transceive_length(card, &apdu);
 	r = sc_transmit_apdu(card, &apdu);
 	LOG_TEST_RET(card->ctx, r, "APDU transmit failed");
 	r = sc_check_sw(card, apdu.sw1, apdu.sw2);
 	LOG_TEST_RET(card->ctx, r, "Card returned error");
 
-	LOG_FUNC_RETURN(card->ctx, count);
+	LOG_FUNC_RETURN(card->ctx, (int)count);
 }
 
 
@@ -311,13 +415,13 @@ iso7816_update_binary(struct sc_card *card,
 	apdu.datalen = count;
 	apdu.data = buf;
 
-	fixup_transceive_length(card, &apdu);
+	iso7816_fixup_transceive_length(card, &apdu);
 	r = sc_transmit_apdu(card, &apdu);
 	LOG_TEST_RET(card->ctx, r, "APDU transmit failed");
 	r = sc_check_sw(card, apdu.sw1, apdu.sw2);
 	LOG_TEST_RET(card->ctx, r, "Card returned error");
 
-	LOG_FUNC_RETURN(card->ctx, count);
+	LOG_FUNC_RETURN(card->ctx, (int)count);
 }
 
 
@@ -329,7 +433,6 @@ iso7816_process_fci(struct sc_card *card, struct sc_file *file,
 	const unsigned char *p, *end;
 	unsigned int cla = 0, tag = 0;
 	size_t length;
-	int size;
 
 	file->status = SC_FILE_STATUS_UNKNOWN;
 
@@ -350,11 +453,22 @@ iso7816_process_fci(struct sc_card *card, struct sc_file *file,
 				/* fall through */
 			case 0x80:
 				/* determine the file size */
-				if (sc_asn1_decode_integer(p, length, &size, 0) == 0 && size >= 0) {
-					file->size = size;
-					sc_log(ctx, "  bytes in file: %"SC_FORMAT_LEN_SIZE_T"u",
-							file->size);
+				file->size = 0;
+				if (p && length <= sizeof(size_t)) {
+					size_t size = 0, i;
+					for (i = 0; i < length; i++) {
+						size <<= 8;
+						size |= p[i];
+					}
+					if (size > MAX_FILE_SIZE) {
+						file->size = MAX_FILE_SIZE;
+						sc_log(ctx, "  file size truncated, encoded length: %"SC_FORMAT_LEN_SIZE_T"u", size);
+					} else {
+						file->size = size;
+					}
 				}
+
+				sc_log(ctx, "  bytes in file: %"SC_FORMAT_LEN_SIZE_T"u", file->size);
 				break;
 
 			case 0x82:
@@ -519,7 +633,8 @@ iso7816_select_file(struct sc_card *card, const struct sc_path *in_path, struct
 	struct sc_apdu apdu;
 	unsigned char buf[SC_MAX_APDU_BUFFER_SIZE];
 	unsigned char pathbuf[SC_MAX_PATH_SIZE], *path = pathbuf;
-	int r, pathlen, pathtype;
+	int r, pathtype;
+	size_t pathlen;
 	int select_mf = 0;
 	struct sc_file *file = NULL;
 	const u8 *buffer;
@@ -688,7 +803,7 @@ iso7816_get_challenge(struct sc_card *card, u8 *rnd, size_t len)
 	if (len < apdu.resplen) {
 		return (int) len;
 	}
-   
+
 	return (int) apdu.resplen;
 }
 
@@ -786,7 +901,7 @@ iso7816_create_file(struct sc_card *card, sc_file_t *file)
 static int
 iso7816_get_response(struct sc_card *card, size_t *count, u8 *buf)
 {
-	struct sc_apdu apdu;
+	struct sc_apdu apdu = {0};
 	int r;
 	size_t rlen;
 
@@ -888,7 +1003,7 @@ iso7816_set_security_env(struct sc_card *card,
 		*p++ = env->algorithm_ref & 0xFF;
 	}
 	if (env->flags & SC_SEC_ENV_FILE_REF_PRESENT) {
-		if (env->file_ref.len > 0xFF)
+		if (env->file_ref.len > SC_MAX_PATH_SIZE)
 			return SC_ERROR_INVALID_ARGUMENTS;
 		if (sizeof(sbuf) - (p - sbuf) < env->file_ref.len + 2)
 			return SC_ERROR_OFFSET_TOO_LARGE;
@@ -906,13 +1021,13 @@ iso7816_set_security_env(struct sc_card *card,
 			*p++ = 0x83;
 		else
 			*p++ = 0x84;
-		if (env->key_ref_len > 0xFF)
+		if (env->key_ref_len > SC_MAX_KEYREF_SIZE)
 			return SC_ERROR_INVALID_ARGUMENTS;
 		*p++ = env->key_ref_len & 0xFF;
 		memcpy(p, env->key_ref, env->key_ref_len);
 		p += env->key_ref_len;
 	}
-	r = p - sbuf;
+	r = (int)(p - sbuf);
 	apdu.lc = r;
 	apdu.datalen = r;
 	apdu.data = sbuf;
@@ -999,11 +1114,11 @@ iso7816_compute_signature(struct sc_card *card,
 	apdu.lc = datalen;
 	apdu.datalen = datalen;
 
-	fixup_transceive_length(card, &apdu);
+	iso7816_fixup_transceive_length(card, &apdu);
 	r = sc_transmit_apdu(card, &apdu);
 	LOG_TEST_RET(card->ctx, r, "APDU transmit failed");
 	if (apdu.sw1 == 0x90 && apdu.sw2 == 0x00)
-		LOG_FUNC_RETURN(card->ctx, apdu.resplen);
+		LOG_FUNC_RETURN(card->ctx, (int)apdu.resplen);
 
 	r = sc_check_sw(card, apdu.sw1, apdu.sw2);
 	LOG_TEST_RET(card->ctx, r, "Card returned error");
@@ -1047,14 +1162,14 @@ iso7816_decipher(struct sc_card *card,
 	apdu.lc = crgram_len + 1;
 	apdu.datalen = crgram_len + 1;
 
-	fixup_transceive_length(card, &apdu);
+	iso7816_fixup_transceive_length(card, &apdu);
 	r = sc_transmit_apdu(card, &apdu);
 	sc_mem_clear(sbuf, crgram_len + 1);
 	free(sbuf);
 	LOG_TEST_RET(card->ctx, r, "APDU transmit failed");
 
 	if (apdu.sw1 == 0x90 && apdu.sw2 == 0x00)
-		LOG_FUNC_RETURN(card->ctx, apdu.resplen);
+		LOG_FUNC_RETURN(card->ctx, (int)apdu.resplen);
 	else
 		LOG_FUNC_RETURN(card->ctx, sc_check_sw(card, apdu.sw1, apdu.sw2));
 }
@@ -1087,6 +1202,9 @@ iso7816_build_pin_apdu(struct sc_card *card, struct sc_apdu *apdu,
 	switch (data->cmd) {
 	case SC_PIN_CMD_VERIFY:
 		ins = 0x20;
+		/* detect overloaded APDU with SC_PIN_CMD_GET_INFO */
+		if (data->pin1.len == 0 && !use_pin_pad)
+			return SC_ERROR_INVALID_PIN_LENGTH;
 		if ((r = sc_build_pin(buf, buf_len, &data->pin1, pad)) < 0)
 			return r;
 		len = r;
@@ -1133,6 +1251,10 @@ iso7816_build_pin_apdu(struct sc_card *card, struct sc_apdu *apdu,
 		} else {
 			p1 |= 0x01;
 		}
+		if (p1 == 0x03) {
+			/* No data to send or to receive */
+			cse = SC_APDU_CASE_1;
+		}
 		break;
 	case SC_PIN_CMD_GET_INFO:
 		ins = 0x20;
@@ -1266,7 +1388,7 @@ static int iso7816_get_data(struct sc_card *card, unsigned int tag,  u8 *buf, si
 	if (apdu.resplen > len)
 		r = SC_ERROR_WRONG_LENGTH;
 	else
-		r = apdu.resplen;
+		r = (int)apdu.resplen;
 
 	LOG_FUNC_RETURN(card->ctx, r);
 }
@@ -1349,7 +1471,7 @@ int iso7816_read_binary_sfid(sc_card_t *card, unsigned char sfid,
 		u8 **ef, size_t *ef_len)
 {
 	int r;
-	size_t read = MAX_SM_APDU_RESP_SIZE;
+	size_t read;
 	sc_apdu_t apdu;
 	u8 *p;
 
@@ -1359,13 +1481,9 @@ int iso7816_read_binary_sfid(sc_card_t *card, unsigned char sfid,
 	}
 	*ef_len = 0;
 
-#if MAX_SM_APDU_RESP_SIZE > (0xff+1)
-	sc_format_apdu(card, &apdu, SC_APDU_CASE_2_EXT,
+	read = card->max_recv_size;
+	sc_format_apdu(card, &apdu, SC_APDU_CASE_2,
 			ISO_READ_BINARY, ISO_P1_FLAG_SFID|sfid, 0);
-#else
-	sc_format_apdu(card, &apdu, SC_APDU_CASE_2_SHORT,
-			ISO_READ_BINARY, ISO_P1_FLAG_SFID|sfid, 0);
-#endif
 	p = realloc(*ef, read);
 	if (!p) {
 		r = SC_ERROR_OUT_OF_MEMORY;
@@ -1383,7 +1501,7 @@ int iso7816_read_binary_sfid(sc_card_t *card, unsigned char sfid,
 	if (r < 0 && r != SC_ERROR_FILE_END_REACHED)
 		goto err;
 	/* emulate the behaviour of iso7816_read_binary */
-	r = apdu.resplen;
+	r = (int)apdu.resplen;
 
 	while(1) {
 		if (r >= 0 && ((size_t) r) != read) {
@@ -1407,11 +1525,10 @@ int iso7816_read_binary_sfid(sc_card_t *card, unsigned char sfid,
 		}
 		*ef = p;
 
-		r = iso7816_read_binary(card, *ef_len,
-				*ef + *ef_len, read, 0);
+		r = iso7816_read_binary(card, (unsigned)*ef_len, *ef + *ef_len, read, 0);
 	}
 
-	r = *ef_len;
+	r = (int)*ef_len;
 
 err:
 	return r;
@@ -1422,34 +1539,18 @@ int iso7816_write_binary_sfid(sc_card_t *card, unsigned char sfid,
 		u8 *ef, size_t ef_len)
 {
 	int r;
-	size_t write = MAX_SM_APDU_DATA_SIZE, wrote = 0;
+	size_t write, wrote = 0;
 	sc_apdu_t apdu;
-#ifdef ENABLE_SM
-	struct iso_sm_ctx *iso_sm_ctx;
-#endif
 
 	if (!card) {
 		r = SC_ERROR_INVALID_ARGUMENTS;
 		goto err;
 	}
 
-#ifdef ENABLE_SM
-	iso_sm_ctx = card->sm_ctx.info.cmd_data;
-	if (write > SC_MAX_APDU_BUFFER_SIZE-2
-			|| (card->sm_ctx.sm_mode == SM_MODE_TRANSMIT
-				&& write > (((SC_MAX_APDU_BUFFER_SIZE-2
-					/* for encrypted APDUs we usually get authenticated status
-					 * bytes (4B), a MAC (11B) and a cryptogram with padding
-					 * indicator (3B without data).  The cryptogram is always
-					 * padded to the block size. */
-					-18) / iso_sm_ctx->block_length)
-					* iso_sm_ctx->block_length - 1)))
-		sc_format_apdu(card, &apdu, SC_APDU_CASE_3_EXT,
-				ISO_WRITE_BINARY, ISO_P1_FLAG_SFID|sfid, 0);
-	else
-#endif
-		sc_format_apdu(card, &apdu, SC_APDU_CASE_3_SHORT,
-				ISO_WRITE_BINARY, ISO_P1_FLAG_SFID|sfid, 0);
+	write = card->max_send_size;
+
+	sc_format_apdu(card, &apdu, SC_APDU_CASE_3,
+			ISO_WRITE_BINARY, ISO_P1_FLAG_SFID|sfid, 0);
 
 	if (write > ef_len) {
 		apdu.datalen = ef_len;
@@ -1464,7 +1565,7 @@ int iso7816_write_binary_sfid(sc_card_t *card, unsigned char sfid,
 	r = sc_transmit_apdu(card, &apdu);
 	/* emulate the behaviour of sc_write_binary */
 	if (r >= 0)
-		r = apdu.datalen;
+		r = (int)apdu.datalen;
 
 	while (1) {
 		if (r < 0 || ((size_t) r) > ef_len) {
@@ -1478,10 +1579,10 @@ int iso7816_write_binary_sfid(sc_card_t *card, unsigned char sfid,
 		if (wrote >= ef_len)
 			break;
 
-		r = sc_write_binary(card, wrote, ef, write, 0);
+		r = sc_write_binary(card, (unsigned)wrote, ef, write, 0);
 	}
 
-	r = wrote;
+	r = (int)wrote;
 
 err:
 	return r;
@@ -1534,7 +1635,7 @@ int iso7816_update_binary_sfid(sc_card_t *card, unsigned char sfid,
 	r = sc_transmit_apdu(card, &apdu);
 	/* emulate the behaviour of sc_write_binary */
 	if (r >= 0)
-		r = apdu.datalen;
+		r = (int)apdu.datalen;
 
 	while (1) {
 		if (r < 0 || ((size_t) r) > ef_len) {
@@ -1548,10 +1649,10 @@ int iso7816_update_binary_sfid(sc_card_t *card, unsigned char sfid,
 		if (wrote >= ef_len)
 			break;
 
-		r = sc_update_binary(card, wrote, ef, write, 0);
+		r = sc_update_binary(card, (unsigned)wrote, ef, write, 0);
 	}
 
-	r = wrote;
+	r = (int)wrote;
 
 err:
 	return r;
