@@ -22,7 +22,7 @@
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
  */
 
-#if HAVE_CONFIG_H
+#ifdef HAVE_CONFIG_H
 #include "config.h"
 #endif
 
@@ -47,7 +47,7 @@ static int set_sec_env(sc_card_t * card, const sc_security_env_t *env,
 	sc_security_env_t tenv = *env;
 	if (tenv.operation == SC_SEC_OPERATION_SIGN)
 		tenv.operation = SC_SEC_OPERATION_DECIPHER;
-	
+
 	if ((r =
 	     card->ops->restore_security_env(card, 0x40)) == SC_SUCCESS)
 		return set_security_env(card, &tenv, se_num);
@@ -71,33 +71,33 @@ static int sc_pkcs15emu_add_pin(sc_pkcs15_card_t *p15card,
                 unsigned int max_length,
                 int flags, int tries_left, const char pad_char, int obj_flags)
 {
-        sc_pkcs15_auth_info_t info;
+	sc_pkcs15_auth_info_t info;
 	sc_pkcs15_object_t   obj;
 
 	memset(&info, 0, sizeof(info));
 	memset(&obj,  0, sizeof(obj));
 
-        info.auth_id           = *id;
+	info.auth_id           = *id;
 	info.auth_type = SC_PKCS15_PIN_AUTH_TYPE_PIN;
-        info.attrs.pin.min_length        = min_length;
-        info.attrs.pin.max_length        = max_length;
-        info.attrs.pin.stored_length     = max_length;
-        info.attrs.pin.type              = type;
-        info.attrs.pin.reference         = ref;
-        info.attrs.pin.flags             = flags;
-        info.attrs.pin.pad_char          = pad_char;
-        info.tries_left        = tries_left;
+	info.attrs.pin.min_length        = min_length;
+	info.attrs.pin.max_length        = max_length;
+	info.attrs.pin.stored_length     = max_length;
+	info.attrs.pin.type              = type;
+	info.attrs.pin.reference         = ref;
+	info.attrs.pin.flags             = flags;
+	info.attrs.pin.pad_char          = pad_char;
+	info.tries_left        = tries_left;
 	info.logged_in = SC_PIN_STATE_UNKNOWN;
 
-        if (path)
-                info.path = *path;
-        if (type == SC_PKCS15_PIN_TYPE_BCD)
-                info.attrs.pin.stored_length /= 2;
+	if (path)
+		info.path = *path;
+	if (type == SC_PKCS15_PIN_TYPE_BCD)
+		info.attrs.pin.stored_length /= 2;
 
 	strlcpy(obj.label, label, sizeof(obj.label));
 	obj.flags = obj_flags;
 
-        return sc_pkcs15emu_add_pin_obj(p15card, &obj, &info);
+	return sc_pkcs15emu_add_pin_obj(p15card, &obj, &info);
 }
 
 static int sc_pkcs15emu_add_prkey(sc_pkcs15_card_t *p15card,
@@ -142,33 +142,33 @@ static int sc_pkcs15emu_actalis_init(sc_pkcs15_card_t * p15card)
 
 #ifdef ENABLE_ZLIB
 	int use_file_cache_backup = p15card->opts.use_file_cache;
-	
+
 	int i = 0, j = 0;
 	const char *certLabel[] = {
 		"User Non-repudiation Certificate",	/* "User Non-repudiation Certificate" */
 		"TSA Certificate",
 		"CA Certificate"
-	};	
+	};
 	const char *certPath[] =
 	    { "3F00300060006002", "3F00300060006003", "3F00300060006004" };
 #endif
 
 	const char *keyPath = "3F00300040000008";
 	const char *pinDfName = "05040200";
-	
+
 	/* const int prkey_usage = SC_PKCS15_PRKEY_USAGE_NONREPUDIATION; */
 	const int authprkey_usage = SC_PKCS15_PRKEY_USAGE_SIGN
 				| SC_PKCS15_PRKEY_USAGE_SIGNRECOVER
 				| SC_PKCS15_PRKEY_USAGE_ENCRYPT
 				| SC_PKCS15_PRKEY_USAGE_DECRYPT;
-	    
+
 	const char *authPIN = "Authentication PIN";
 	/* const char *nonrepPIN = "Non-repudiation PIN"; */
 
 	const char *authPRKEY = "Authentication Key";
 	/* const char *nonrepPRKEY = "Non repudiation Key"; */
 
-	p15card->opts.use_file_cache = SC_PKCS15_OPTS_CACHE_ALL_FILES;	
+	p15card->opts.use_file_cache = SC_PKCS15_OPTS_CACHE_ALL_FILES;
 
 	/* Get Serial number */
 	sc_format_path("3F0030000001", &path);
@@ -195,9 +195,9 @@ static int sc_pkcs15emu_actalis_init(sc_pkcs15_card_t * p15card)
 	serial[8] = '\0';
 
 	/* Controllo che il serial number inizi per "H" */
-	if( serial[0] != 'H' ) 
+	if( serial[0] != 'H' )
 		return SC_ERROR_WRONG_CARD;
-			
+
 	set_string(&p15card->tokeninfo->label, "Actalis");
 	set_string(&p15card->tokeninfo->manufacturer_id, "Actalis");
 	set_string(&p15card->tokeninfo->serial_number, (char *)serial);
@@ -237,7 +237,7 @@ static int sc_pkcs15emu_actalis_init(sc_pkcs15_card_t * p15card)
 				continue;
 			}
 			cpath.index = 0;
-			cpath.count = len;
+			cpath.count = (int)len;
 
 			sc_pkcs15_cache_file(p15card, &cpath, cert, len);
 			id.value[0] = j + 1;
@@ -263,15 +263,15 @@ static int sc_pkcs15emu_actalis_init(sc_pkcs15_card_t * p15card)
 		}
 	}
 #endif
-	
+
 	/* adding PINs & private keys */
 	flags = SC_PKCS15_PIN_FLAG_CASE_SENSITIVE |
 	    SC_PKCS15_PIN_FLAG_INITIALIZED |
-	    SC_PKCS15_PIN_FLAG_NEEDS_PADDING;	
-	
+	    SC_PKCS15_PIN_FLAG_NEEDS_PADDING;
+
 	sc_format_path(pinDfName, &path);
 	path.type = SC_PATH_TYPE_DF_NAME;
-	
+
 	id.value[0] = 1;
 	id.len = 1;
 	sc_pkcs15emu_add_pin(p15card, &id,
@@ -280,7 +280,7 @@ static int sc_pkcs15emu_actalis_init(sc_pkcs15_card_t * p15card)
 			     5, 8, flags, 3, 0,
 			     SC_PKCS15_CO_FLAG_MODIFIABLE |
 			     SC_PKCS15_CO_FLAG_PRIVATE);
-	
+
 	sc_format_path(keyPath, &path);
 	id.value[0] = 1;
 	id.len = 1;
@@ -293,7 +293,7 @@ static int sc_pkcs15emu_actalis_init(sc_pkcs15_card_t * p15card)
 		       &path, 0x08,
 		       &auth_id,
 		       SC_PKCS15_CO_FLAG_PRIVATE);
-	
+
 	/* return to MF */
 	sc_format_path("3F00", &path);
 	sc_select_file(card, &path, NULL);
@@ -304,7 +304,7 @@ static int sc_pkcs15emu_actalis_init(sc_pkcs15_card_t * p15card)
 		card->ops->set_security_env  = set_sec_env;
 		card->ops->compute_signature = do_sign;
 	}
-	
+
 	return SC_SUCCESS;
 
 }
@@ -316,7 +316,7 @@ static int actalis_detect_card(sc_pkcs15_card_t * p15card)
 	/* check if we have the correct card OS */
 	if (strcmp(card->name, "CardOS M4"))
 		return SC_ERROR_WRONG_CARD;
-	
+
 	return SC_SUCCESS;
 }
 
