@@ -24,10 +24,10 @@
  *
  * You should have received a copy of the GNU Lesser General Public
  * License along with this library; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
  */
 
-#if HAVE_CONFIG_H
+#ifdef HAVE_CONFIG_H
 #include "config.h"
 #endif
 
@@ -258,9 +258,9 @@ static int sc_pkcs15emu_cac_init(sc_pkcs15_card_t *p15card)
 			sc_log(card->ctx,  "No cert found,i=%d", i);
 			continue;
 		}
-		cert_info.path.count = cert_der.len;
+		cert_info.path.count = (int)cert_der.len;
 
-		sc_log(card->ctx, 
+		sc_log(card->ctx,
 			 "cert len=%"SC_FORMAT_LEN_SIZE_T"u, cert_info.path.count=%d r=%d\n",
 			 cert_der.len, cert_info.path.count, r);
 		sc_log_hex(card->ctx, "cert", cert_der.value, cert_der.len);
@@ -327,7 +327,7 @@ static int sc_pkcs15emu_cac_init(sc_pkcs15_card_t *p15card)
 				sc_dump_hex(cert_info.id.value, cert_info.id.len),
 				 usage, pubkey_info.usage, prkey_info.usage);
 		if (cert_out->key->algorithm != SC_ALGORITHM_RSA) {
-			sc_log(card->ctx, "unsupported key.algorithm %d", cert_out->key->algorithm);
+			sc_log(card->ctx, "unsupported key.algorithm %lu", cert_out->key->algorithm);
 			sc_pkcs15_free_certificate(cert_out);
 			free(pubkey_info.direct.spki.value);
 			continue;
