@@ -11,13 +11,13 @@
  *
  * You should have received a copy of the GNU Lesser General Public
  * License along with this library; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 */
 
 /* Initially written by David Mattes <david.mattes@boeing.com> */
 /* Support for multiple key containers by Lukas Wunner <lukas@wunner.de> */
 
-#if HAVE_CONFIG_H
+#ifdef HAVE_CONFIG_H
 #include "config.h"
 #endif
 
@@ -67,8 +67,8 @@ typedef struct cdata_st {
 	char	   *label;
 	int	    authority;
 	const char *path;
-	size_t	    index;
-	size_t	    count;
+	int	    index;
+	int	    count;
 	const char *id;
 	int         obj_flags;
 } cdata;
@@ -166,8 +166,10 @@ static int gemsafe_get_cert_len(sc_card_t *card)
 	u8 *iptr;
 	struct sc_path path;
 	struct sc_file *file;
-	size_t objlen, certlen;
+	size_t objlen;
+	int certlen;
 	unsigned int ind, i=0;
+	int read_len;
 
 	sc_format_path(GEMSAFE_PATH, &path);
 	r = sc_select_file(card, &path, &file);
@@ -176,9 +178,11 @@ static int gemsafe_get_cert_len(sc_card_t *card)
 	sc_file_free(file);
 
 	/* Initial read */
-	r = sc_read_binary(card, 0, ibuf, GEMSAFE_READ_QUANTUM, 0);
-	if (r < 0)
+	read_len = sc_read_binary(card, 0, ibuf, GEMSAFE_READ_QUANTUM, 0);
+	if (read_len <= 2) {
+		sc_log(card->ctx, "Invalid size of object data: %d", read_len);
 		return SC_ERROR_INTERNAL;
+	}
 
 	/* Actual stored object size is encoded in first 2 bytes
 	 * (allocated EF space is much greater!)
@@ -207,7 +211,7 @@ static int gemsafe_get_cert_len(sc_card_t *card)
 	 * the private key.
 	 */
 	ind = 2; /* skip length */
-	while (ibuf[ind] == 0x01 && i < gemsafe_cert_max) {
+	while (ind + 1 < (size_t)read_len && ibuf[ind] == 0x01 && i < gemsafe_cert_max) {
 		if (ibuf[ind+1] == 0xFE) {
 			gemsafe_prkeys[i].ref = ibuf[ind+4];
 			sc_log(card->ctx, "Key container %d is allocated and uses key_ref %d",
@@ -234,15 +238,22 @@ static int gemsafe_get_cert_len(sc_card_t *card)
 	/* Read entire file, then dissect in memory.
 	 * Gemalto ClassicClient seems to do it the same way.
 	 */
-	iptr = ibuf + GEMSAFE_READ_QUANTUM;
+	iptr = ibuf + read_len;
 	while ((size_t)(iptr - ibuf) < objlen) {
-		r = sc_read_binary(card, iptr - ibuf, iptr,
+		r = sc_read_binary(card, (unsigned)(iptr - ibuf), iptr,
 				   MIN(GEMSAFE_READ_QUANTUM, objlen - (iptr - ibuf)), 0);
 		if (r < 0) {
 			sc_log(card->ctx, "Could not read cert object");
 			return SC_ERROR_INTERNAL;
 		}
-		iptr += GEMSAFE_READ_QUANTUM;
+		if (r == 0)
+			break;
+		read_len += r;
+		iptr += r;
+	}
+	if ((size_t)read_len < objlen) {
+		sc_log(card->ctx, "Could not read cert object");
+		return SC_ERROR_INTERNAL;
 	}
 
 	/* Search buffer for certificates, they start with 0x3082. */
@@ -259,9 +270,9 @@ static int gemsafe_get_cert_len(sc_card_t *card)
 			/* DER cert len is encoded this way */
 			if (ind+3 >= sizeof ibuf)
 				return SC_ERROR_INVALID_DATA;
-			certlen = ((((size_t) ibuf[ind+2]) << 8) | ibuf[ind+3]) + 4;
+			certlen = ((((int)ibuf[ind + 2]) << 8) | ibuf[ind + 3]) + 4;
 			sc_log(card->ctx,
-			       "Found certificate of key container %d at offset %d, len %"SC_FORMAT_LEN_SIZE_T"u",
+			       "Found certificate of key container %d at offset %d, len %d",
 			       i+1, ind, certlen);
 			gemsafe_cert[i].index = ind;
 			gemsafe_cert[i].count = certlen;
@@ -410,7 +421,7 @@ static int sc_pkcs15emu_gemsafeV1_init( sc_pkcs15_card_t *p15card)
 		 */
 		if ( p15card->card->flags & 0x0F) {
 			key_ref = p15card->card->flags & 0x0F;
-			sc_log(p15card->card->ctx, 
+			sc_log(p15card->card->ctx,
 				 "Overriding key_ref %d with %d\n",
 				 gemsafe_prkeys[i].ref, key_ref);
 		} else
