@@ -16,10 +16,10 @@
  *
  * You should have received a copy of the GNU Lesser General Public
  * License along with this library; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
  */
 
-#if HAVE_CONFIG_H
+#ifdef HAVE_CONFIG_H
 #include "config.h"
 #endif
 
@@ -55,6 +55,7 @@ typedef struct _pgp_pin_cfg {
 	int		do_index;
 } pgp_pin_cfg_t;
 
+// clang-format off
 /* OpenPGP cards v1:
  * "Signature PIN2 & "Encryption PIN" are two different PINs - not sync'ed by hardware
  */
@@ -71,6 +72,7 @@ static const pgp_pin_cfg_t	pin_cfg_v2[3] = {
 	{ "User PIN (sig)", 0x01, PGP_USER_PIN_FLAGS,  6, 0 },	// used for PSO:CDS
 	{ "Admin PIN",      0x03, PGP_ADMIN_PIN_FLAGS, 8, 2 }
 };
+// clang-format on
 
 static struct sc_object_id curve25519_oid = {{1, 3, 6, 1, 4, 1, 3029, 1, 5, 1, -1}};
 
@@ -103,6 +105,7 @@ typedef struct cdata_st {
 	int         obj_flags;
 } cdata;
 
+// clang-format off
 static const pgp_key_cfg_t key_cfg[3] = {
 	{ "Signature key",      "B601", 1, PGP_SIG_PRKEY_USAGE,  PGP_SIG_PUBKEY_USAGE  },
 	{ "Encryption key",     "B801", 2, PGP_ENC_PRKEY_USAGE,  PGP_ENC_PUBKEY_USAGE  },
@@ -114,12 +117,14 @@ static const cdata certs[PGP_MAX_NUM_CERTS] = {
 	{"DEC certificate", 0, "3F007F21", "2", SC_PKCS15_CO_FLAG_MODIFIABLE},
 	{"SIG certificate", 0, "3F007F21", "1", SC_PKCS15_CO_FLAG_MODIFIABLE}
 };
+// clang-format on
 
 typedef struct _pgp_manuf_map {
 	unsigned short		id;
 	const char	*name;
 } pgp_manuf_map_t;
 
+// clang-format off
 static const pgp_manuf_map_t manuf_map[] = {
 	{ 0x0001, "PPC Card Systems"		},
 	{ 0x0002, "Prism"			},
@@ -147,6 +152,7 @@ static const pgp_manuf_map_t manuf_map[] = {
 	{ 0xffff, "test card"			},
 	{ 0, NULL }
 };
+// clang-format on
 
 /*
  * This function pretty much follows what find_tlv in the GNUpg
@@ -176,6 +182,7 @@ sc_pkcs15emu_openpgp_init(sc_pkcs15_card_t *p15card)
 {
 	sc_card_t	*card = p15card->card;
 	sc_context_t	*ctx = card->ctx;
+	struct pgp_priv_data *priv = DRVDATA(card);
 	char		string[256];
 	u8		c4data[10];
 	u8		c5data[100];
@@ -225,8 +232,7 @@ sc_pkcs15emu_openpgp_init(sc_pkcs15_card_t *p15card)
 	if ((r = read_file(card, "006E:0073:00C4", c4data, sizeof(c4data))) < 0)
 		goto failed;
 	if (r != 7) {
-		sc_log(ctx, 
-			"CHV status bytes have unexpected length (expected 7, got %d)\n", r);
+		sc_log(ctx, "CHV status bytes have unexpected length (expected 7, got %d)\n", r);
 		r = SC_ERROR_OBJECT_NOT_VALID;
 		goto failed;
 	}
@@ -397,6 +403,10 @@ sc_pkcs15emu_openpgp_init(sc_pkcs15_card_t *p15card)
 
 			case SC_OPENPGP_KEYALGO_RSA:
 				if (cxdata_len >= 3) {
+					/* with Authentication key, can only decrypt if can change MSE */
+					if (i == 2 && !(priv->ext_caps & EXT_CAP_MSE)) {
+						prkey_info.usage &= ~PGP_ENC_PRKEY_USAGE;
+					}
 					prkey_info.modulus_length = bebytes2ushort(cxdata + 1);
 					r = sc_pkcs15emu_add_rsa_prkey(p15card, &prkey_obj, &prkey_info);
 					break;
@@ -521,6 +531,10 @@ sc_pkcs15emu_openpgp_init(sc_pkcs15_card_t *p15card)
 				break;
 			case SC_OPENPGP_KEYALGO_RSA:
 				if (cxdata_len >= 3) {
+					/* with Authentication pubkey, can only encrypt if can change MSE */
+					if (i == 2 && !(priv->ext_caps & EXT_CAP_MSE)) {
+						pubkey_info.usage &= ~PGP_ENC_PUBKEY_USAGE;
+					}
 					pubkey_info.modulus_length = bebytes2ushort(cxdata + 1);
 					r = sc_pkcs15emu_add_rsa_pubkey(p15card, &pubkey_obj, &pubkey_info);
 					break;
@@ -556,14 +570,9 @@ sc_pkcs15emu_openpgp_init(sc_pkcs15_card_t *p15card)
 		memset(&cert_info, 0, sizeof(cert_info));
 		memset(&cert_obj,  0, sizeof(cert_obj));
 
-		/* only try to SELECT DATA for OpenPGP >= v3 */
-		if (card->type >= SC_CARD_TYPE_OPENPGP_V3) {
-			r = sc_card_ctl(card, SC_CARDCTL_OPENPGP_SELECT_DATA, &i);
-			if (r < 0) {
-				free(buffer);
-				LOG_TEST_RET(card->ctx, r, "Failed OpenPGP - select data");
-			}
-		}
+		/* try to SELECT DATA. Will only work for OpenPGP >= v3, errors are non-critical */
+		sc_card_ctl(card, SC_CARDCTL_OPENPGP_SELECT_DATA, &i);
+
 		sc_format_path(certs[i].path, &cert_info.path);
 
 		/* Certificate ID. We use the same ID as the authentication key */
@@ -619,6 +628,9 @@ failed:
 	LOG_FUNC_RETURN(ctx, r);
 }
 
+// see https://stackoverflow.com/a/10536254 for explanation
+#define LEN_MAX_INT_AS_STR (3 * sizeof(int) + 2)
+
 static int
 sc_pkcs15emu_openpgp_add_data(sc_pkcs15_card_t *p15card)
 {
@@ -630,8 +642,8 @@ sc_pkcs15emu_openpgp_add_data(sc_pkcs15_card_t *p15card)
 	for (i = 1; i <= PGP_NUM_PRIVDO; i++) {
 		sc_pkcs15_data_info_t dat_info;
 		sc_pkcs15_object_t dat_obj;
-		char name[8];
-		char path[9];
+		char name[6 + LEN_MAX_INT_AS_STR];
+		char path[7 + LEN_MAX_INT_AS_STR];
 		u8 content[254];
 		memset(&dat_info, 0, sizeof(dat_info));
 		memset(&dat_obj, 0, sizeof(dat_obj));
