@@ -18,7 +18,7 @@
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
  */
 
-#if HAVE_CONFIG_H
+#ifdef HAVE_CONFIG_H
 #include "config.h"
 #endif
 
@@ -745,7 +745,7 @@ sc_pkcs15_encode_pubkey(sc_context_t *ctx, struct sc_pkcs15_pubkey *key,
 		key->algorithm == SC_ALGORITHM_XEDDSA) /* XXX encoding is the same here */
 		return sc_pkcs15_encode_pubkey_eddsa(ctx, &key->u.eddsa, buf, len);
 
-	sc_log(ctx, "Encoding of public key type %u not supported", key->algorithm);
+	sc_log(ctx, "Encoding of public key type %lu not supported", key->algorithm);
 	LOG_FUNC_RETURN(ctx, SC_ERROR_NOT_SUPPORTED);
 }
 
@@ -777,7 +777,7 @@ sc_pkcs15_encode_pubkey_as_spki(sc_context_t *ctx, struct sc_pkcs15_pubkey *pubk
 	pkey.value =  NULL;
 	pkey.len = 0;
 
-	sc_log(ctx, "Encoding public key with algorithm %i", pubkey->algorithm);
+	sc_log(ctx, "Encoding public key with algorithm %lu", pubkey->algorithm);
 	if (!pubkey->alg_id)   {
 		pubkey->alg_id = calloc(1, sizeof(struct sc_algorithm_id));
 		if (!pubkey->alg_id)
@@ -808,7 +808,7 @@ sc_pkcs15_encode_pubkey_as_spki(sc_context_t *ctx, struct sc_pkcs15_pubkey *pubk
 			if (!ec_params)
 				LOG_FUNC_RETURN(ctx, SC_ERROR_OUT_OF_MEMORY);
 			ec_params->type = 1;
-			ec_params->der.value = calloc(pubkey->u.ec.params.der.len, 1);
+			ec_params->der.value = calloc(1, pubkey->u.ec.params.der.len);
 			if (!ec_params->der.value) {
 				free(ec_params);
 				LOG_FUNC_RETURN(ctx, SC_ERROR_OUT_OF_MEMORY);
@@ -871,7 +871,7 @@ sc_pkcs15_decode_pubkey(sc_context_t *ctx, struct sc_pkcs15_pubkey *key,
 		key->algorithm == SC_ALGORITHM_XEDDSA)
 		return sc_pkcs15_decode_pubkey_eddsa(ctx, &key->u.eddsa, buf, len);
 
-	sc_log(ctx, "Decoding of public key type %u not supported", key->algorithm);
+	sc_log(ctx, "Decoding of public key type %lu not supported", key->algorithm);
 	return SC_ERROR_NOT_SUPPORTED;
 }
 
@@ -949,7 +949,7 @@ sc_pkcs15_read_pubkey(struct sc_pkcs15_card *p15card, const struct sc_pkcs15_obj
 	else if (p15card->card->ops->read_public_key)   {
 		sc_log(ctx, "Call card specific 'read-public-key' handle");
 		r = p15card->card->ops->read_public_key(p15card->card, algorithm,
-				(struct sc_path *)&info->path, info->key_reference, info->modulus_length,
+				(struct sc_path *)&info->path, info->key_reference, (unsigned)info->modulus_length,
 				&data, &len);
 		LOG_TEST_GOTO_ERR(ctx, r, "Card specific 'read-public' procedure failed.");
 
@@ -1241,6 +1241,7 @@ sc_pkcs15_read_der_file(sc_context_t *ctx, char * filename,
 	const u8 * body = NULL;
 	size_t bodylen;
 	unsigned int cla_out, tag_out;
+	ssize_t sz;
 
 	LOG_FUNC_CALLED(ctx);
 	if (!buf || !buflen)
@@ -1255,13 +1256,13 @@ sc_pkcs15_read_der_file(sc_context_t *ctx, char * filename,
 		goto out;
 	}
 
-	r = read(f, tagbuf, sizeof(tagbuf)); /* get tag and length */
-	if (r < 2) {
+	sz = read(f, tagbuf, sizeof(tagbuf)); /* get tag and length */
+	if (sz < 2) {
 		sc_log(ctx, "Problem with '%s'", filename);
 		r =  SC_ERROR_DATA_OBJECT_NOT_FOUND;
 		goto out;
 	}
-	len = r;
+	len = sz;
 
 	body = tagbuf;
 	r = sc_asn1_read_tag(&body, len, &cla_out, &tag_out, &bodylen);
@@ -1288,8 +1289,8 @@ sc_pkcs15_read_der_file(sc_context_t *ctx, char * filename,
 	memcpy(rbuf, tagbuf, len); /* copy first or only part */
 	if (rbuflen > len) {
 		/* read rest of file */
-		r = read(f, rbuf + len, rbuflen - len);
-		if (r < (int)(rbuflen - len)) {
+		sz = read(f, rbuf + len, rbuflen - len);
+		if (sz < (ssize_t)(rbuflen - len)) {
 			r = SC_ERROR_INVALID_ASN1_OBJECT;
 			free (rbuf);
 			rbuf = NULL;
@@ -1299,7 +1300,7 @@ sc_pkcs15_read_der_file(sc_context_t *ctx, char * filename,
 	*buflen = rbuflen;
 	*buf = rbuf;
 	rbuf = NULL;
-	r = rbuflen;
+	r = (int)rbuflen;
 out:
 	if (f >= 0)
 		close(f);
@@ -1364,8 +1365,10 @@ sc_pkcs15_pubkey_from_spki_fields(struct sc_context *ctx, struct sc_pkcs15_pubke
 	memcpy(pubkey->alg_id, &pk_alg, sizeof(struct sc_algorithm_id));
 	pubkey->algorithm = pk_alg.algorithm;
 	pk_alg.params = NULL;
-	sc_log(ctx, "DEE pk_alg.algorithm=%d", pk_alg.algorithm);
+	sc_log(ctx, "DEE pk_alg.algorithm=%lu", pk_alg.algorithm);
 
+	if (pk.len == 0)
+		LOG_TEST_GOTO_ERR(ctx, SC_ERROR_INTERNAL, "Incorrect length of key");
 	pk.len = (pk.len + 7)/8;	/* convert number of bits to bytes */
 
 	if (pk_alg.algorithm == SC_ALGORITHM_EC)   {
@@ -1608,7 +1611,7 @@ sc_pkcs15_convert_pubkey(struct sc_pkcs15_pubkey *pkcs15_key, void *evp_key)
 			return SC_ERROR_INCOMPATIBLE_KEY;
 		RSA_get0_key(src, &src_n, &src_e, NULL);
 		if (!src_n || !src_e) {
-			free(src);
+			RSA_free(src);
 			return SC_ERROR_INTERNAL;
 		}
 #else
