@@ -15,10 +15,10 @@
  *
  * You should have received a copy of the GNU Lesser General Public
  * License along with this library; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
  */
 
-#if HAVE_CONFIG_H
+#ifdef HAVE_CONFIG_H
 #include "config.h"
 #endif
 
@@ -386,7 +386,7 @@ int sc_pkcs15emu_sc_hsm_decode_cvc(sc_pkcs15_card_t * p15card,
 	struct sc_asn1_entry asn1_cvcert[C_ASN1_CVCERT_SIZE];
 	struct sc_asn1_entry asn1_cvc_body[C_ASN1_CVC_BODY_SIZE];
 	struct sc_asn1_entry asn1_cvc_pubkey[C_ASN1_CVC_PUBKEY_SIZE];
-	unsigned int cla,tag;
+	unsigned int cla = 0, tag = 0;
 	size_t taglen;
 	const u8 *tbuf;
 	int r;
@@ -894,7 +894,7 @@ static int sc_pkcs15emu_sc_hsm_get_ec_public_key(struct sc_context *ctx, sc_cvc_
 		return SC_ERROR_OUT_OF_MEMORY;
 
 	ecp->der.len = oid->len + 2;
-	ecp->der.value = calloc(ecp->der.len, 1);
+	ecp->der.value = calloc(1, ecp->der.len);
 	if (!ecp->der.value) {
 		free(ecp);
 		return SC_ERROR_OUT_OF_MEMORY;
@@ -1044,7 +1044,7 @@ static int sc_pkcs15emu_sc_hsm_add_pubkey(sc_pkcs15_card_t *p15card, u8 *efbin,
 	} else {
 		/* TODO fix if support of non multiple of 8 curves are added */
 		pubkey_info.field_length = cvc.primeOrModuluslen << 3;
-		pubkey_info.usage = SC_PKCS15_PRKEY_USAGE_VERIFY;
+		pubkey_info.usage = SC_PKCS15_PRKEY_USAGE_VERIFY|SC_PKCS15_PRKEY_USAGE_DERIVE;
 		r = sc_pkcs15emu_add_ec_pubkey(p15card, &pubkey_obj, &pubkey_info);
 	}
 	if (r < 0)
@@ -1076,6 +1076,11 @@ static int sc_pkcs15emu_sc_hsm_add_prkd(sc_pkcs15_card_t * p15card, u8 keyid) {
 	size_t len;
 	int r;
 
+	if (keyid == 0) {
+		// Device authentication key does not have PKCS#15 meta data
+		return SC_SUCCESS;
+	}
+
 	fid[0] = PRKD_PREFIX;
 	fid[1] = keyid;
 
@@ -1106,6 +1111,10 @@ static int sc_pkcs15emu_sc_hsm_add_prkd(sc_pkcs15_card_t * p15card, u8 keyid) {
 	if (prkd.type == SC_PKCS15_TYPE_PRKEY_RSA) {
 		r = sc_pkcs15emu_add_rsa_prkey(p15card, &prkd, key_info);
 	} else {
+		if (key_info->field_length == 528) {
+			// Fix a bug for secp521 key generated with OpenSCDP
+			key_info->field_length = 521;
+		}
 		r = sc_pkcs15emu_add_ec_prkey(p15card, &prkd, key_info);
 	}
 
@@ -1374,7 +1383,7 @@ static int sc_pkcs15emu_sc_hsm_init (sc_pkcs15_card_t * p15card)
 	len -= 5;
 
 	free(p15card->tokeninfo->serial_number);
-	p15card->tokeninfo->serial_number = calloc(len + 1, 1);
+	p15card->tokeninfo->serial_number = calloc(1, len + 1);
 	if (p15card->tokeninfo->serial_number == NULL) {
 		sc_pkcs15_card_clear(p15card);
 		LOG_FUNC_RETURN(card->ctx, SC_ERROR_OUT_OF_MEMORY);
@@ -1394,6 +1403,7 @@ static int sc_pkcs15emu_sc_hsm_init (sc_pkcs15_card_t * p15card)
 	pin_info.auth_id.value[0] = 1;
 	pin_info.path.aid = sc_hsm_aid;
 	pin_info.auth_type = SC_PKCS15_PIN_AUTH_TYPE_PIN;
+	pin_info.auth_method = SC_AC_CHV;
 	pin_info.attrs.pin.reference = 0x81;
 	pin_info.attrs.pin.flags = SC_PKCS15_PIN_FLAG_LOCAL|SC_PKCS15_PIN_FLAG_INITIALIZED|SC_PKCS15_PIN_FLAG_EXCHANGE_REF_DATA;
 	pin_info.attrs.pin.type = SC_PKCS15_PIN_TYPE_ASCII_NUMERIC;
@@ -1409,10 +1419,21 @@ static int sc_pkcs15emu_sc_hsm_init (sc_pkcs15_card_t * p15card)
 	strlcpy(pin_obj.label, "UserPIN", sizeof(pin_obj.label));
 	pin_obj.flags = SC_PKCS15_CO_FLAG_PRIVATE|SC_PKCS15_CO_FLAG_MODIFIABLE;
 
-	r = sc_pkcs15emu_add_pin_obj(p15card, &pin_obj, &pin_info);
-	if (r < 0) {
-		sc_pkcs15_card_clear(p15card);
-		LOG_FUNC_RETURN(card->ctx, r);
+	pin_obj.data = &pin_info;
+
+	r = sc_pkcs15_get_pin_info(p15card, &pin_obj);
+
+	if (r != SC_ERROR_DATA_OBJECT_NOT_FOUND) {
+		if (r < 0) {
+			sc_pkcs15_card_clear(p15card);
+			LOG_FUNC_RETURN(card->ctx, r);
+		}
+
+		r = sc_pkcs15emu_add_pin_obj(p15card, &pin_obj, &pin_info);
+		if (r < 0) {
+			sc_pkcs15_card_clear(p15card);
+			LOG_FUNC_RETURN(card->ctx, r);
+		}
 	}
 
 	memset(&pin_info, 0, sizeof(pin_info));
@@ -1422,8 +1443,9 @@ static int sc_pkcs15emu_sc_hsm_init (sc_pkcs15_card_t * p15card)
 	pin_info.auth_id.value[0] = 2;
 	pin_info.path.aid = sc_hsm_aid;
 	pin_info.auth_type = SC_PKCS15_PIN_AUTH_TYPE_PIN;
+	pin_info.auth_method = SC_AC_CHV;
 	pin_info.attrs.pin.reference = 0x88;
-	pin_info.attrs.pin.flags = SC_PKCS15_PIN_FLAG_LOCAL|SC_PKCS15_PIN_FLAG_INITIALIZED|SC_PKCS15_PIN_FLAG_UNBLOCK_DISABLED|SC_PKCS15_PIN_FLAG_SO_PIN;
+	pin_info.attrs.pin.flags = SC_PKCS15_PIN_FLAG_LOCAL|SC_PKCS15_PIN_FLAG_UNBLOCK_DISABLED|SC_PKCS15_PIN_FLAG_SO_PIN;
 	pin_info.attrs.pin.type = SC_PKCS15_PIN_TYPE_BCD;
 	pin_info.attrs.pin.min_length = 16;
 	pin_info.attrs.pin.stored_length = 0;
@@ -1435,6 +1457,21 @@ static int sc_pkcs15emu_sc_hsm_init (sc_pkcs15_card_t * p15card)
 	strlcpy(pin_obj.label, "SOPIN", sizeof(pin_obj.label));
 	pin_obj.flags = SC_PKCS15_CO_FLAG_PRIVATE;
 
+	pin_obj.data = &pin_info;
+
+	r = sc_pkcs15_get_pin_info(p15card, &pin_obj);
+
+	if (r != SC_ERROR_DATA_OBJECT_NOT_FOUND) {
+		pin_info.attrs.pin.flags |= SC_PKCS15_PIN_FLAG_INITIALIZED;
+	} else {
+		r = SC_SUCCESS;
+	}
+
+	if (r < 0) {
+		sc_pkcs15_card_clear(p15card);
+		LOG_FUNC_RETURN(card->ctx, r);
+	}
+
 	r = sc_pkcs15emu_add_pin_obj(p15card, &pin_obj, &pin_info);
 	if (r < 0) {
 		sc_pkcs15_card_clear(p15card);
