@@ -15,7 +15,7 @@
  *
  * You should have received a copy of the GNU Lesser General Public
  * License along with this library; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
  */
 
 #include "config.h"
@@ -204,7 +204,7 @@ struct sc_pkcs11_framework_ops framework_pkcs15init = {
 	NULL,	/* login */
 	NULL,	/* logout */
 	NULL,	/* change_pin */
-	NULL,	/* inti_token */
+	NULL,	/* init_token */
 	NULL,	/* init_pin */
 	NULL,	/* create_object */
 	NULL,	/* gen_keypair */
