@@ -15,7 +15,7 @@
  *
  * You should have received a copy of the GNU Lesser General Public
  * License along with this library; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
  */
 
 #include "config.h"
@@ -23,6 +23,8 @@
 #include <stdlib.h>
 #include <string.h>
 
+#include "common/compat_overflow.h"
+#include "common/constant-time.h"
 #include "sc-pkcs11.h"
 
 /* Also used for verification data */
@@ -39,7 +41,7 @@ struct operation_data {
 	struct hash_signature_info *info;
 	sc_pkcs11_operation_t *md;
 	CK_BYTE			*buffer;
-	unsigned int	buffer_len;
+	CK_ULONG		buffer_len;
 };
 
 static struct operation_data *
@@ -60,14 +62,16 @@ operation_data_release(struct operation_data *data)
 
 static CK_RV
 signature_data_buffer_append(struct operation_data *data,
-		const CK_BYTE *in, unsigned int in_len)
+		const CK_BYTE *in, CK_ULONG in_len)
 {
 	if (!data)
 		return CKR_ARGUMENTS_BAD;
 	if (in_len == 0)
 		return CKR_OK;
 
-	unsigned int new_len = data->buffer_len + in_len;
+	CK_ULONG new_len;
+	if (__builtin_uaddl_overflow(data->buffer_len, in_len, &new_len))
+		return CKR_ARGUMENTS_BAD;
 	CK_BYTE *new_buffer = sc_mem_secure_alloc(new_len);
 	if (!new_buffer)
 		return CKR_HOST_MEMORY;
@@ -97,12 +101,12 @@ static CK_RV
 sc_pkcs11_copy_mechanism(sc_pkcs11_mechanism_type_t *mt,
 				sc_pkcs11_mechanism_type_t **new_mt)
 {
-	int rv;
+	CK_RV rv;
 
 	*new_mt = calloc(1, sizeof(sc_pkcs11_mechanism_type_t));
 	if (!(*new_mt))
 		return CKR_HOST_MEMORY;
-	
+
 	memcpy(*new_mt, mt, sizeof(sc_pkcs11_mechanism_type_t));
 	/* mech_data needs specific function for making copy*/
 	if (mt->copy_mech_data
@@ -110,7 +114,7 @@ sc_pkcs11_copy_mechanism(sc_pkcs11_mechanism_type_t *mt,
 		free(*new_mt);
 		return rv;
 	}
-	
+
 	return CKR_OK;
 }
 
@@ -127,7 +131,8 @@ sc_pkcs11_register_mechanism(struct sc_pkcs11_card *p11card,
 	sc_pkcs11_mechanism_type_t *existing_mt;
 	sc_pkcs11_mechanism_type_t *copy_mt = NULL;
 	sc_pkcs11_mechanism_type_t **p;
-	int i, rv;
+	int i;
+	CK_RV rv;
 
 	if (mt == NULL)
 		return CKR_HOST_MEMORY;
@@ -192,7 +197,7 @@ _validate_key_type(sc_pkcs11_mechanism_type_t *mech, CK_KEY_TYPE key_type) {
  * Look up a mechanism
  */
 sc_pkcs11_mechanism_type_t *
-sc_pkcs11_find_mechanism(struct sc_pkcs11_card *p11card, CK_MECHANISM_TYPE mech, unsigned int flags)
+sc_pkcs11_find_mechanism(struct sc_pkcs11_card *p11card, CK_MECHANISM_TYPE mech, CK_FLAGS flags)
 {
 	sc_pkcs11_mechanism_type_t *mt;
 	unsigned int n;
@@ -859,7 +864,6 @@ sc_pkcs11_verify_final(sc_pkcs11_operation_t *operation,
 
 	if (key_type != CKK_GOSTR3410)
 		attr.type = CKA_SPKI;
-		
 
 	rv = key->ops->get_attribute(operation->session, key, &attr);
 	if (rv != CKR_OK)
@@ -880,10 +884,10 @@ sc_pkcs11_verify_final(sc_pkcs11_operation_t *operation,
 			goto done;
 	}
 
-	rv = sc_pkcs11_verify_data(pubkey_value, (unsigned int) attr.ulValueLen,
+	rv = sc_pkcs11_verify_data(pubkey_value, attr.ulValueLen,
 		params, sizeof(params),
 		&operation->mechanism, data->md,
-		data->buffer, data->buffer_len, pSignature, (unsigned int) ulSignatureLen);
+		data->buffer, data->buffer_len, pSignature, ulSignatureLen);
 
 done:
 	free(pubkey_value);
@@ -1062,10 +1066,6 @@ sc_pkcs11_decr_init(struct sc_pkcs11_session *session,
 	/* Validate the mechanism parameters */
 	if (key->ops->init_params) {
 		rv = key->ops->init_params(operation->session, &operation->mechanism);
-		if (rv != CKR_OK) {
-			/* Probably bad arguments */
-			LOG_FUNC_RETURN(context, (int) rv);
-		}
 	}
 
 	if (rv != CKR_OK)
@@ -1089,7 +1089,9 @@ sc_pkcs11_decr(struct sc_pkcs11_session *session,
 	rv = op->type->decrypt(op, pEncryptedData, ulEncryptedDataLen,
 	                       pData, pulDataLen);
 
-	if (rv != CKR_BUFFER_TOO_SMALL && pData != NULL)
+	/* terminate session for any return value except CKR_BUFFER_TOO_SMALL,
+	 * perform check in time side-channel free way to prevent Marvin attack */
+	if (!constant_time_eq_s(rv, CKR_BUFFER_TOO_SMALL) && pData != NULL)
 		session_stop_operation(session, SC_PKCS11_OPERATION_DECRYPT);
 
 	return rv;
@@ -1110,10 +1112,12 @@ sc_pkcs11_decr_update(struct sc_pkcs11_session *session,
 	rv = op->type->decrypt_update(op, pEncryptedData, ulEncryptedDataLen,
 			pData, pulDataLen);
 
-	/* terminate session for any error except CKR_BUFFER_TOO_SMALL */
-	if (rv != CKR_OK && rv != CKR_BUFFER_TOO_SMALL)
+	/* terminate session for any return value except CKR_BUFFER_TOO_SMALL,
+	 * perform check in time side-channel free way to prevent Marvin attack */
+	if (~constant_time_eq_s(rv, CKR_OK) & ~constant_time_eq_s(rv, CKR_BUFFER_TOO_SMALL))
 		session_stop_operation(session, SC_PKCS11_OPERATION_DECRYPT);
-	LOG_FUNC_RETURN(context, (int)rv);
+	/* do not log error code to prevent side channel attack */
+	return rv;
 }
 
 CK_RV
@@ -1406,7 +1410,9 @@ sc_pkcs11_encrypt(sc_pkcs11_operation_t *operation,
 
 	/* EncryptFinalize */
 	rv = key->ops->encrypt(operation->session, key, &operation->mechanism,
-			NULL, 0, pEncryptedData + ulEncryptedDataLen, &ulLastEncryptedPartLen);
+			NULL, 0,
+			pEncryptedData ? pEncryptedData + ulEncryptedDataLen : NULL,
+			&ulLastEncryptedPartLen);
 
 	if (pulEncryptedDataLen)
 		*pulEncryptedDataLen = ulEncryptedDataLen + ulLastEncryptedPartLen;
@@ -1530,6 +1536,10 @@ sc_pkcs11_decrypt(sc_pkcs11_operation_t *operation,
 	if (pulDataLen)
 		*pulDataLen = ulDataLen;
 
+	/* Skip DecryptFinalize for PKCS#1 v1.5 padding to prevent time side-channel leakage */
+	if (((CK_MECHANISM_PTR)&operation->mechanism)->mechanism == CKM_RSA_PKCS)
+		return rv;
+
 	if (rv != CKR_OK)
 		return rv;
 
@@ -1541,7 +1551,8 @@ sc_pkcs11_decrypt(sc_pkcs11_operation_t *operation,
 
 	/* DecryptFinalize */
 	rv = key->ops->decrypt(operation->session, key, &operation->mechanism,
-			NULL, 0, pData + ulDataLen, &ulLastDataLen);
+			NULL, 0, pData ? pData + ulDataLen : NULL,
+			&ulLastDataLen);
 	if (pulDataLen)
 		*pulDataLen = ulDataLen + ulLastDataLen;
 	return rv;
@@ -1753,7 +1764,7 @@ CK_RV copy_hash_signature_info(const void *mech_data, void **new_data)
 	*new_data = calloc(1, sizeof(struct hash_signature_info));
 	if (!(*new_data))
 		return CKR_HOST_MEMORY;
-	
+
 	memcpy(*new_data, mech_data, sizeof(struct hash_signature_info));
 	return CKR_OK;
 }
@@ -1773,33 +1784,37 @@ sc_pkcs11_register_sign_and_hash_mechanism(struct sc_pkcs11_card *p11card,
 	CK_MECHANISM_INFO mech_info;
 	CK_RV rv;
 
+	LOG_FUNC_CALLED(p11card->card->ctx);
+	sc_log(p11card->card->ctx, "mech = %lx, hash_mech = %lx", mech, hash_mech);
+
 	if (!sign_type)
-		return CKR_MECHANISM_INVALID;
+		LOG_FUNC_RETURN(p11card->card->ctx, CKR_MECHANISM_INVALID);
 	mech_info = sign_type->mech_info;
 
 	if (!(hash_type = sc_pkcs11_find_mechanism(p11card, hash_mech, CKF_DIGEST)))
-		return CKR_MECHANISM_INVALID;
+		LOG_FUNC_RETURN(p11card->card->ctx, CKR_MECHANISM_INVALID);
 
 	/* These hash-based mechs can only be used for sign/verify */
 	mech_info.flags &= (CKF_SIGN | CKF_SIGN_RECOVER | CKF_VERIFY | CKF_VERIFY_RECOVER);
 
 	info = calloc(1, sizeof(*info));
 	if (!info)
-		return CKR_HOST_MEMORY;
+		LOG_FUNC_RETURN(p11card->card->ctx, CKR_HOST_MEMORY);
 
 	info->mech = mech;
 	info->hash_type = hash_type;
 	info->sign_mech = sign_type->mech;
 	info->hash_mech = hash_mech;
 
-	new_type = sc_pkcs11_new_fw_mechanism(mech, &mech_info, sign_type->key_types[0], info, free_info, copy_hash_signature_info);
+	new_type = sc_pkcs11_new_fw_mechanism(mech, &mech_info, sign_type->key_types[0],
+			info, free_info, copy_hash_signature_info);
 	if (!new_type) {
 		free_info(info);
-		return CKR_HOST_MEMORY;
+		LOG_FUNC_RETURN(p11card->card->ctx, CKR_HOST_MEMORY);
 	}
 
 	rv = sc_pkcs11_register_mechanism(p11card, new_type, NULL);
 	sc_pkcs11_free_mechanism(&new_type);
 
-	return rv;
+	LOG_FUNC_RETURN(p11card->card->ctx, (int)rv);
 }
