@@ -15,7 +15,7 @@
  *
  * You should have received a copy of the GNU Lesser General Public
  * License along with this library; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
  */
 
 #include "config.h"
@@ -59,6 +59,7 @@ pid_t initialized_pid = (pid_t)-1;
 static int in_finalize = 0;
 extern CK_FUNCTION_LIST pkcs11_function_list;
 extern CK_FUNCTION_LIST_3_0 pkcs11_function_list_3_0;
+int nesting = 0;
 
 #ifdef PKCS11_THREAD_LOCKING
 
@@ -290,27 +291,47 @@ CK_RV C_Initialize(CK_VOID_PTR pInitArgs)
 {
 	CK_RV rv;
 #if !defined(_WIN32)
-	pid_t current_pid = getpid();
+	pid_t current_pid;
 #endif
 	int rc;
 	sc_context_param_t ctx_opts;
 
 #if !defined(_WIN32)
 	/* Handle fork() exception */
+	C_INITIALIZE_M_LOCK
+	current_pid = getpid();
 	if (current_pid != initialized_pid) {
-		if (context)
+		if (context && CKR_OK == sc_pkcs11_lock()) {
 			context->flags |= SC_CTX_FLAG_TERMINATE;
+			sc_pkcs11_unlock();
+		}
 		C_Finalize(NULL_PTR);
 	}
 	initialized_pid = current_pid;
 	in_finalize = 0;
+	C_INITIALIZE_M_UNLOCK
 #endif
 
+	/* protect from nesting */
+	C_INITIALIZE_M_LOCK
+	nesting++;
+	if (nesting > 1) {
+		nesting--;
+		C_INITIALIZE_M_UNLOCK
+		return CKR_GENERAL_ERROR;
+	}
+	C_INITIALIZE_M_UNLOCK
+	/* protect from nesting */
+
 	/* protect from multiple threads tryng to setup locking */
 	C_INITIALIZE_M_LOCK
 
 	if (context != NULL) {
-		sc_log(context, "C_Initialize(): Cryptoki already initialized\n");
+		if (CKR_OK == sc_pkcs11_lock()) {
+			sc_log(context, "C_Initialize(): Cryptoki already initialized\n");
+			sc_pkcs11_unlock();
+		}
+		nesting--;
 		C_INITIALIZE_M_UNLOCK
 		return CKR_CRYPTOKI_ALREADY_INITIALIZED;
 	}
@@ -364,6 +385,7 @@ out:
 	}
 
 	/* protect from multiple threads tryng to setup locking */
+	nesting--;
 	C_INITIALIZE_M_UNLOCK
 
 	return rv;
@@ -644,7 +666,7 @@ CK_RV C_GetSlotInfo(CK_SLOT_ID slotID, CK_SLOT_INFO_PTR pInfo)
 		memcpy(pInfo, &slot->slot_info, sizeof(CK_SLOT_INFO));
 
 	sc_log(context, "C_GetSlotInfo() flags 0x%lX", pInfo->flags);
-	
+
 	name = lookup_enum(RV_T, rv);
 	if (name)
 		sc_log(context, "C_GetSlotInfo(0x%lx) = %s", slotID, name);
@@ -705,13 +727,32 @@ CK_RV C_InitToken(CK_SLOT_ID slotID,
 {
 	struct sc_pkcs11_session *session;
 	struct sc_pkcs11_slot *slot;
+	unsigned char *label, *cpo;
 	CK_RV rv;
 	unsigned int i;
 
-	sc_log(context, "C_InitToken(pLabel='%s') called", pLabel);
+	/* Strip trailing whitespace and null terminate the label.
+	 * Keep the fixed-length buffer though as some other layers or drivers (SC-HSM)
+	 * might expect the length is fixed! */
+	label = malloc(33);
+	if (label == NULL) {
+		sc_log(context, "Failed to allocate label memory");
+		return CKR_HOST_MEMORY;
+	}
+	memcpy(label, pLabel, 32);
+	label[32] = 0;
+	cpo = label + 31;
+	while ((cpo >= label) && (*cpo == ' ')) {
+		*cpo = 0;
+		cpo--;
+	}
+
+	sc_log(context, "C_InitToken(pLabel='%s') called", label);
 	rv = sc_pkcs11_lock();
-	if (rv != CKR_OK)
+	if (rv != CKR_OK) {
+		free(label);
 		return rv;
+	}
 
 	rv = slot_get_token(slotID, &slot);
 	if (rv != CKR_OK)   {
@@ -735,7 +776,7 @@ CK_RV C_InitToken(CK_SLOT_ID slotID,
 		}
 	}
 
-	rv = slot->p11card->framework->init_token(slot, slot->fw_data, pPin, ulPinLen, pLabel);
+	rv = slot->p11card->framework->init_token(slot, slot->fw_data, pPin, ulPinLen, label);
 	if (rv == CKR_OK) {
 		/* Now we should re-bind all tokens so they get the
 		 * corresponding function vector and flags */
@@ -743,7 +784,8 @@ CK_RV C_InitToken(CK_SLOT_ID slotID,
 
 out:
 	sc_pkcs11_unlock();
-	sc_log(context, "C_InitToken(pLabel='%s') returns 0x%lX", pLabel, rv);
+	sc_log(context, "C_InitToken(pLabel='%s') returns 0x%lX", label, rv);
+	free(label);
 	return rv;
 }
 
@@ -821,10 +863,12 @@ out:
  */
 #define NUM_INTERFACES 2
 #define DEFAULT_INTERFACE 0
+// clang-format off
 CK_INTERFACE interfaces[NUM_INTERFACES] = {
 	{"PKCS 11", (void *)&pkcs11_function_list_3_0, 0},
 	{"PKCS 11", (void *)&pkcs11_function_list, 0}
 };
+// clang-format on
 
 CK_RV C_GetInterfaceList(CK_INTERFACE_PTR pInterfacesList,  /* returned interfaces */
 			 CK_ULONG_PTR pulCount)         /* number of interfaces returned */
