@@ -11,7 +11,7 @@
  *
  * You should have received a copy of the GNU Lesser General Public
  * License along with this library; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
  */
 /* Initially written by Weitao Sun (weitao@ftsafe.com) 2008*/
 /* Disable RSA:512bits by Shengchao Niu (shengchao@ftsafe.com) 2012 */
@@ -472,7 +472,7 @@ static int entersafe_generate_key(sc_profile_t *profile, sc_pkcs15_card_t *p15ca
 static int entersafe_sanity_check(sc_profile_t *profile, sc_pkcs15_card_t *p15card)
 {
 	struct sc_context *ctx = p15card->card->ctx;
-	struct sc_pkcs15_auth_info profile_auth;
+	struct sc_pkcs15_auth_info profile_auth = {0};
 	struct sc_pkcs15_object *objs[32];
 	int rv, nn, ii, update_df = 0;
 
