@@ -17,13 +17,17 @@
  *
  * You should have received a copy of the GNU Lesser General Public
  * License along with this library; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
  */
 
 #ifdef HAVE_CONFIG_H
 #include <config.h>
 #endif
 
+#ifndef FIX_UNUSED
+#define FIX_UNUSED(X) (void) (X) /* avoid warnings for unused params */
+#endif
+
 #ifdef ENABLE_OPENSSL   /* empty file without openssl */
 
 #include <stdlib.h>
@@ -33,16 +37,6 @@
 #include <errno.h>
 #include <stdio.h>
 
-#include <openssl/bn.h>
-#include <openssl/evp.h>
-#include <openssl/pem.h>
-#include <openssl/err.h>
-#include <openssl/rand.h>
-#include <openssl/sha.h>
-#include <openssl/rsa.h>
-#include <openssl/x509.h>
-#include <openssl/pkcs12.h>
-
 #include "../libopensc/opensc.h"
 #include "../libopensc/cardctl.h"
 #include "../libopensc/log.h"
@@ -93,7 +87,7 @@ iasecc_pkcs15_delete_file(struct sc_pkcs15_card *p15card, struct sc_profile *pro
 	rv = sc_pkcs15init_authenticate(profile, p15card, df, SC_AC_OP_DELETE);
 	card->caps = caps;
 
-	LOG_TEST_RET(ctx, rv, "Cannnot authenticate SC_AC_OP_DELETE");
+	LOG_TEST_RET(ctx, rv, "Cannot authenticate SC_AC_OP_DELETE");
 
 	memset(&path, 0, sizeof(path));
 	path.type = SC_PATH_TYPE_FILE_ID;
@@ -121,7 +115,7 @@ iasecc_pkcs15_erase_card(struct sc_profile *profile, struct sc_pkcs15_card *p15c
 
 	LOG_FUNC_CALLED(ctx);
 
-	if (p15card->app->ddo.aid.len)   {
+	if (p15card->app && p15card->app->ddo.aid.len)   {
 		memset(&path, 0, sizeof(struct sc_path));
 		path.type = SC_PATH_TYPE_DF_NAME;
 		memcpy(path.value, p15card->app->ddo.aid.value, p15card->app->ddo.aid.len);
@@ -162,6 +156,7 @@ iasecc_pkcs15_erase_card(struct sc_profile *profile, struct sc_pkcs15_card *p15c
 			}
 
 			sc_pkcs15_remove_object(p15card, objs[ii]);
+			sc_pkcs15_free_object(objs[ii]);
 		}
 
 		rv = sc_select_file(p15card->card, &df->path, &file);
@@ -241,7 +236,9 @@ iasecc_pkcs15_new_file(struct sc_profile *profile, struct sc_card *card,
 	file->path.value[file->path.len - 1] = file->id & 0xFF;
 	file->path.count = -1;
 
-	sc_log(ctx, "file size %i; ef type %i/%i; id %04X\n", file->size, file->type, file->ef_structure, file->id);
+	sc_log(ctx,
+	       "file size %"SC_FORMAT_LEN_SIZE_T"u; ef type %i/%i; id %04X\n",
+	       file->size, file->type, file->ef_structure, file->id);
 	sc_log(ctx, "path type %X; path '%s'", file->path.type, sc_print_path(&file->path));
 
 	if (out)
@@ -270,6 +267,7 @@ iasecc_pkcs15_select_key_reference(struct sc_profile *profile, struct sc_pkcs15_
 			sc_print_path(&key_info->path));
 
 	rv = sc_select_file(card, &key_info->path, &file);
+	sc_file_free(file);
 	LOG_TEST_RET(ctx, rv, "Cannot select DF to select key reference in");
 
 	/* 1 <= ObjReference <= 31 */
@@ -300,8 +298,6 @@ iasecc_pkcs15_select_key_reference(struct sc_profile *profile, struct sc_pkcs15_
 	key_info->key_reference = idx | IASECC_OBJECT_REF_LOCAL;
 	sc_log(ctx, "selected key reference %i", key_info->key_reference);
 
-	if (file)
-		sc_file_free(file);
 	LOG_FUNC_RETURN(ctx, SC_SUCCESS);
 }
 
@@ -334,7 +330,8 @@ iasecc_file_convert_acls(struct sc_context *ctx, struct sc_profile *profile, str
 		if (acl)   {
 			switch (acl->method)   {
 			case SC_AC_IDA:
-				LOG_TEST_RET(ctx, SC_ERROR_NOT_SUPPORTED, "'IDA' not actually supported");
+				sc_log(ctx, "'IDA' not actually supported");
+				return SC_ERROR_NOT_SUPPORTED;
 			case SC_AC_SCB:
 				if ((acl->key_ref & IASECC_SCB_METHOD_MASK) == IASECC_SCB_METHOD_USER_AUTH)   {
 					acl->method = SC_AC_SEN;
@@ -372,11 +369,11 @@ iasecc_sdo_set_key_acls_from_profile(struct sc_profile *profile, struct sc_card
 
 	/* Get ACLs from profile template */
 	rv = sc_profile_get_file(profile, template, &file);
-	LOG_TEST_RET(ctx, rv, "IasEcc: cannot instanciate private key file");
+	LOG_TEST_RET(ctx, rv, "IasEcc: cannot instantiate private key file");
 
 	/* Convert PKCS15 ACLs to SE ACLs */
 	rv = iasecc_file_convert_acls(ctx, profile, file);
-	if (rv < 0 && file)
+	if (rv < 0)
 		sc_file_free(file);
 	LOG_TEST_RET(ctx, rv, "Cannot convert profile ACLs");
 
@@ -401,8 +398,7 @@ iasecc_sdo_set_key_acls_from_profile(struct sc_profile *profile, struct sc_card
 		}
 		else if (acl->method == SC_AC_SEN || acl->method == SC_AC_PRO || acl->method == SC_AC_AUT)   {
 			if ((acl->key_ref & 0xF) == 0 || (acl->key_ref & 0xF) == 0xF)   {
-				if (file)
-					sc_file_free(file);
+				sc_file_free(file);
 				LOG_TEST_RET(ctx, SC_ERROR_INVALID_DATA, "Invalid SE reference");
 			}
 
@@ -416,14 +412,12 @@ iasecc_sdo_set_key_acls_from_profile(struct sc_profile *profile, struct sc_card
 				scb[cntr++] = acl->key_ref | IASECC_SCB_METHOD_EXT_AUTH;
 		}
 		else   {
-			if (file)
-				sc_file_free(file);
+			sc_file_free(file);
 			LOG_TEST_RET(ctx, SC_ERROR_INVALID_DATA, "Unknown SCB method");
 		}
 	}
 
-	if (file)
-		sc_file_free(file);
+	sc_file_free(file);
 
 	/* Copy ACLs into the DOCP*/
 	sdo->docp.acls_contact.tag = IASECC_DOCP_TAG_ACLS_CONTACT;
@@ -434,8 +428,8 @@ iasecc_sdo_set_key_acls_from_profile(struct sc_profile *profile, struct sc_card
 	*(sdo->docp.acls_contact.value + 0) = amb;
 	memcpy(sdo->docp.acls_contact.value + 1, scb, cntr);
 
-	sc_log(ctx, "AMB: %X, CNTR %i, %x %x %x %x %x %x",
-			amb, cntr, scb[0], scb[1], scb[2], scb[3], scb[4], scb[5], scb[6]);
+	sc_log(ctx, "AMB: %X, CNTR %i, %x %x %x %x %x %x %x",
+	       amb, cntr, scb[0], scb[1], scb[2], scb[3], scb[4], scb[5], scb[6]);
 	LOG_FUNC_RETURN(ctx, SC_SUCCESS);
 }
 
@@ -467,25 +461,33 @@ iasecc_sdo_allocate_prvkey(struct sc_profile *profile, struct sc_card *card,
 		sdo->not_on_card = 1;
 
 		rv = iasecc_sdo_set_key_acls_from_profile(profile, card, "private-key", sdo);
+		if (rv != SC_SUCCESS)
+			iasecc_sdo_free(card, sdo);
 		LOG_TEST_RET(ctx, rv, "IasEcc: cannot set ACLs for SDO from the 'private-key'");
 
 		/* FIXME: set here sdo->docp.name and sdo->docp.idata */
 
 		sdo->docp.non_repudiation.value = calloc(1, 1);
-		if (!sdo->docp.non_repudiation.value)
+		if (!sdo->docp.non_repudiation.value)   {
+			iasecc_sdo_free(card, sdo);
 			LOG_FUNC_RETURN(ctx, SC_ERROR_OUT_OF_MEMORY);
-		sdo->docp.non_repudiation.tag = IASECC_DOCP_TAG_NON_REPUDATION;
+		}
+		sdo->docp.non_repudiation.tag = IASECC_DOCP_TAG_NON_REPUDIATION;
 		sdo->docp.non_repudiation.size = 1;
 
 		sdo->data.prv_key.compulsory.value = calloc(1, 1);
-		if (!sdo->data.prv_key.compulsory.value)
+		if (!sdo->data.prv_key.compulsory.value)   {
+			iasecc_sdo_free(card, sdo);
 			LOG_FUNC_RETURN(ctx, SC_ERROR_OUT_OF_MEMORY);
+		}
 		sdo->data.prv_key.compulsory.tag = IASECC_SDO_PRVKEY_TAG_COMPULSORY;
 		sdo->data.prv_key.compulsory.size = 1;
 
 		sdo->docp.size.value = calloc(1, 2);
-		if (!sdo->docp.size.value)
+		if (!sdo->docp.size.value)   {
+			iasecc_sdo_free(card, sdo);
 			LOG_FUNC_RETURN(ctx, SC_ERROR_OUT_OF_MEMORY);
+		}
 		sdo->docp.size.tag = IASECC_DOCP_TAG_SIZE;
 		sdo->docp.size.size = 2;
 		*(sdo->docp.size.value + 0) = (sz >> 8) & 0xFF;
@@ -497,7 +499,7 @@ iasecc_sdo_allocate_prvkey(struct sc_profile *profile, struct sc_card *card,
 		If present, this attribute has to be the same in the 'GENERATE KEY' template data.
 */
 		if (!(key_info->access_flags & SC_PKCS15_PRKEY_ACCESS_LOCAL) && (key_info->usage & SC_PKCS15_PRKEY_USAGE_NONREPUDIATION))
-			sc_log(ctx, "Non fatal error: NON_REPUDATION can be used only for the localy generated keys");
+			sc_log(ctx, "Non fatal error: NON_REPUDIATION can be used only for the locally generated keys");
 
 		if ((key_info->access_flags & SC_PKCS15_PRKEY_ACCESS_LOCAL)
 				&& (key_info->usage & SC_PKCS15_PRKEY_USAGE_SIGN)
@@ -509,14 +511,15 @@ iasecc_sdo_allocate_prvkey(struct sc_profile *profile, struct sc_card *card,
 		sc_log(ctx, "non_repudiation %i", *(sdo->docp.non_repudiation.value + 0));
 		sc_log(ctx, "compulsory 0x%X", *(sdo->data.prv_key.compulsory.value + 0));
 	}
-	else   {
+	else if (rv < 0) {
+		iasecc_sdo_free(card, sdo);
 		LOG_TEST_RET(ctx, rv, "IasEcc: error while getting private key SDO data");
 	}
 
 	if (out)
 		*out = sdo;
 	else
-		free(sdo);
+		iasecc_sdo_free(card, sdo);
 
 	LOG_FUNC_RETURN(ctx, SC_SUCCESS);
 }
@@ -546,11 +549,15 @@ iasecc_sdo_allocate_pubkey(struct sc_profile *profile, struct sc_card *card, str
 		sdo->not_on_card = 1;
 
 		rv = iasecc_sdo_set_key_acls_from_profile(profile, card, "public-key", sdo);
+		if (rv != SC_SUCCESS)
+			iasecc_sdo_free(card, sdo);
 		LOG_TEST_RET(ctx, rv, "iasecc_sdo_allocate_pubkey() cannot set ACLs for SDO from the 'public-key'");
 
 		sdo->docp.size.value = calloc(1, 2);
-		if (!sdo->docp.size.value)
+		if (!sdo->docp.size.value)   {
+			iasecc_sdo_free(card, sdo);
 			LOG_FUNC_RETURN(ctx, SC_ERROR_OUT_OF_MEMORY);
+		}
 		sdo->docp.size.size = 2;
 		sdo->docp.size.tag = IASECC_DOCP_TAG_SIZE;
 		*(sdo->docp.size.value + 0) = (sz >> 8) & 0xFF;
@@ -561,26 +568,31 @@ iasecc_sdo_allocate_pubkey(struct sc_profile *profile, struct sc_card *card, str
 		}
 		else   {
 			sdo->data.pub_key.cha.value = calloc(1, 2);
-			if (!sdo->data.pub_key.cha.value)
+			if (!sdo->data.pub_key.cha.value)   {
+				iasecc_sdo_free(card, sdo);
 				LOG_FUNC_RETURN(ctx, SC_ERROR_OUT_OF_MEMORY);
+			}
 			sdo->data.pub_key.cha.size = 2;
 			sdo->data.pub_key.cha.tag = IASECC_SDO_PUBKEY_TAG_CHA;
 		}
 
 		sdo->data.pub_key.compulsory.value = calloc(1, 1);
-		if (!sdo->data.pub_key.compulsory.value)
+		if (!sdo->data.pub_key.compulsory.value)   {
+			iasecc_sdo_free(card, sdo);
 			LOG_FUNC_RETURN(ctx, SC_ERROR_OUT_OF_MEMORY);
+		}
 		sdo->data.pub_key.compulsory.tag = IASECC_SDO_PUBKEY_TAG_COMPULSORY;
 		sdo->data.pub_key.compulsory.size = 1;
 	}
-	else   {
+	else if (rv < 0) {
+		iasecc_sdo_free(card, sdo);
 		LOG_TEST_RET(ctx, rv, "iasecc_sdo_allocate_pubkey() error while getting public key SDO data");
 	}
 
 	if (out)
 		*out = sdo;
 	else
-		free(sdo);
+		iasecc_sdo_free(card, sdo);
 
 	LOG_FUNC_RETURN(ctx, SC_SUCCESS);
 }
@@ -779,7 +791,7 @@ iasecc_pkcs15_fix_file_access(struct sc_pkcs15_card *p15card, struct sc_file *fi
 }
 
 
-static int
+int
 iasecc_pkcs15_encode_supported_algos(struct sc_pkcs15_card *p15card, struct sc_pkcs15_object *object)
 {
 	struct sc_context *ctx = p15card->card->ctx;
@@ -798,7 +810,8 @@ iasecc_pkcs15_encode_supported_algos(struct sc_pkcs15_card *p15card, struct sc_p
 			LOG_TEST_RET(ctx, rv, "cannot add supported algorithm DECIPHER:CKM_RSA_PKCS");
 		}
 
-		if (prkey_info->usage & SC_PKCS15_PRKEY_USAGE_SIGN)   {
+		if (prkey_info->usage & (SC_PKCS15_PRKEY_USAGE_SIGN |
+		                         SC_PKCS15_PRKEY_USAGE_NONREPUDIATION))   {
 			if (prkey_info->usage & SC_PKCS15_PRKEY_USAGE_NONREPUDIATION)   {
 				algo = sc_pkcs15_get_supported_algo(p15card, SC_PKCS15_ALGO_OP_COMPUTE_SIGNATURE, CKM_SHA1_RSA_PKCS);
 				rv = sc_pkcs15_add_supported_algo_ref(object, algo);
@@ -853,8 +866,7 @@ iasecc_sdo_store_key(struct sc_profile *profile, struct sc_pkcs15_card *p15card,
 	card->caps &= ~SC_CARD_CAP_USE_FCI_AC;
 	rv = sc_pkcs15init_authenticate(profile, p15card, dummy_file, SC_AC_OP_UPDATE);
 	card->caps = caps;
-	if (dummy_file)
-		sc_file_free(dummy_file);
+	sc_file_free(dummy_file);
 
 	LOG_TEST_RET(ctx, rv, "SDO PRIVATE KEY UPDATE authentication failed");
 
@@ -919,8 +931,10 @@ iasecc_pkcs15_fix_private_key_attributes(struct sc_profile *profile, struct sc_p
 
 	sc_log(ctx, "SDO(class:%X,ref:%X,usage:%X)",
 			sdo_prvkey->sdo_class, sdo_prvkey->sdo_ref, sdo_prvkey->usage);
-	sc_log(ctx, "SDO ACLs(%i):%s", sdo_prvkey->docp.acls_contact.size,
-			sc_dump_hex(sdo_prvkey->docp.acls_contact.value, sdo_prvkey->docp.acls_contact.size));
+	sc_log(ctx, "SDO ACLs(%"SC_FORMAT_LEN_SIZE_T"u):%s",
+	       sdo_prvkey->docp.acls_contact.size,
+	       sc_dump_hex(sdo_prvkey->docp.acls_contact.value,
+			   sdo_prvkey->docp.acls_contact.size));
 	sc_log(ctx, "SDO AMB:%X, SCBS:%s", sdo_prvkey->docp.amb,
 			sc_dump_hex(sdo_prvkey->docp.scbs, IASECC_MAX_SCBS));
 
@@ -1037,12 +1051,9 @@ iasecc_pkcs15_create_key_slot(struct sc_profile *profile, struct sc_pkcs15_card
 	LOG_TEST_GOTO_ERR(ctx, rv, "create key slot: cannot create public key: ctl failed");
 
 err:
-	if (file_p_prvkey)
-		sc_file_free(file_p_prvkey);
-	if (file_p_pubkey)
-		sc_file_free(file_p_pubkey);
-	if (parent)
-		sc_file_free(parent);
+	sc_file_free(file_p_prvkey);
+	sc_file_free(file_p_pubkey);
+	sc_file_free(parent);
 
 	LOG_FUNC_RETURN(ctx, rv);
 }
@@ -1060,10 +1071,13 @@ iasecc_pkcs15_create_key(struct sc_profile *profile, struct sc_pkcs15_card *p15c
 	int	 rv;
 
 	LOG_FUNC_CALLED(ctx);
-	sc_log(ctx, "create private key(keybits:%i,usage:%X,access:%X,ref:%X)",
-			keybits, key_info->usage, key_info->access_flags, key_info->key_reference);
+	sc_log(ctx,
+	       "create private key(keybits:%"SC_FORMAT_LEN_SIZE_T"u,usage:%X,access:%X,ref:%X)",
+	       keybits, key_info->usage, key_info->access_flags,
+	       key_info->key_reference);
 	if (keybits < 1024 || keybits > 2048 || (keybits % 256))   {
-		sc_log(ctx, "Unsupported key size %u", keybits);
+		sc_log(ctx, "Unsupported key size %"SC_FORMAT_LEN_SIZE_T"u",
+		       keybits);
 		return SC_ERROR_INVALID_ARGUMENTS;
 	}
 
@@ -1074,7 +1088,7 @@ iasecc_pkcs15_create_key(struct sc_profile *profile, struct sc_pkcs15_card *p15c
 	sc_log(ctx, "iasecc_pkcs15_create_key() sdo_pubkey->not_on_card %i", sdo_pubkey->not_on_card);
 
 	rv = iasecc_sdo_allocate_prvkey(profile, card, key_info, &sdo_prvkey);
-	LOG_TEST_RET(ctx, rv, "IasEcc: init SDO private key failed");
+	LOG_TEST_GOTO_ERR(ctx, rv, "IasEcc: init SDO private key failed");
 	sc_log(ctx, "iasecc_pkcs15_create_key() sdo_prvkey->not_on_card %i", sdo_prvkey->not_on_card);
 
 	if (!sdo_prvkey->not_on_card && !sdo_pubkey->not_on_card)   {
@@ -1082,18 +1096,20 @@ iasecc_pkcs15_create_key(struct sc_profile *profile, struct sc_pkcs15_card *p15c
 	}
 	else   {
 		rv = iasecc_pkcs15_create_key_slot(profile, p15card, sdo_prvkey, sdo_pubkey, key_info);
-		LOG_TEST_RET(ctx, rv, "Cannot create key slot");
+		LOG_TEST_GOTO_ERR(ctx, rv, "Cannot create key slot");
 	}
 
 	rv = sc_pkcs15_allocate_object_content(ctx, object, (unsigned char *)sdo_prvkey, sizeof(struct iasecc_sdo));
-	LOG_TEST_RET(ctx, rv, "Failed to allocate PrvKey SDO as object content");
+	LOG_TEST_GOTO_ERR(ctx, rv, "Failed to allocate PrvKey SDO as object content");
 
 	rv = iasecc_pkcs15_fix_private_key_attributes(profile, p15card, object, (struct iasecc_sdo *)object->content.value);
-	LOG_TEST_RET(ctx, rv, "Failed to fix private key PKCS#15 attributes");
+	LOG_TEST_GOTO_ERR(ctx, rv, "Failed to fix private key PKCS#15 attributes");
 
 	key_info->path.len = 0;
 
+err:
 	iasecc_sdo_free(card, sdo_pubkey);
+	iasecc_sdo_free(card, sdo_prvkey);
 
 	LOG_FUNC_RETURN(ctx, rv);
 }
@@ -1119,8 +1135,10 @@ iasecc_pkcs15_generate_key(struct sc_profile *profile, sc_pkcs15_card_t *p15card
 	int rv;
 
 	LOG_FUNC_CALLED(ctx);
-	sc_log(ctx, "generate key(bits:%i,path:%s,AuthID:%s\n", keybits,
-			sc_print_path(&key_info->path), sc_pkcs15_print_id(&object->auth_id));
+	sc_log(ctx,
+	       "generate key(bits:%"SC_FORMAT_LEN_SIZE_T"u,path:%s,AuthID:%s\n",
+	       keybits, sc_print_path(&key_info->path),
+	       sc_pkcs15_print_id(&object->auth_id));
 
 	if (!object->content.value || object->content.len != sizeof(struct iasecc_sdo))
 		LOG_TEST_RET(ctx, SC_ERROR_INVALID_DATA, "Invalid PrKey SDO data");
@@ -1130,19 +1148,19 @@ iasecc_pkcs15_generate_key(struct sc_profile *profile, sc_pkcs15_card_t *p15card
 		LOG_TEST_RET(ctx, SC_ERROR_INVALID_DATA, "'Magic' control failed for SDO PrvKey");
 
 	if (keybits < 1024 || keybits > 2048 || (keybits%0x100))   {
-		sc_log(ctx, "Unsupported key size %u\n", keybits);
+		sc_log(ctx, "Unsupported key size %"SC_FORMAT_LEN_SIZE_T"u\n",
+		       keybits);
 		return SC_ERROR_INVALID_ARGUMENTS;
 	}
 
-	/* TODO: Check if native IAS middleware accepts the meaningfull path value. */
+	/* TODO: Check if native IAS middleware accepts the meaningful path value. */
 	rv = sc_profile_get_parent(profile, "private-key", &file);
 	LOG_TEST_RET(ctx, rv, "IasEcc: cannot get private key parent file");
 
 	rv = sc_select_file(card, &file->path, NULL);
 	LOG_TEST_RET(ctx, rv, "DF for private objects not defined");
 
-	if (file)
-		sc_file_free(file);
+	sc_file_free(file);
 
 	rv = iasecc_sdo_convert_to_file(card, sdo_prvkey, &file);
 	LOG_TEST_RET(ctx, rv, "Cannot convert SDO PRIVKEY to file");
@@ -1213,8 +1231,10 @@ iasecc_pkcs15_store_key(struct sc_profile *profile, struct sc_pkcs15_card *p15ca
 	int rv;
 
 	LOG_FUNC_CALLED(ctx);
-	sc_log(ctx, "Store IAS/ECC key(keybits:%i,AuthID:%s,path:%s)",
-			keybits, sc_pkcs15_print_id(&object->auth_id), sc_print_path(&key_info->path));
+	sc_log(ctx,
+	       "Store IAS/ECC key(keybits:%"SC_FORMAT_LEN_SIZE_T"u,AuthID:%s,path:%s)",
+	       keybits, sc_pkcs15_print_id(&object->auth_id),
+	       sc_print_path(&key_info->path));
 
 	if (!object->content.value || object->content.len != sizeof(struct iasecc_sdo))
 		LOG_TEST_RET(ctx, SC_ERROR_INVALID_DATA, "Invalid PrKey SDO data");
@@ -1225,8 +1245,10 @@ iasecc_pkcs15_store_key(struct sc_profile *profile, struct sc_pkcs15_card *p15ca
 	if (sdo_prvkey->magic != SC_CARDCTL_IASECC_SDO_MAGIC)
 		LOG_TEST_RET(ctx, SC_ERROR_INVALID_DATA, "'Magic' control failed for SDO PrvKey");
 
-	sc_log(ctx, "key compulsory attr(size:%i,on_card:%i)",
-			sdo_prvkey->data.prv_key.compulsory.size, sdo_prvkey->data.prv_key.compulsory.on_card);
+	sc_log(ctx,
+	       "key compulsory attr(size:%"SC_FORMAT_LEN_SIZE_T"u,on_card:%i)",
+	       sdo_prvkey->data.prv_key.compulsory.size,
+	       sdo_prvkey->data.prv_key.compulsory.on_card);
 
 	rv = sc_profile_get_parent(profile, "private-key", &file);
 	LOG_TEST_RET(ctx, rv, "cannot instantiate parent DF of the private key");
@@ -1234,8 +1256,7 @@ iasecc_pkcs15_store_key(struct sc_profile *profile, struct sc_pkcs15_card *p15ca
 	rv = sc_select_file(card, &file->path, NULL);
 	LOG_TEST_RET(ctx, rv, "failed to select parent DF");
 
-	if (file)
-		sc_file_free(file);
+	sc_file_free(file);
 
 	key_info->access_flags &= ~SC_PKCS15_PRKEY_ACCESS_LOCAL;
 
@@ -1303,8 +1324,7 @@ iasecc_pkcs15_delete_sdo (struct sc_profile *profile, struct sc_pkcs15_card *p15
 	rv = sc_pkcs15init_authenticate(profile, p15card, dummy_file, SC_AC_OP_UPDATE);
 	card->caps = save_card_caps;
 
-	if (dummy_file)
-		sc_file_free(dummy_file);
+	sc_file_free(dummy_file);
 
 	if (rv < 0)   {
 		iasecc_sdo_free(card, sdo);
@@ -1464,7 +1484,7 @@ iasecc_md_gemalto_unset_default(struct sc_pkcs15_card *p15card, struct sc_profil
 	struct sc_pkcs15_prkey_info *key_info = (struct sc_pkcs15_prkey_info *)key_obj->data;
 	unsigned char guid[40];
 	size_t guid_len;
-	int rv, ii, keys_num;
+	int rv, ii, keys_num, private_obj;
 
 	LOG_FUNC_CALLED(ctx);
 
@@ -1478,11 +1498,15 @@ iasecc_md_gemalto_unset_default(struct sc_pkcs15_card *p15card, struct sc_profil
 	if (rv == SC_ERROR_OBJECT_NOT_FOUND)
 		LOG_FUNC_RETURN(ctx, SC_SUCCESS);
 
-	rv = sc_pkcs15_read_data_object(p15card, (struct sc_pkcs15_data_info *)data_obj->data, &dod);
+	private_obj = data_obj->flags & SC_PKCS15_CO_FLAG_PRIVATE;
+	rv = sc_pkcs15_read_data_object(p15card, (struct sc_pkcs15_data_info *)data_obj->data, private_obj, &dod);
 	LOG_TEST_RET(ctx, rv, "Cannot read from 'CSP/'Default Key Container'");
 
-	if (guid_len != dod->data_len || memcmp(guid, dod->data, guid_len))
+	if (guid_len != dod->data_len || memcmp(guid, dod->data, guid_len)) {
+		sc_pkcs15_free_data_object(dod);
 		LOG_FUNC_RETURN(ctx, SC_SUCCESS);
+	}
+	sc_pkcs15_free_data_object(dod);
 
 	rv = sc_pkcs15_get_objects(p15card, SC_PKCS15_TYPE_PRKEY, key_objs, 32);
 	LOG_TEST_RET(ctx, rv, "Get private key PKCS#15 objects error");
@@ -1656,7 +1680,8 @@ iasecc_store_pubkey(struct sc_pkcs15_card *p15card, struct sc_profile *profile,
 	pubkey_info->usage |= prkey_info->usage & SC_PKCS15_PRKEY_USAGE_DECRYPT ? SC_PKCS15_PRKEY_USAGE_ENCRYPT : 0;
 	pubkey_info->usage |= prkey_info->usage & SC_PKCS15_PRKEY_USAGE_UNWRAP ? SC_PKCS15_PRKEY_USAGE_WRAP : 0;
 
-	iasecc_pkcs15_add_access_rule(object, SC_PKCS15_ACCESS_RULE_MODE_READ, NULL);
+	rv = iasecc_pkcs15_add_access_rule(object, SC_PKCS15_ACCESS_RULE_MODE_READ, NULL);
+	LOG_TEST_RET(ctx, rv, "Too many access rules");
 
 	memcpy(&pubkey_info->algo_refs[0], &prkey_info->algo_refs[0], sizeof(pubkey_info->algo_refs));
 
@@ -1692,8 +1717,7 @@ iasecc_store_cert(struct sc_pkcs15_card *p15card, struct sc_profile *profile,
 	rv = iasecc_pkcs15_fix_file_access(p15card, pfile, object);
 	LOG_TEST_RET(ctx, rv, "encode file access rules failed");
 
-	if (pfile)
-		sc_file_free(pfile);
+	sc_file_free(pfile);
 
 	/* NOT_IMPLEMENTED error code indicates to the upper call to execute the default 'store data' procedure */
 	LOG_FUNC_RETURN(ctx, SC_ERROR_NOT_IMPLEMENTED);
@@ -1733,10 +1757,11 @@ iasecc_store_data_object(struct sc_pkcs15_card *p15card, struct sc_profile *prof
 		if (ii == nn_objs)
 			break;
 		sc_file_free(file);
+		file = NULL;
 	}
 
 	if (indx == MAX_DATA_OBJS)
-		LOG_TEST_RET(ctx, SC_ERROR_TOO_MANY_OBJECTS, "iasecc_store_data_object() too many DATA objects.");
+		LOG_TEST_GOTO_ERR(ctx, SC_ERROR_TOO_MANY_OBJECTS, "iasecc_store_data_object() too many DATA objects.");
 
 	do  {
 		const struct sc_acl_entry *acl;
@@ -1764,50 +1789,46 @@ iasecc_store_data_object(struct sc_pkcs15_card *p15card, struct sc_profile *prof
 	} while(0);
 
 	rv = iasecc_file_convert_acls(ctx, profile, file);
-	LOG_TEST_RET(ctx, rv, "iasecc_store_data_object() cannot convert profile ACLs");
+	LOG_TEST_GOTO_ERR(ctx, rv, "iasecc_store_data_object() cannot convert profile ACLs");
 
 	rv = sc_profile_get_parent(profile, "public-data", &parent);
-	LOG_TEST_RET(ctx, rv, "iasecc_store_data_object() cannot get object parent");
+	LOG_TEST_GOTO_ERR(ctx, rv, "iasecc_store_data_object() cannot get object parent");
 	sc_log(ctx, "iasecc_store_data_object() parent path '%s'\n", sc_print_path(&parent->path));
 
 	rv = sc_select_file(card, &parent->path, NULL);
-	LOG_TEST_RET(ctx, rv, "iasecc_store_data_object() cannot select parent");
+	LOG_TEST_GOTO_ERR(ctx, rv, "iasecc_store_data_object() cannot select parent");
 
 	rv = sc_select_file(card, &file->path, &cfile);
 	if (!rv)   {
 		rv = sc_pkcs15init_authenticate(profile, p15card, cfile, SC_AC_OP_DELETE);
-		LOG_TEST_RET(ctx, rv, "iasecc_store_data_object() DELETE authentication failed");
+		LOG_TEST_GOTO_ERR(ctx, rv, "iasecc_store_data_object() DELETE authentication failed");
 
 		rv = iasecc_pkcs15_delete_file(p15card, profile, cfile);
-		LOG_TEST_RET(ctx, rv, "s_pkcs15init_store_data_object() delete pkcs15 file error");
+		LOG_TEST_GOTO_ERR(ctx, rv, "s_pkcs15init_store_data_object() delete pkcs15 file error");
 	}
 	else if (rv != SC_ERROR_FILE_NOT_FOUND)   {
-		LOG_TEST_RET(ctx, rv, "iasecc_store_data_object() select file error");
+		LOG_TEST_GOTO_ERR(ctx, rv, "iasecc_store_data_object() select file error");
 	}
 
 	rv = sc_pkcs15init_authenticate(profile, p15card, parent, SC_AC_OP_CREATE);
-	LOG_TEST_RET(ctx, rv, "iasecc_store_data_object() parent CREATE authentication failed");
+	LOG_TEST_GOTO_ERR(ctx, rv, "iasecc_store_data_object() parent CREATE authentication failed");
 
 	file->size = data->len;
 	rv = sc_create_file(card, file);
-	LOG_TEST_RET(ctx, rv, "iasecc_store_data_object() cannot create DATA file");
+	LOG_TEST_GOTO_ERR(ctx, rv, "iasecc_store_data_object() cannot create DATA file");
 
 	rv = sc_pkcs15init_authenticate(profile, p15card, file, SC_AC_OP_UPDATE);
-	LOG_TEST_RET(ctx, rv, "iasecc_store_data_object() data file UPDATE authentication failed");
+	LOG_TEST_GOTO_ERR(ctx, rv, "iasecc_store_data_object() data file UPDATE authentication failed");
 
 	rv = sc_update_binary(card, 0, data->value, data->len, 0);
-	LOG_TEST_RET(ctx, rv, "iasecc_store_data_object() update DATA file failed");
+	LOG_TEST_GOTO_ERR(ctx, rv, "iasecc_store_data_object() update DATA file failed");
 
 	if (path)
 		*path = file->path;
-
-	if (parent)
-		sc_file_free(parent);
-
+err:
+	sc_file_free(parent);
 	sc_file_free(file);
-
-	if (cfile)
-		sc_file_free(cfile);
+	sc_file_free(cfile);
 
 	LOG_FUNC_RETURN(ctx, rv);
 #undef MAX_DATA_OBJS
@@ -1878,4 +1899,19 @@ sc_pkcs15init_get_iasecc_ops(void)
 	return &sc_pkcs15init_iasecc_operations;
 }
 
+#else /* ENABLE_OPENSSL */
+#include "../libopensc/log.h"
+#include "pkcs15-init.h"
+
+int
+iasecc_pkcs15_encode_supported_algos(struct sc_pkcs15_card *p15card, struct sc_pkcs15_object *object)
+{
+	struct sc_context *ctx = p15card->card->ctx;
+	FIX_UNUSED(object);
+
+	LOG_FUNC_CALLED(ctx);
+	sc_log(ctx, "OpenSC was built without OpenSSL support: skipping");
+	LOG_FUNC_RETURN(ctx, SC_ERROR_NOT_IMPLEMENTED);
+}
+
 #endif /* ENABLE_OPENSSL */
