@@ -15,7 +15,7 @@
  *
  * You should have received a copy of the GNU Lesser General Public
  * License along with this library; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
  *
  * Random notes
  *  -	the "key" command should go away, it's obsolete
@@ -54,6 +54,8 @@
 
 #define TEMPLATE_FILEID_MIN_DIFF	0x20
 
+#define WORD_SIZE	64
+
 /*
 #define DEBUG_PROFILE
 */
@@ -259,8 +261,9 @@ static struct auth_info *	new_key(struct sc_profile *,
 				unsigned int, unsigned int);
 static void		set_pin_defaults(struct sc_profile *,
 				struct pin_info *);
-static void		new_macro(sc_profile_t *, const char *, scconf_list *);
+static int		new_macro(sc_profile_t *, const char *, scconf_list *);
 static sc_macro_t *	find_macro(sc_profile_t *, const char *);
+static int		is_macro_character(char c);
 
 static sc_file_t *
 init_file(unsigned int type)
@@ -644,8 +647,7 @@ sc_profile_add_file(sc_profile_t *profile, const char *name, sc_file_t *file)
 	LOG_FUNC_CALLED(ctx);
 	if (!path.len)   {
 		parent = profile->df_info;
-		        }
-        else   {
+	} else {
 		path.len -= 2;
 		parent = sc_profile_find_file_by_path(profile, &path);
 	}
@@ -1469,7 +1471,7 @@ do_aid(struct state *cur, int argc, char **argv)
 {
 	struct sc_file	*file = NULL;
 	const char	*name = argv[0];
-	unsigned int	len;
+	size_t len;
 	int		res = 0;
 
 	if (!cur->file) {
@@ -1499,7 +1501,7 @@ do_exclusive_aid(struct state *cur, int argc, char **argv)
 {
 	struct sc_file	*file = NULL;
 	const char	*name = argv[0];
-	unsigned int	len;
+	size_t len;
 	int		res = 0;
 
 	if (!cur->file) {
@@ -1575,7 +1577,10 @@ do_acl(struct state *cur, int argc, char **argv)
 	while (argc--) {
 		unsigned int	op, method, id;
 
+		if (strlen(*argv) >= sizeof(oper))
+			goto bad;
 		strlcpy(oper, *argv++, sizeof(oper));
+
 		if ((what = strchr(oper, '=')) == NULL)
 			goto bad;
 		*what++ = '\0';
@@ -1804,7 +1809,7 @@ do_pin_storedlength(struct state *cur, int argc, char **argv)
 static int
 do_pin_flags(struct state *cur, int argc, char **argv)
 {
-	unsigned int	flags;
+	unsigned int flags = 0;
 	int		i, r;
 
 	if (cur->pin->pin.auth_type != SC_PKCS15_PIN_AUTH_TYPE_PIN)
@@ -1826,35 +1831,55 @@ process_macros(struct state *cur, struct block *info,
 {
 	scconf_item	*item;
 	const char	*name;
+	int		 r;
 
 	for (item = blk->items; item; item = item->next) {
+		char *s = item->key;
 		name = item->key;
 		if (item->type != SCCONF_ITEM_TYPE_VALUE || !name)
 			continue;
+
+		/* make sure the macro name consist only of allowed characters.
+		 * This is not guaranteed by the tokenizer */
+		while (is_macro_character(*s)) {
+			s++;
+		}
+		if (*s != '\0') {
+#ifdef DEBUG_PROFILE
+			printf("Invalid macro name %s\n", name);
+#endif
+			return SC_ERROR_SYNTAX_ERROR;
+		}
 #ifdef DEBUG_PROFILE
 		printf("Defining %s\n", name);
 #endif
-		new_macro(cur->profile, name, item->value.list);
+		r = new_macro(cur->profile, name, item->value.list);
+		if (r != SC_SUCCESS)
+			return r;
 	}
 
-	return 0;
+	return SC_SUCCESS;
 }
 
-static void
+static int
 new_macro(sc_profile_t *profile, const char *name, scconf_list *value)
 {
 	sc_macro_t	*mac;
 
+	if (!profile || !name || !value)
+		return SC_ERROR_INVALID_ARGUMENTS;
+
 	if ((mac = find_macro(profile, name)) == NULL) {
 		mac = calloc(1, sizeof(*mac));
 		if (mac == NULL)
-			return;
+			return SC_ERROR_OUT_OF_MEMORY;
 		mac->name = strdup(name);
 		mac->next = profile->macro_list;
 		profile->macro_list = mac;
 	}
 
 	mac->value = value;
+	return SC_SUCCESS;
 }
 
 static sc_macro_t *
@@ -1976,21 +2001,68 @@ static struct block	root_ops = {
 };
 
 static int
-check_macro_reference_loop(scconf_list *start, scconf_list *current, struct state *cur) {
-	sc_macro_t *mac = NULL;
-	const char *str = NULL;
-
-	if (!start || !current || !cur)
-		return 0;
-
-	str = current->data;
-	if (str[0] != '$')
-		return 0;
-	if (!(mac = find_macro(cur->profile, str + 1)))
-		return 0;
-	if (!strcmp(mac->name, start->data + 1))
+is_macro_character(char c)
+{
+	if (isalnum(c) || c == '-' || c == '_')
 		return 1;
-	return check_macro_reference_loop(start, mac->value, cur);
+	return 0;
+}
+
+static int
+get_inner_word(char *str, char word[WORD_SIZE])
+{
+	char *inner = NULL;
+	size_t len = 0;
+
+	inner = str;
+
+	while (is_macro_character(*inner)) {
+		inner++;
+		len++;
+	}
+	if (len >= WORD_SIZE)
+		return 1;
+	memcpy(word, str, len);
+	word[len] = '\0';
+	return 0;
+}
+
+/*
+ * Checks for a reference loop for macro named start_name in macro definitions.
+ * Function returns 1 if a reference loop is detected, 0 otherwise.
+ */
+static int
+check_macro_reference_loop(const char *start_name, sc_macro_t *macro, sc_profile_t *profile, int depth)
+{
+	scconf_list *value;
+	char *name = NULL;
+	sc_macro_t	*m;
+	char word[WORD_SIZE];
+
+	if (!start_name || !macro || !profile || depth == 16)
+		return 1;
+
+	/* For some reason, the macro value is a list where we need to check for references */
+	for (value = macro->value; value != NULL; value = value->next) {
+		/* Find name in macro value */
+		char *macro_value = value->data;
+		if (!(name = strchr(macro_value, '$')))
+			continue;
+		/* Extract the macro name from the string */
+		if (get_inner_word(name + 1, word))
+			return 1;
+		/* Find whether name corresponds to some other macro */
+		if (!(m = find_macro(profile, word)))
+			continue;
+		/* Check for loop */
+		if (!strcmp(m->name, start_name))
+			return 1;
+		/* Reference loop was found to the original macro name */
+		if (check_macro_reference_loop(start_name, m, profile, depth + 1) == 1) {
+			return 1;
+		}
+	}
+	return 0;
 }
 
 static int
@@ -1999,7 +2071,7 @@ build_argv(struct state *cur, const char *cmdname,
 {
 	unsigned int	argc;
 	const char	*str;
-	sc_macro_t	*mac;
+	sc_macro_t	*macro;
 	int		r;
 
 	for (argc = 0; list; list = list->next) {
@@ -2010,34 +2082,46 @@ build_argv(struct state *cur, const char *cmdname,
 
 		str = list->data;
 		if (str[0] != '$') {
+			/* When str contains macro inside, macro reference loop needs to be checked */
+			char *macro_name = NULL;
+			if ((macro_name = strchr(str, '$'))) {
+				/* Macro does not to start at the first position */
+				char word[WORD_SIZE];
+				get_inner_word(macro_name + 1, word);
+				if ((macro = find_macro(cur->profile, word))
+				    && check_macro_reference_loop(macro->name, macro, cur->profile, 0)) {
+					return SC_ERROR_SYNTAX_ERROR;
+				}
+			}
+
 			argv[argc++] = list->data;
 			continue;
 		}
 
 		/* Expand macro reference */
-		if (!(mac = find_macro(cur->profile, str + 1))) {
+		if (!(macro = find_macro(cur->profile, str + 1))) {
 			parse_error(cur, "%s: unknown macro \"%s\"",
 					cmdname, str);
 			return SC_ERROR_SYNTAX_ERROR;
 		}
 
-		if (list == mac->value) {
+		if (list == macro->value) {
 			return SC_ERROR_SYNTAX_ERROR;
 		}
-		if (check_macro_reference_loop(list, mac->value, cur)) {
+		if (check_macro_reference_loop(macro->name, macro, cur->profile, 0)) {
 			return SC_ERROR_SYNTAX_ERROR;
 		}
 #ifdef DEBUG_PROFILE
 		{
 			scconf_list *list;
 
-			printf("Expanding macro %s:", mac->name);
-			for (list = mac->value; list; list = list->next)
+			printf("Expanding macro %s:", macro->name);
+			for (list = macro->value; list; list = list->next)
 				printf(" %s", list->data);
 			printf("\n");
 		}
 #endif
-		r = build_argv(cur, cmdname, mac->value,
+		r = build_argv(cur, cmdname, macro->value,
 				argv + argc, max - argc);
 		if (r < 0)
 			return r;
@@ -2163,7 +2247,7 @@ sc_profile_find_file(struct sc_profile *pro,
 		const sc_path_t *path, const char *name)
 {
 	struct file_info	*fi;
-	unsigned int		len;
+	size_t				len;
 	const u8			*value;
 
 	value = path ? path->value : (const u8*) "";
@@ -2270,6 +2354,9 @@ get_authid(struct state *cur, const char *value,
 		return get_uint(cur, value, type);
 	}
 
+	if (strlen(value) >= sizeof(temp))
+		return 1;
+
 	n = strcspn(value, "0123456789x");
 	strlcpy(temp, value, (sizeof(temp) > n) ? n + 1 : sizeof(temp));
 
@@ -2285,17 +2372,23 @@ static int
 get_uint(struct state *cur, const char *value, unsigned int *vp)
 {
 	char	*ep;
+	unsigned long tmp;
 
 	if (strstr(value, "0x") == value)
-		*vp = strtoul(value + 2, &ep, 16);
+		tmp = strtoul(value + 2, &ep, 16);
 	else if (strstr(value, "x") == value)
-		*vp = strtoul(value + 1, &ep, 16);
+		tmp = strtoul(value + 1, &ep, 16);
 	else
-		*vp = strtoul(value, &ep, 0);
+		tmp = strtoul(value, &ep, 0);
 	if (*ep != '\0') {
 		parse_error(cur, "invalid integer argument \"%s\"\n", value);
 		return 1;
 	}
+	if (tmp > INT_MAX) {
+		parse_error(cur, "the number \"%s\" is too large\n", value);
+		return 1;
+	}
+	*vp = (int)tmp;
 	return 0;
 }
 
@@ -2366,7 +2459,7 @@ struct num_exp_ctx {
 	jmp_buf		error;
 
 	int		j;
-	char		word[64];
+	char		word[WORD_SIZE];
 
 	char *		unget;
 	char *		str;
@@ -2423,7 +2516,7 @@ __expr_get(struct num_exp_ctx *ctx, int eof_okay)
 	}
 	else if (*s == '$') {
 		expr_put(ctx, *s++);
-		while (isalnum((unsigned char)*s) || *s == '-' || *s == '_')
+		while (is_macro_character(*s))
 			expr_put(ctx, *s++);
 	}
 	else if (strchr("*/+-()|&", *s)) {
@@ -2479,10 +2572,14 @@ expr_term(struct num_exp_ctx *ctx, unsigned int *vp, int opening_brackets)
 	}
 	else if (isdigit((unsigned char)*tok)) {
 		char	*ep;
+		unsigned long tmp;
 
-		*vp = strtoul(tok, &ep, 0);
+		tmp = strtoul(tok, &ep, 0);
 		if (*ep)
 			expr_fail(ctx);
+		if (tmp > UINT_MAX)
+			expr_fail(ctx);
+		*vp = (unsigned int)tmp;
 	}
 	else if (*tok == '$') {
 		sc_macro_t	*mac;
