@@ -17,7 +17,7 @@
  *
  * You should have received a copy of the GNU Lesser General Public
  * License along with this library; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
  */
 
 #ifdef HAVE_CONFIG_H
@@ -59,14 +59,13 @@ sm_oberthur_diversify_keyset(struct sc_context *ctx, struct sm_info *sm_info,
 	unsigned char *tmp;
 	int rv = 0, ii, tmp_len;
 
-	if (gp_keyset->kmc_len == 48)   {
+	if (gp_keyset->kmc_len == 48) {
 		for (ii=0; ii<3; ii++)
 			memcpy(keys[ii], gp_keyset->kmc + 16*ii, 16);
-	}
-	else if (gp_keyset->kmc_len == 16 || gp_keyset->kmc_len == 0)   {
+	} else if (gp_keyset->kmc_len == 16 || gp_keyset->kmc_len == 0) {
 		if (gp_keyset->kmc_len == 16)
 			memcpy(master_key, gp_keyset->kmc, 16);
-		sc_log(ctx, "KMC: %s", sc_dump_hex(master_key, sizeof(master_key)));
+		sc_debug(ctx, SC_LOG_DEBUG_SM, "KMC: %s", sc_dump_hex(master_key, sizeof(master_key)));
 		for (ii=0; ii<3; ii++)   {
 			key_buff[0] = key_buff[8] = 0;
 			key_buff[1] = key_buff[9] = 0;
@@ -77,25 +76,26 @@ sm_oberthur_diversify_keyset(struct sc_context *ctx, struct sm_info *sm_info,
 			key_buff[6] = 0xF0,  key_buff[14] = 0x0F;
 			key_buff[7] = key_buff[15] = ii+1;
 
-			sc_log(ctx, "key_buf:%s", sc_dump_hex(key_buff, 16));
+			sc_debug(ctx, SC_LOG_DEBUG_SM, "key_buf:%s", sc_dump_hex(key_buff, 16));
 
-			rv = sm_encrypt_des_ecb3(master_key, key_buff, sizeof(key_buff), &tmp, &tmp_len);
+			rv = sm_encrypt_des_ecb3(ctx, master_key, key_buff, sizeof(key_buff), &tmp, &tmp_len);
 			LOG_TEST_RET(ctx, rv, "GP init session: cannot derive key");
 
 			memcpy(keys[ii], tmp, sizeof(gp_keyset->enc));
 			free(tmp);
 		}
-	}
-	else   {
+	} else {
 		LOG_TEST_RET(ctx, SC_ERROR_INVALID_DATA, "GP init session: invalid KMC data");
 	}
 
-	if (!rv && ctx)   {
-		sc_log_hex(ctx, "Card challenge", gp_session->card_challenge, sizeof(gp_session->card_challenge));
-		sc_log_hex(ctx, "Host challenge", gp_session->host_challenge, sizeof(gp_session->host_challenge));
-		sc_log_hex(ctx, "ENC", gp_keyset->enc, sizeof(gp_keyset->enc));
-		sc_log_hex(ctx, "MAC", gp_keyset->mac, sizeof(gp_keyset->mac));
-		sc_log_hex(ctx, "KEK", gp_keyset->kek, sizeof(gp_keyset->kek));
+	if (!rv && ctx) {
+		sc_debug_hex(ctx, SC_LOG_DEBUG_SM, "Card challenge", gp_session->card_challenge,
+				sizeof(gp_session->card_challenge));
+		sc_debug_hex(ctx, SC_LOG_DEBUG_SM, "Host challenge", gp_session->host_challenge,
+				sizeof(gp_session->host_challenge));
+		sc_debug_hex(ctx, SC_LOG_DEBUG_SM, "ENC", gp_keyset->enc, sizeof(gp_keyset->enc));
+		sc_debug_hex(ctx, SC_LOG_DEBUG_SM, "MAC", gp_keyset->mac, sizeof(gp_keyset->mac));
+		sc_debug_hex(ctx, SC_LOG_DEBUG_SM, "KEK", gp_keyset->kek, sizeof(gp_keyset->kek));
 	}
 
 	return rv;
@@ -110,7 +110,7 @@ sm_authentic_encode_apdu(struct sc_context *ctx, struct sm_info *sm_info)
 	int rv = SC_ERROR_INVALID_ARGUMENTS;
 
 	LOG_FUNC_CALLED(ctx);
-	sc_log(ctx, "SM encode APDU: offset:");
+	sc_debug(ctx, SC_LOG_DEBUG_SM, "SM encode APDU: offset:");
 
 	rv = sm_gp_securize_apdu(ctx, sm_info, NULL, apdu);
 	LOG_TEST_RET(ctx, rv, "SM encode APDU: securize error");
@@ -130,16 +130,18 @@ sm_authentic_get_apdus(struct sc_context *ctx, struct sm_info *sm_info,
 	if (!sm_info)
 		LOG_FUNC_RETURN(ctx, SC_ERROR_INVALID_ARGUMENTS);
 
-	sc_log(ctx, "SM get APDUs: rdata:%p, init_len:%"SC_FORMAT_LEN_SIZE_T"u",
+	sc_debug(ctx, SC_LOG_DEBUG_SM, "SM get APDUs: rdata:%p, init_len:%"SC_FORMAT_LEN_SIZE_T"u",
 	       rdata, init_len);
-	sc_log(ctx, "SM get APDUs: serial %s", sc_dump_hex(sm_info->serialnr.value, sm_info->serialnr.len));
+	sc_debug(ctx, SC_LOG_DEBUG_SM, "SM get APDUs: serial %s",
+			sc_dump_hex(sm_info->serialnr.value, sm_info->serialnr.len));
 
-	if (init_data)   {
-		rv = sm_gp_external_authentication(ctx, sm_info, init_data, init_len, rdata, sm_oberthur_diversify_keyset);
+	if (init_data) {
+		rv = sm_gp_external_authentication(ctx, sm_info, init_data, init_len, rdata,
+				sm_oberthur_diversify_keyset);
 		LOG_TEST_RET(ctx, rv, "SM get APDUs: cannot authenticate card");
 	}
 
-	switch (sm_info->cmd)  {
+	switch (sm_info->cmd) {
 	case SM_CMD_APDU_TRANSMIT:
 		rv = sm_authentic_encode_apdu(ctx, sm_info);
 		LOG_TEST_RET(ctx, rv, "SM get APDUs: cannot encode APDU");
