@@ -19,7 +19,7 @@
  * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
-#if HAVE_CONFIG_H
+#ifdef HAVE_CONFIG_H
 #include "config.h"
 #endif
 
@@ -509,10 +509,10 @@ static void test_verify(const uint8_t *data, size_t size)
 		memcpy(in_buffer, verify_data, verify_data_size);
 		p11->C_Verify(session, in_buffer, verify_data_size, sig_buffer, sig_len);
 	} else {
-		while (size > 0) {
+		while (verify_data_size > 0) {
 			to_process = verify_data_size < sizeof(in_buffer) ? verify_data_size : sizeof(in_buffer);
 			verify_data_size -= to_process;
-			memcpy(in_buffer, data, to_process);
+			memcpy(in_buffer, verify_data, to_process);
 			verify_data += to_process;
 
 			if (p11->C_VerifyUpdate(session, in_buffer, to_process) != CKR_OK)
