@@ -74,7 +74,7 @@ static int dump_unusedspace(void)
 	}
 	path.count = -1;
 
-	r = sc_pkcs15_read_file(p15card, &path, &buf, &buf_len);
+	r = sc_pkcs15_read_file(p15card, &path, &buf, &buf_len, 0);
 	if (r < 0) {
 		if (r == SC_ERROR_FILE_NOT_FOUND) {
 			printf("\nNo EF(UnusedSpace) file\n");
@@ -123,6 +123,7 @@ int main(int argc, char *argv[])
 	/* Keep card locked to prevent useless calls to sc_logout */
 	if (i) {
 		fprintf(stderr, "failed: %s\n", sc_strerror(i));
+		sc_test_cleanup();
 		return 1;
 	}
 	printf("found.\n");
