@@ -18,7 +18,7 @@
  *
  * You should have received a copy of the GNU Lesser General Public
  * License along with this library; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
  */
 
 #include "config.h"
@@ -44,6 +44,7 @@
 
 #include "libopensc/opensc.h"
 #include "libopensc/cards.h"
+#include "libopensc/log.h"
 #include "util.h"
 
 #if OPENSSL_VERSION_NUMBER >= 0x30000000L
@@ -198,6 +199,8 @@ static int cardos_info(void)
 	} else if (apdu.resp[0] == 0xc9 &&
 			(apdu.resp[1] == 0x02 || apdu.resp[1] == 0x03)) {
 		printf(" (that's CardOS V5.3)\n");
+	} else if (apdu.resp[0] == 0xc9 && apdu.resp[1] == 0x04) {
+		printf(" (that's CardOS V5.4)\n");
 	} else {
 		printf(" (unknown Version)\n");
 	}
@@ -444,6 +447,7 @@ static int cardos_sm4h(const unsigned char *in, size_t inlen, unsigned char
 #if OPENSSL_VERSION_NUMBER >= 0x30000000L
 	if (!legacy_provider) {
 		if (!(legacy_provider = OSSL_PROVIDER_try_load(NULL, "legacy", 1))) {
+			sc_log_openssl(ctx);
 			printf("Failed to load legacy provider, aborting\n");
 			free(mac_input);
 			return 0;
@@ -455,6 +459,7 @@ static int cardos_sm4h(const unsigned char *in, size_t inlen, unsigned char
 	if (!cctx ||
 		!EVP_EncryptInit_ex(cctx, EVP_des_ecb(), NULL, key1, NULL) ||
 		!EVP_CIPHER_CTX_set_padding(cctx, 0)) {
+		sc_log_openssl(ctx);
 		printf("Can not setup context, aborting\n");
 		free(mac_input);
 		EVP_CIPHER_CTX_free(cctx);
@@ -464,6 +469,7 @@ static int cardos_sm4h(const unsigned char *in, size_t inlen, unsigned char
 	/* first block: XOR with IV and encrypt with key A IV is 8 bytes 00 */
 	for (i=0; i < 8; i++) des_in[i] = mac_input[i]^00;
 	if (!EVP_EncryptUpdate(cctx, des_out, &tmplen, des_in, 8)) {
+		sc_log_openssl(ctx);
 		printf("Can not setup context, aborting\n");
 		free(mac_input);
 		EVP_CIPHER_CTX_free(cctx);
@@ -474,6 +480,7 @@ static int cardos_sm4h(const unsigned char *in, size_t inlen, unsigned char
 	for (j=1; j < (mac_input_len / 8); j++) {
 		for (i=0; i < 8; i++) des_in[i] = mac_input[i+j*8]^des_out[i];
 		if (!EVP_EncryptUpdate(cctx, des_out, &tmplen, des_in, 8)) {
+			sc_log_openssl(ctx);
 			printf("Can not encrypt, aborting\n");
 			free(mac_input);
 			EVP_CIPHER_CTX_free(cctx);
@@ -481,6 +488,7 @@ static int cardos_sm4h(const unsigned char *in, size_t inlen, unsigned char
 		}
 	}
 	if (!EVP_EncryptFinal_ex(cctx, des_out + tmplen, &tmplen)) {
+		sc_log_openssl(ctx);
 		printf("Can not encrypt, aborting\n");
 		free(mac_input);
 		EVP_CIPHER_CTX_free(cctx);
@@ -491,6 +499,7 @@ static int cardos_sm4h(const unsigned char *in, size_t inlen, unsigned char
 	/* (a noop if key A and B are the same, e.g. 8 bytes ff */
 	if (!EVP_DecryptInit_ex(cctx, EVP_des_ecb(), NULL, key2, NULL) ||
 		!EVP_CIPHER_CTX_set_padding(cctx, 0)) {
+		sc_log_openssl(ctx);
 		printf("Can not setup context, aborting\n");
 		free(mac_input);
 		EVP_CIPHER_CTX_free(cctx);
@@ -498,12 +507,14 @@ static int cardos_sm4h(const unsigned char *in, size_t inlen, unsigned char
 	}
 	for (i=0; i < 8; i++) des_in[i] = des_out[i];
 	if (!EVP_DecryptUpdate(cctx, des_out, &tmplen, des_in, 8)) {
+		sc_log_openssl(ctx);
 		printf("Can not setup context, aborting\n");
 		free(mac_input);
 		EVP_CIPHER_CTX_free(cctx);
 		return 0;
 	}
 	if (!EVP_EncryptFinal_ex(cctx, des_out + tmplen, &tmplen)) {
+		sc_log_openssl(ctx);
 		printf("Can not encrypt, aborting\n");
 		free(mac_input);
 		EVP_CIPHER_CTX_free(cctx);
@@ -512,6 +523,7 @@ static int cardos_sm4h(const unsigned char *in, size_t inlen, unsigned char
 
 	if (!EVP_EncryptInit_ex(cctx, EVP_des_ecb(), NULL, key1, NULL) ||
 		!EVP_CIPHER_CTX_set_padding(cctx, 0)) {
+		sc_log_openssl(ctx);
 		printf("Can not setup context, aborting\n");
 		free(mac_input);
 		EVP_CIPHER_CTX_free(cctx);
@@ -519,12 +531,14 @@ static int cardos_sm4h(const unsigned char *in, size_t inlen, unsigned char
 	}
 	for (i=0; i < 8; i++) des_in[i] = des_out[i];
 	if (!EVP_EncryptUpdate(cctx, des_out, &tmplen, des_in, 8)) {
+		sc_log_openssl(ctx);
 		printf("Can not encrypt, aborting\n");
 		free(mac_input);
 		EVP_CIPHER_CTX_free(cctx);
 		return 0;
 	}
 	if (!EVP_EncryptFinal_ex(cctx, des_out + tmplen, &tmplen)) {
+		sc_log_openssl(ctx);
 		printf("Can not encrypt, aborting\n");
 		free(mac_input);
 		EVP_CIPHER_CTX_free(cctx);
@@ -566,6 +580,7 @@ static int cardos_sm4h(const unsigned char *in, size_t inlen, unsigned char
 	if (!cctx ||
 		!EVP_EncryptInit_ex(cctx, EVP_des_ede_ecb(), NULL, key, NULL) ||
 		!EVP_CIPHER_CTX_set_padding(cctx, 0)) {
+		sc_log_openssl(ctx);
 		printf("Can not setup context, aborting\n");
 		free(mac_input);
 		free(enc_input);
@@ -578,6 +593,7 @@ static int cardos_sm4h(const unsigned char *in, size_t inlen, unsigned char
 
 	/* encrypt with des2 (triple des, but using keys A-B-A) */
 	if (!EVP_EncryptUpdate(cctx, des_out, &tmplen, des_in, 8)) {
+		sc_log_openssl(ctx);
 		printf("Can not encrypt, aborting\n");
 		free(mac_input);
 		free(enc_input);
@@ -595,6 +611,7 @@ static int cardos_sm4h(const unsigned char *in, size_t inlen, unsigned char
 
 		/* encrypt with des2 (triple des, but using keys A-B-A) */
 		if (!EVP_EncryptUpdate(cctx, des_out, &tmplen, des_in, 8)) {
+			sc_log_openssl(ctx);
 			printf("Can not encrypt, aborting\n");
 			free(mac_input);
 			free(enc_input);
@@ -1182,11 +1199,18 @@ int main(int argc, char *argv[])
 			util_print_usage_and_die(app_name, options, option_help, NULL);
 		}
 	}
+	
+	if (action_count == 0)
+		util_print_usage_and_die(app_name, options, option_help, NULL);
 
 	/* create sc_context_t object */
 	memset(&ctx_param, 0, sizeof(ctx_param));
 	ctx_param.ver      = 0;
 	ctx_param.app_name = app_name;
+	ctx_param.debug    = verbose;
+	if (verbose)
+		ctx_param.debug_file = stderr;
+
 	r = sc_context_create(&ctx, &ctx_param);
 	if (r) {
 		fprintf(stderr, "Failed to establish context: %s\n",
@@ -1202,7 +1226,7 @@ int main(int argc, char *argv[])
 		goto end;
 	}
 
-	err = util_connect_card(ctx, &card, opt_reader, opt_wait, verbose);
+	err = util_connect_card(ctx, &card, opt_reader, opt_wait);
 	if (err)
 		goto end;
 
