@@ -15,7 +15,7 @@
  *
  * You should have received a copy of the GNU Lesser General Public
  * License along with this library; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
  */
 #ifdef HAVE_CONFIG_H
 #include "config.h"
@@ -24,10 +24,9 @@
 #include "fread_to_eof.h"
 #include "npa-tool-cmdline.h"
 #include "sm/sm-eac.h"
-#include "sm/sslutil.h"
 #include "util.h"
-#include <eac/pace.h>
 #include <eac/objects.h>
+#include <eac/pace.h>
 #include <libopensc/card-npa.h>
 #include <libopensc/log.h>
 #include <libopensc/opensc.h>
@@ -370,8 +369,6 @@ main (int argc, char **argv)
 	}
 	if (cmdline.tr_03110v201_flag)
 		tr_version = EAC_TR_VERSION_2_01;
-	if (cmdline.disable_all_checks_flag)
-		eac_default_flags |= EAC_FLAG_DISABLE_CHECK_ALL;
 	if (cmdline.disable_ta_checks_flag)
 		eac_default_flags |= EAC_FLAG_DISABLE_CHECK_TA;
 	if (cmdline.disable_ca_checks_flag)
@@ -381,6 +378,9 @@ main (int argc, char **argv)
 	memset(&ctx_param, 0, sizeof(ctx_param));
 	ctx_param.ver      = 0;
 	ctx_param.app_name = app_name;
+	ctx_param.debug    = cmdline.verbose_given;
+	if (cmdline.verbose_given)
+		ctx_param.debug_file = stderr;
 
 	r = sc_context_create(&ctx, &ctx_param);
 	if (r) {
@@ -392,7 +392,7 @@ main (int argc, char **argv)
 	if (r)
 		goto err;
 
-	r = util_connect_card_ex(ctx, &card, cmdline.reader_arg, 0, 0, cmdline.verbose_given);
+	r = util_connect_card_ex(ctx, &card, cmdline.reader_arg, 0, 0);
 	if (r)
 		goto err;
 
@@ -574,9 +574,9 @@ main (int argc, char **argv)
 				exit(1);
 			}
 
-			certs = calloc(sizeof *certs, cmdline.cv_certificate_given + 1);
-			certs_lens = calloc(sizeof *certs_lens,
-					cmdline.cv_certificate_given + 1);
+			certs = calloc(cmdline.cv_certificate_given + 1, sizeof *certs);
+			certs_lens = calloc(cmdline.cv_certificate_given + 1,
+					 sizeof *certs_lens);
 			if (!certs || !certs_lens) {
 				r = SC_ERROR_OUT_OF_MEMORY;
 				goto err;
@@ -597,15 +597,15 @@ main (int argc, char **argv)
 						|| !cvc_cert->body->certificate_authority_reference
 						|| !cvc_cert->body->chat) {
 					fprintf(stderr, "Could not parse certificate.\n");
-					ssl_error(ctx);
+					sc_log_openssl(ctx);
 					r = SC_ERROR_INVALID_DATA;
 					goto err;
 				}
 				pace_input.chat_length = i2d_CVC_CHAT(cvc_cert->body->chat, &certs_chat);
 				if (0 >= (int) pace_input.chat_length) {
 					fprintf(stderr, "Could not parse CHAT.\n");
+					sc_log_openssl(ctx);
 					r = SC_ERROR_INVALID_DATA;
-					ssl_error(ctx);
 					goto err;
 				}
 				pace_input.chat = certs_chat;
