@@ -15,7 +15,7 @@
  *
  * You should have received a copy of the GNU Lesser General Public
  * License along with this library; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
  */
 
 
@@ -298,7 +298,7 @@ static int cert2der(X509 *in_cert, u8 **value)
 static int create_file_cert(sc_card_t *card)
 {
 	int r;
-	int size = 0;
+	size_t size = 0;
 	sc_path_t path;
 	sc_file_t *file = NULL;
 
@@ -384,7 +384,7 @@ int main(int argc, char *argv[])
 				opt_wait = 1;
 				break;
 			case 'g':
-				if(keylen == 0) keylen = 1536;
+				if(keylen == 0) keylen = 2048;
 				break;
 			case 'o':
 				overwrite = 1;
@@ -428,6 +428,9 @@ int main(int argc, char *argv[])
 	memset(&ctx_param, 0, sizeof(ctx_param));
 	ctx_param.ver      = 0;
 	ctx_param.app_name = argv[0];
+	ctx_param.debug    = verbose;
+	if (verbose)
+		ctx_param.debug_file = stderr;
 
 	r = sc_context_create(&ctx, &ctx_param);
 	if (r)
@@ -446,7 +449,7 @@ int main(int argc, char *argv[])
 		}
 	}
 
-	r = util_connect_card(ctx, &card, opt_reader, opt_wait, 0);
+	r = util_connect_card(ctx, &card, opt_reader, opt_wait);
 	if (r)
 		goto out;
 
