@@ -15,12 +15,12 @@
  *
  * You should have received a copy of the GNU Lesser General Public
  * License along with this library; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
  */
 
 #include <windows.h>
 
-HWND create_invisible_window(LPCTSTR lpszClassName,
+static HWND create_invisible_window(LPCTSTR lpszClassName,
 		LRESULT (CALLBACK* WndProc)(HWND, UINT, WPARAM, LPARAM),
 		HINSTANCE hInstance)
 {
