@@ -11,16 +11,28 @@ if [[ ! -f $P11LIB ]]; then
     exit 77;
 fi
 card_setup
-echo "data to sign (max 100 bytes)" > data
+assert $? "Failed to set up card"
+
+# get informaation about OS
+source /etc/os-release || true
 
 echo "======================================================="
 echo "Test RSA keys"
 echo "======================================================="
 for HASH in "" "SHA1" "SHA224" "SHA256" "SHA384" "SHA512"; do
+    RETOSSL="0"
+
+    if [[ "$ID" == "rhel" || "$ID_LIKE" =~ ".*rhel.*" ]] && [[ "$VERSION" -gt 8 ]] && [[ "$HASH" == "SHA1" ]]; then
+        RETOSSL="1"
+    fi
     for SIGN_KEY in "01" "02"; do
         METHOD="RSA-PKCS"
+        # RSA-PKCS works only on small data - generate small data:
+        head -c 64 </dev/urandom > data
         if [[ ! -z $HASH ]]; then
             METHOD="$HASH-$METHOD"
+            # hash- methods should work on data > 512 bytes
+            head -c 1024 </dev/urandom > data
         fi
         echo
         echo "======================================================="
@@ -33,11 +45,17 @@ for HASH in "" "SHA1" "SHA224" "SHA256" "SHA384" "SHA512"; do
         # OpenSSL verification
         if [[ -z $HASH ]]; then
             openssl rsautl -verify -inkey $SIGN_KEY.pub -in data.sig -pubin
+            # pkeyutl does not work with libressl
+            #openssl pkeyutl -verify -inkey $SIGN_KEY.pub -in data -sigfile data.sig -pubin
         else
             openssl dgst -keyform PEM -verify $SIGN_KEY.pub -${HASH,,*} \
                    -signature data.sig data
         fi
-        assert $? "Failed to Verify signature using OpenSSL"
+        if [[ "$RETOSSL" == "0" ]]; then
+            assert $? "Failed to Verify signature using OpenSSL"
+        elif [[ "$?" == "0" ]]; then
+            assert 1 "Unexpectedly Verified signature using OpenSSL"
+        fi
 
         # pkcs11-tool verification
         $PKCS11_TOOL --id $SIGN_KEY --verify -m $METHOD --module $P11LIB \
@@ -46,6 +64,8 @@ for HASH in "" "SHA1" "SHA224" "SHA256" "SHA384" "SHA512"; do
         rm data.sig
 
         METHOD="$METHOD-PSS"
+        # -PSS methods should work on data > 512 bytes; generate data:
+        head -c 1024 </dev/urandom > data
         if [[ "$HASH" == "SHA512" ]]; then
             continue; # This one is broken
         fi
@@ -79,7 +99,11 @@ for HASH in "" "SHA1" "SHA224" "SHA256" "SHA384" "SHA512"; do
         openssl dgst -keyform PEM -verify $SIGN_KEY.pub $VERIFY_DGEST \
                -sigopt rsa_padding_mode:pss  $VERIFY_OPTS -sigopt rsa_pss_saltlen:-1 \
                -signature data.sig data
-        assert $? "Failed to Verify signature using openssl"
+        if [[ "$RETOSSL" == "0" ]]; then
+            assert $? "Failed to Verify signature using openssl"
+        elif [[ "$?" == "0" ]]; then
+            assert 1 "Unexpectedly Verified signature using OpenSSL"
+        fi
 
         # pkcs11-tool verification
         $PKCS11_TOOL --id $SIGN_KEY --verify -m $METHOD --module $P11LIB \
@@ -89,11 +113,55 @@ for HASH in "" "SHA1" "SHA224" "SHA256" "SHA384" "SHA512"; do
         rm data.{sig,hash}
     done
 
+    METHOD="RSA-PKCS-OAEP"
+    # RSA-PKCS-OAEP works only on small data (input length <= k-2-2hLen)
+    # generate small data:
+    head -c 64 </dev/urandom > data
+    for ENC_KEY in "01" "02"; do
+        # SoftHSM only supports SHA1 for both hashAlg and mgf
+        if [[ -z $HASH ]]; then
+        	continue
+        elif [[ "$HASH" != "SHA1" ]]; then
+        	continue
+        fi
+        echo
+        echo "======================================================="
+        echo "$METHOD: Encrypt & Decrypt (KEY $ENC_KEY)"
+        echo "======================================================="
+        # OpenSSL Encryption
+        # pkeyutl does not work with libressl
+        openssl rsautl -encrypt -oaep -inkey $ENC_KEY.pub -in data -pubin -out data.crypt
+        assert $? "Failed to encrypt data using OpenSSL"
+        $PKCS11_TOOL --id $ENC_KEY --decrypt -p $PIN --module $P11LIB \
+               -m $METHOD --hash-algorithm "SHA-1" --mgf "MGF1-SHA1" \
+               --input-file data.crypt --output-file data.decrypted
+        assert $? "Failed to decrypt data using pkcs11-tool"
+        diff data{,.decrypted}
+        assert $? "The decrypted data do not match the original"
+        rm data.{crypt,decrypted}
+
+        $PKCS11_TOOL --id $ENC_KEY --encrypt -p $PIN --module $P11LIB \
+               -m $METHOD --hash-algorithm "SHA-1" --mgf "MGF1-SHA1" \
+               --input-file data --output-file data.crypt
+        assert $? "Failed to encrypt data using pkcs11-tool"
+        # It would be better to decrypt with OpenSSL but we can't read the
+        # private key with the pkcs11-tool (yet)
+        $PKCS11_TOOL --id $ENC_KEY --decrypt -p $PIN --module $P11LIB \
+               -m $METHOD --hash-algorithm "SHA-1" --mgf "MGF1-SHA1" \
+               --input-file data.crypt --output-file data.decrypted
+        assert $? "Failed to decrypt data using pkcs11-tool"
+        diff data{,.decrypted}
+        assert $? "The decrypted data do not match the original"
+        rm data.{crypt,decrypted}
+    done
+
     # Skip hashed algorithms (do not support encryption & decryption)
     if [[ ! -z "$HASH" ]]; then
         continue;
     fi
     METHOD="RSA-PKCS"
+    # RSA-PKCS works only on small data - generate small data:
+    head -c 64 </dev/urandom > data
     for ENC_KEY in "01" "02"; do
         echo
         echo "======================================================="
@@ -102,6 +170,9 @@ for HASH in "" "SHA1" "SHA224" "SHA256" "SHA384" "SHA512"; do
         # OpenSSL Encryption
         openssl rsautl -encrypt -inkey $ENC_KEY.pub -in data \
                -pubin -out data.crypt
+        # pkeyutl does not work with libressl
+        #openssl pkeyutl -encrypt -inkey $ENC_KEY.pub -in data \
+        #       -pubin -out data.crypt
         assert $? "Failed to encrypt data using OpenSSL"
         $PKCS11_TOOL --id $ENC_KEY --decrypt -p $PIN -m $METHOD \
                --module $P11LIB --input-file data.crypt > data.decrypted
@@ -116,6 +187,8 @@ done
 echo "======================================================="
 echo "Test ECDSA keys"
 echo "======================================================="
+# operations with ECDSA keys should work on data > 512 bytes; generate data:
+head -c 1024 </dev/urandom > data
 for SIGN_KEY in "03" "04"; do
     METHOD="ECDSA"
 
@@ -156,10 +229,12 @@ for MECHANISM in "SHA-1-HMAC" "SHA256-HMAC" "SHA384-HMAC" "SHA512-HMAC"; do
 	echo "$MECHANISM: Sign & Verify (KEY (First Found))"
 	echo "======================================================="
 
-	$PKCS11_TOOL --login --pin=1234 --sign --mechanism=$MECHANISM \
-		--input-file=data.msg --output-file=data.sig
-	$PKCS11_TOOL --login --pin=1234 --verify --mechanism=$MECHANISM \
-		--input-file=data.msg --signature-file=data.sig
+	$PKCS11_TOOL --login --pin=$PIN --sign --mechanism=$MECHANISM \
+		--input-file=data.msg --output-file=data.sig --module $P11LIB
+	assert $? "Failed to Sign data"
+	$PKCS11_TOOL --login --pin=$PIN --verify --mechanism=$MECHANISM \
+		--input-file=data.msg --signature-file=data.sig --module $P11LIB
+	assert $? "Failed to Verify signature using pkcs11-tool"
 	rm data.sig
 done;
 
